# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from dataclay.communication.grpc.messages.common import common_messages_pb2 as dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2
from dataclay.communication.grpc.messages.dataservice import dataservice_messages_pb2 as dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2


class DataServiceStub(object):
  """Interface exported by the server.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.initBackendID = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/initBackendID',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.InitBackendIDRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.associateExecutionEnvironment = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/associateExecutionEnvironment',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.AssociateExecutionEnvironmentRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.deployMetaClasses = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/deployMetaClasses',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.DeployMetaClassesRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.deployClasses = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/deployClasses',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.DeployClassesRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.enrichClass = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/enrichClass',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.EnrichClassRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.newPersistentInstance = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/newPersistentInstance',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewPersistentInstanceRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewPersistentInstanceResponse.FromString,
        )
    self.storeObjects = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/storeObjects',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.StoreObjectsRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.getCopyOfObject = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/getCopyOfObject',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetCopyOfObjectRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetCopyOfObjectResponse.FromString,
        )
    self.updateObject = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/updateObject',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.UpdateObjectRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.getObjects = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/getObjects',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetObjectsRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetObjectsResponse.FromString,
        )
    self.newMetaData = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/newMetaData',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewMetaDataRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.newVersion = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/newVersion',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewVersionRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewVersionResponse.FromString,
        )
    self.consolidateVersion = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/consolidateVersion',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ConsolidateVersionRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.upsertObjects = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/upsertObjects',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.UpsertObjectsRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.newReplica = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/newReplica',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewReplicaRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewReplicaResponse.FromString,
        )
    self.moveObjects = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/moveObjects',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.MoveObjectsRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.MoveObjectsResponse.FromString,
        )
    self.removeObjects = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/removeObjects',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.RemoveObjectsRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.RemoveObjectsResponse.FromString,
        )
    self.migrateObjectsToBackends = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/migrateObjectsToBackends',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.MigrateObjectsRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.MigrateObjectsResponse.FromString,
        )
    self.getClassIDFromObjectInMemory = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/getClassIDFromObjectInMemory',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetClassIDFromObjectInMemoryRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetClassIDFromObjectInMemoryResponse.FromString,
        )
    self.executeImplementation = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/executeImplementation',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ExecuteImplementationRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ExecuteImplementationResponse.FromString,
        )
    self.makePersistent = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/makePersistent',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.MakePersistentRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.federate = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/federate',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.FederateRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.unfederate = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/unfederate',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.UnfederateRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.exists = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/exists',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ExistsRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ExistsResponse.FromString,
        )
    self.getFederatedObjects = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/getFederatedObjects',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetFederatedObjectsRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetFederatedObjectsResponse.FromString,
        )
    self.getReferencedObjectsIDs = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/getReferencedObjectsIDs',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetReferencedObjectIDsRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetReferencedObjectIDsResponse.FromString,
        )
    self.filterObject = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/filterObject',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.FilterObjectRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.FilterObjectResponse.FromString,
        )
    self.storeToDB = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/storeToDB',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.StoreToDBRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.getFromDB = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/getFromDB',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetFromDBRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetFromDBResponse.FromString,
        )
    self.updateToDB = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/updateToDB',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.UpdateToDBRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.deleteToDB = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/deleteToDB',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.DeleteToDBRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.existsInDB = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/existsInDB',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ExistsInDBRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ExistsInDBResponse.FromString,
        )
    self.cleanExecutionClassDirectory = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/cleanExecutionClassDirectory',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.closeDbHandler = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/closeDbHandler',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.shutDown = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/shutDown',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.disconnectFromOthers = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/disconnectFromOthers',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.registerPendingObjects = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/registerPendingObjects',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.cleanCaches = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/cleanCaches',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.activateTracing = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/activateTracing',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ActivateTracingRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.deactivateTracing = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/deactivateTracing',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.getTraces = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/getTraces',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.GetTracesResponse.FromString,
        )
    self.closeSessionInDS = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/closeSessionInDS',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.CloseSessionInDSRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.updateRefs = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/updateRefs',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.UpdateRefsRequest.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.FromString,
        )
    self.getRetainedReferences = channel.unary_unary(
        '/dataclay.communication.grpc.dataservice.DataService/getRetainedReferences',
        request_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.SerializeToString,
        response_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetRetainedReferencesResponse.FromString,
        )


class DataServiceServicer(object):
  """Interface exported by the server.
  """

  def initBackendID(self, request, context):
    """Deployment
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def associateExecutionEnvironment(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def deployMetaClasses(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def deployClasses(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def enrichClass(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def newPersistentInstance(self, request, context):
    """Execution Environment
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def storeObjects(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getCopyOfObject(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def updateObject(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getObjects(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def newMetaData(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def newVersion(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def consolidateVersion(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def upsertObjects(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def newReplica(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def moveObjects(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def removeObjects(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def migrateObjectsToBackends(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getClassIDFromObjectInMemory(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def executeImplementation(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def makePersistent(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def federate(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def unfederate(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def exists(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getFederatedObjects(self, request, context):
    """Federation
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getReferencedObjectsIDs(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def filterObject(self, request, context):
    """Filtering
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def storeToDB(self, request, context):
    """Storage Location
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getFromDB(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def updateToDB(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def deleteToDB(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def existsInDB(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def cleanExecutionClassDirectory(self, request, context):
    """Others
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def closeDbHandler(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def shutDown(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def disconnectFromOthers(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def registerPendingObjects(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def cleanCaches(self, request, context):
    """Paraver
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def activateTracing(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def deactivateTracing(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getTraces(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def closeSessionInDS(self, request, context):
    """Garbage collection
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def updateRefs(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getRetainedReferences(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_DataServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'initBackendID': grpc.unary_unary_rpc_method_handler(
          servicer.initBackendID,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.InitBackendIDRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'associateExecutionEnvironment': grpc.unary_unary_rpc_method_handler(
          servicer.associateExecutionEnvironment,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.AssociateExecutionEnvironmentRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'deployMetaClasses': grpc.unary_unary_rpc_method_handler(
          servicer.deployMetaClasses,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.DeployMetaClassesRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'deployClasses': grpc.unary_unary_rpc_method_handler(
          servicer.deployClasses,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.DeployClassesRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'enrichClass': grpc.unary_unary_rpc_method_handler(
          servicer.enrichClass,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.EnrichClassRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'newPersistentInstance': grpc.unary_unary_rpc_method_handler(
          servicer.newPersistentInstance,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewPersistentInstanceRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewPersistentInstanceResponse.SerializeToString,
      ),
      'storeObjects': grpc.unary_unary_rpc_method_handler(
          servicer.storeObjects,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.StoreObjectsRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'getCopyOfObject': grpc.unary_unary_rpc_method_handler(
          servicer.getCopyOfObject,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetCopyOfObjectRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetCopyOfObjectResponse.SerializeToString,
      ),
      'updateObject': grpc.unary_unary_rpc_method_handler(
          servicer.updateObject,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.UpdateObjectRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'getObjects': grpc.unary_unary_rpc_method_handler(
          servicer.getObjects,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetObjectsRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetObjectsResponse.SerializeToString,
      ),
      'newMetaData': grpc.unary_unary_rpc_method_handler(
          servicer.newMetaData,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewMetaDataRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'newVersion': grpc.unary_unary_rpc_method_handler(
          servicer.newVersion,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewVersionRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewVersionResponse.SerializeToString,
      ),
      'consolidateVersion': grpc.unary_unary_rpc_method_handler(
          servicer.consolidateVersion,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ConsolidateVersionRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'upsertObjects': grpc.unary_unary_rpc_method_handler(
          servicer.upsertObjects,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.UpsertObjectsRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'newReplica': grpc.unary_unary_rpc_method_handler(
          servicer.newReplica,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewReplicaRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.NewReplicaResponse.SerializeToString,
      ),
      'moveObjects': grpc.unary_unary_rpc_method_handler(
          servicer.moveObjects,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.MoveObjectsRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.MoveObjectsResponse.SerializeToString,
      ),
      'removeObjects': grpc.unary_unary_rpc_method_handler(
          servicer.removeObjects,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.RemoveObjectsRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.RemoveObjectsResponse.SerializeToString,
      ),
      'migrateObjectsToBackends': grpc.unary_unary_rpc_method_handler(
          servicer.migrateObjectsToBackends,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.MigrateObjectsRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.MigrateObjectsResponse.SerializeToString,
      ),
      'getClassIDFromObjectInMemory': grpc.unary_unary_rpc_method_handler(
          servicer.getClassIDFromObjectInMemory,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetClassIDFromObjectInMemoryRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetClassIDFromObjectInMemoryResponse.SerializeToString,
      ),
      'executeImplementation': grpc.unary_unary_rpc_method_handler(
          servicer.executeImplementation,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ExecuteImplementationRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ExecuteImplementationResponse.SerializeToString,
      ),
      'makePersistent': grpc.unary_unary_rpc_method_handler(
          servicer.makePersistent,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.MakePersistentRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'federate': grpc.unary_unary_rpc_method_handler(
          servicer.federate,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.FederateRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'unfederate': grpc.unary_unary_rpc_method_handler(
          servicer.unfederate,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.UnfederateRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'exists': grpc.unary_unary_rpc_method_handler(
          servicer.exists,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ExistsRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ExistsResponse.SerializeToString,
      ),
      'getFederatedObjects': grpc.unary_unary_rpc_method_handler(
          servicer.getFederatedObjects,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetFederatedObjectsRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetFederatedObjectsResponse.SerializeToString,
      ),
      'getReferencedObjectsIDs': grpc.unary_unary_rpc_method_handler(
          servicer.getReferencedObjectsIDs,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetReferencedObjectIDsRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetReferencedObjectIDsResponse.SerializeToString,
      ),
      'filterObject': grpc.unary_unary_rpc_method_handler(
          servicer.filterObject,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.FilterObjectRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.FilterObjectResponse.SerializeToString,
      ),
      'storeToDB': grpc.unary_unary_rpc_method_handler(
          servicer.storeToDB,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.StoreToDBRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'getFromDB': grpc.unary_unary_rpc_method_handler(
          servicer.getFromDB,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetFromDBRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetFromDBResponse.SerializeToString,
      ),
      'updateToDB': grpc.unary_unary_rpc_method_handler(
          servicer.updateToDB,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.UpdateToDBRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'deleteToDB': grpc.unary_unary_rpc_method_handler(
          servicer.deleteToDB,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.DeleteToDBRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'existsInDB': grpc.unary_unary_rpc_method_handler(
          servicer.existsInDB,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ExistsInDBRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ExistsInDBResponse.SerializeToString,
      ),
      'cleanExecutionClassDirectory': grpc.unary_unary_rpc_method_handler(
          servicer.cleanExecutionClassDirectory,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'closeDbHandler': grpc.unary_unary_rpc_method_handler(
          servicer.closeDbHandler,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'shutDown': grpc.unary_unary_rpc_method_handler(
          servicer.shutDown,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'disconnectFromOthers': grpc.unary_unary_rpc_method_handler(
          servicer.disconnectFromOthers,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'registerPendingObjects': grpc.unary_unary_rpc_method_handler(
          servicer.registerPendingObjects,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'cleanCaches': grpc.unary_unary_rpc_method_handler(
          servicer.cleanCaches,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'activateTracing': grpc.unary_unary_rpc_method_handler(
          servicer.activateTracing,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.ActivateTracingRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'deactivateTracing': grpc.unary_unary_rpc_method_handler(
          servicer.deactivateTracing,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'getTraces': grpc.unary_unary_rpc_method_handler(
          servicer.getTraces,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.GetTracesResponse.SerializeToString,
      ),
      'closeSessionInDS': grpc.unary_unary_rpc_method_handler(
          servicer.closeSessionInDS,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.CloseSessionInDSRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'updateRefs': grpc.unary_unary_rpc_method_handler(
          servicer.updateRefs,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.UpdateRefsRequest.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.ExceptionInfo.SerializeToString,
      ),
      'getRetainedReferences': grpc.unary_unary_rpc_method_handler(
          servicer.getRetainedReferences,
          request_deserializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.EmptyMessage.FromString,
          response_serializer=dataclay_dot_communication_dot_grpc_dot_messages_dot_dataservice_dot_dataservice__messages__pb2.GetRetainedReferencesResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'dataclay.communication.grpc.dataservice.DataService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
