
""" Class description goes here. """

"""Generic GRPC paraver interceptor mechanisms.

In general, the source of examples for making this work comes from:

Heavily inspired in:
https://github.com/grpc/grpc/blob/master/examples/python/interceptors/
"""

HEADER_MESSAGEID = "messageid"
HEADER_CLIENTPORT = "clientport"
