
""" Class description goes here. """

from dataclay.util.MgrObject import ManagementObject

    
class SessionProperty(ManagementObject):
    _fields = ["id",
               "propertyID",
               ]

    _internal_fields = []

