
""" Class description goes here. """

from dataclay.serialization.DataClaySerializable import DataClaySerializable

__author__ = 'Alex Barcelo <alex.barcelo@bsc.es>'
__copyright__ = '2015 Barcelona Supercomputing Center (BSC-CNS)'


class DataClayPythonWrapper(DataClaySerializable):
    __slots__ = ()
