
""" Class description goes here. """

"""
Public dataclay functions exported to use (friendly) "from dataclay import ..."
"""
from dataclay.commonruntime.Runtime import getRuntime
from dataclay.commonruntime.Runtime import StorageObject
from dataclay.DataClayObject import DataClayObject
from dataclay.commonruntime.Initializer import *
from dataclay.DataClayObjMethods import dclayMethod
from dataclay.DataClayObjMethods import dclayEmptyMethod

