"""Common Data Structures for the Storage API.

Note that at the moment (2020-01-09) the classes available in this module are
intended to be used **only** for demonstration purposes.

Please do not use those classes for HPC applications nor any kind of production
applications. However, you can use their API and general behaviour as an
authoritative-ish source for semantics and motivation.
"""
