
""" Class description goes here. """

IGNORE_PREFIXES = [
    "es.bsc.dataclay.util.management.classmgr.java",
    "es.bsc.dataclay.util.management.classmgr.features",
]

IGNORE_CLASSES = [
    "es.bsc.dataclay.util.management.classmgr.PrefetchingInformation",
]
