
""" Class description goes here. """

"""Placeholder module for the properties.

Note that the files included in this folder are (typically) hard links to the
Java folder (containing the properties' files).

Maybe git (or simply developers) will break this. Be cautious.
"""

__author__ = 'Alex Barcelo <alex.barcelo@bsc.es>'
__copyright__ = '2015 Barcelona Supercomputing Center (BSC-CNS)'
