# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dataclay/communication/grpc/messages/logicmodule/logicmodule_messages.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from dataclay.communication.grpc.messages.common import common_messages_pb2 as dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='dataclay/communication/grpc/messages/logicmodule/logicmodule_messages.proto',
  package='dataclay.communication.grpc.logicmodule',
  syntax='proto3',
  serialized_options=_b('\n0dataclay.communication.grpc.messages.logicmoduleP\000'),
  serialized_pb=_b('\nKdataclay/communication/grpc/messages/logicmodule/logicmodule_messages.proto\x12\'dataclay.communication.grpc.logicmodule\x1a\x41\x64\x61taclay/communication/grpc/messages/common/common_messages.proto\"2\n\x18NewAccountNoAdminRequest\x12\x16\n\x0eyamlNewAccount\x18\x01 \x01(\t\"\xb4\x01\n\x11NewAccountRequest\x12>\n\x07\x61\x64minID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12G\n\x0f\x61\x64mincredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x16\n\x0eyamlNewAccount\x18\x03 \x01(\t\"\x9d\x01\n\x12NewAccountResponse\x12\x43\n\x0cnewAccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"7\n\x15PublishAddressRequest\x12\x10\n\x08hostname\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\"*\n\x13GetAccountIDRequest\x12\x13\n\x0b\x61\x63\x63ountName\x18\x01 \x01(\t\"\x9f\x01\n\x14GetAccountIDResponse\x12\x43\n\x0cnewAccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x9d\x01\n\x15\x41utoRegisterSLRequest\x12P\n\x11storageLocationID\x18\x01 \x01(\x0b\x32\x35.dataclay.communication.grpc.common.StorageLocationID\x12\x0e\n\x06\x64sName\x18\x02 \x01(\t\x12\x12\n\ndsHostname\x18\x03 \x01(\t\x12\x0e\n\x06\x64sPort\x18\x04 \x01(\x05\"\xe0\x01\n\x15\x41utoRegisterEERequest\x12Z\n\x16\x65xecutionEnvironmentID\x18\x01 \x01(\x0b\x32:.dataclay.communication.grpc.common.ExecutionEnvironmentID\x12\x0e\n\x06\x65\x65Name\x18\x02 \x01(\t\x12\x12\n\neeHostname\x18\x03 \x01(\t\x12\x0e\n\x06\x65\x65Port\x18\x04 \x01(\x05\x12\x37\n\x04lang\x18\x05 \x01(\x0e\x32).dataclay.communication.grpc.common.Langs\"\xae\x01\n\x16\x41utoRegisterEEResponse\x12P\n\x11storageLocationID\x18\x01 \x01(\x0b\x32\x35.dataclay.communication.grpc.common.StorageLocationID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"t\n UnregisterStorageLocationRequest\x12P\n\x11storageLocationID\x18\x01 \x01(\x0b\x32\x35.dataclay.communication.grpc.common.StorageLocationID\"\x83\x01\n%UnregisterExecutionEnvironmentRequest\x12Z\n\x16\x65xecutionEnvironmentID\x18\x01 \x01(\x0b\x32:.dataclay.communication.grpc.common.ExecutionEnvironmentID\"\xaf\x01\n\x19PerformSetAccountsRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x0c\n\x04yaml\x18\x03 \x01(\t\"t\n\x1aPerformSetAccountsResponse\x12\x12\n\nresultYaml\x18\x01 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xb1\x01\n\x1bPerformSetOperationsRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x0c\n\x04yaml\x18\x03 \x01(\t\"v\n\x1cPerformSetOperationsResponse\x12\x12\n\nresultYaml\x18\x01 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xa0\x01\n\x15GetAccountListRequest\x12>\n\x07\x61\x64minID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12G\n\x0f\x61\x64mincredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\"p\n\x16GetAccountListResponse\x12\x12\n\naccountIDs\x18\x01 \x03(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xa6\x03\n\x11NewSessionRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x43\n\x0b\x63ontractIDs\x18\x03 \x03(\x0b\x32..dataclay.communication.grpc.common.ContractID\x12\x41\n\ndataSetIDs\x18\x04 \x03(\x0b\x32-.dataclay.communication.grpc.common.DataSetID\x12\x43\n\x0cstoreDataSet\x18\x05 \x01(\x0b\x32-.dataclay.communication.grpc.common.DataSetID\x12>\n\x0bsessionLang\x18\x06 \x01(\x0e\x32).dataclay.communication.grpc.common.Langs\"m\n\x12NewSessionResponse\x12\x13\n\x0bsessionInfo\x18\x01 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"`\n\x1cGetInfoOfSessionForDSRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\"\xf6\x01\n\x1dGetInfoOfSessionForDSResponse\x12@\n\tdataSetID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.DataSetID\x12\x0c\n\x04\x64\x61te\x18\x02 \x01(\x03\x12\x41\n\ndataSetIDs\x18\x03 \x03(\x0b\x32-.dataclay.communication.grpc.common.DataSetID\x12\x42\n\x07\x65xcInfo\x18\x04 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xb5\x01\n\x13NewNamespaceRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x18\n\x10newNamespaceYaml\x18\x03 \x01(\t\"\xa0\x01\n\x14NewNamespaceResponse\x12\x44\n\x0bnamespaceID\x18\x01 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x9c\x01\n\x14GetNamespacesRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\"o\n\x15GetNamespacesResponse\x12\x12\n\nnamespaces\x18\x01 \x03(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xb5\x01\n\x16RemoveNamespaceRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x15\n\rnamespaceName\x18\x03 \x01(\t\"\xb4\x01\n\x15GetNamespaceIDRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x15\n\rnamespaceName\x18\x03 \x01(\t\"\xa2\x01\n\x16GetNamespaceIDResponse\x12\x44\n\x0bnamespaceID\x18\x01 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xb6\x01\n\x17GetNamespaceLangRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x15\n\rnamespaceName\x18\x03 \x01(\t\"\x9b\x01\n\x18GetNamespaceLangResponse\x12;\n\x08language\x18\x01 \x01(\x0e\x32).dataclay.communication.grpc.common.Langs\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x9d\x01\n\x19GetObjectDataSetIDRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\"\xa2\x01\n\x1aGetObjectDataSetIDResponse\x12@\n\tdataSetID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.DataSetID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xee\x02\n\x16ImportInterfaceRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\x12\x42\n\ncontractID\x18\x04 \x01(\x0b\x32..dataclay.communication.grpc.common.ContractID\x12\x44\n\x0binterfaceID\x18\x05 \x01(\x0b\x32/.dataclay.communication.grpc.common.InterfaceID\"\xa7\x02\n\x15ImportContractRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\x12\x42\n\ncontractID\x18\x04 \x01(\x0b\x32..dataclay.communication.grpc.common.ContractID\"\xae\x01\n\x11NewDataSetRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x13\n\x0b\x64\x61taSetYaml\x18\x03 \x01(\t\"\x9a\x01\n\x12NewDataSetResponse\x12@\n\tdataSetID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.DataSetID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xb1\x01\n\x14RemoveDataSetRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x13\n\x0b\x64\x61taSetName\x18\x03 \x01(\t\"\xb0\x01\n\x13GetDataSetIDRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x13\n\x0b\x64\x61taSetName\x18\x03 \x01(\t\"\x9c\x01\n\x14GetDataSetIDResponse\x12@\n\tdataSetID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.DataSetID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xa0\x01\n\x18GetPublicDataSetsRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\"q\n\x19GetPublicDataSetsResponse\x12\x10\n\x08\x64\x61taSets\x18\x01 \x03(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xa1\x01\n\x19GetAccountDataSetsRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\"r\n\x1aGetAccountDataSetsResponse\x12\x10\n\x08\x64\x61taSets\x18\x01 \x03(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"_\n\x1b\x43heckDataSetIsPublicRequest\x12@\n\tdataSetID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.DataSetID\"t\n\x1c\x43heckDataSetIsPublicResponse\x12\x10\n\x08isPublic\x18\x01 \x01(\x08\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xe5\x02\n\x0fNewClassRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12;\n\x08language\x18\x03 \x01(\x0e\x32).dataclay.communication.grpc.common.Langs\x12\\\n\nnewClasses\x18\x04 \x03(\x0b\x32H.dataclay.communication.grpc.logicmodule.NewClassRequest.NewClassesEntry\x1a\x31\n\x0fNewClassesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xe8\x01\n\x10NewClassResponse\x12]\n\nnewClasses\x18\x01 \x03(\x0b\x32I.dataclay.communication.grpc.logicmodule.NewClassResponse.NewClassesEntry\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\x1a\x31\n\x0fNewClassesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xfc\x02\n\x11NewClassIDRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12;\n\x08language\x18\x03 \x01(\x0e\x32).dataclay.communication.grpc.common.Langs\x12\x11\n\tclassName\x18\x04 \x01(\t\x12^\n\nnewClasses\x18\x05 \x03(\x0b\x32J.dataclay.communication.grpc.logicmodule.NewClassIDRequest.NewClassesEntry\x1a\x31\n\x0fNewClassesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x9a\x01\n\x12NewClassIDResponse\x12@\n\x07\x63lassID\x18\x01 \x01(\x0b\x32/.dataclay.communication.grpc.common.MetaClassID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xf3\x01\n\x12RemoveClassRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\x12\x11\n\tclassName\x18\x04 \x01(\t\"\x9a\x02\n\x16RemoveOperationRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\x12\x11\n\tclassName\x18\x04 \x01(\t\x12!\n\x19operationNameAndSignature\x18\x05 \x01(\t\"\xcc\x02\n\x1bRemoveImplementationRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\x12\x11\n\tclassName\x18\x04 \x01(\t\x12N\n\x10implementationID\x18\x05 \x01(\x0b\x32\x34.dataclay.communication.grpc.common.ImplementationID\"\x99\x02\n\x15GetOperationIDRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\x12\x11\n\tclassName\x18\x04 \x01(\t\x12!\n\x19operationNameAndSignature\x18\x05 \x01(\t\"\xa2\x01\n\x16GetOperationIDResponse\x12\x44\n\x0boperationID\x18\x01 \x01(\x0b\x32/.dataclay.communication.grpc.common.OperationID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x8b\x02\n\x14GetPropertyIDRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\x12\x11\n\tclassName\x18\x04 \x01(\t\x12\x14\n\x0cpropertyName\x18\x05 \x01(\t\"\x9f\x01\n\x15GetPropertyIDResponse\x12\x42\n\npropertyID\x18\x01 \x01(\x0b\x32..dataclay.communication.grpc.common.PropertyID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xf2\x01\n\x11GetClassIDRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\x12\x11\n\tclassName\x18\x04 \x01(\t\"\x9a\x01\n\x12GetClassIDResponse\x12@\n\x07\x63lassID\x18\x01 \x01(\x0b\x32/.dataclay.communication.grpc.common.MetaClassID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xf4\x01\n\x13GetClassInfoRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\x12\x11\n\tclassName\x18\x04 \x01(\t\"q\n\x14GetClassInfoResponse\x12\x15\n\rmetaClassYaml\x18\x01 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xf0\x01\n\"GetInfoOfClassesInNamespaceRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\"\x91\x02\n#GetInfoOfClassesInNamespaceResponse\x12r\n\x0b\x63lassesInfo\x18\x01 \x03(\x0b\x32].dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceResponse.ClassesInfoEntry\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\x1a\x32\n\x10\x43lassesInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xf6\x01\n(GetImportedClassesInfoInNamespaceRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\"\xaa\x02\n)GetImportedClassesInfoInNamespaceResponse\x12\x80\x01\n\x0fimportedClasses\x18\x01 \x03(\x0b\x32g.dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceResponse.ImportedClassesEntry\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\x1a\x36\n\x14ImportedClassesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xfc\x01\n\x1bGetClassIDFromImportRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\x12\x11\n\tclassName\x18\x04 \x01(\t\"\xa4\x01\n\x1cGetClassIDFromImportResponse\x12@\n\x07\x63lassID\x18\x01 \x01(\x0b\x32/.dataclay.communication.grpc.common.MetaClassID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xb3\x01\n\x12NewContractRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x17\n\x0fnewContractYaml\x18\x03 \x01(\t\"\x9d\x01\n\x13NewContractResponse\x12\x42\n\ncontractID\x18\x01 \x01(\x0b\x32..dataclay.communication.grpc.common.ContractID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xeb\x01\n\x1fRegisterToPublicContractRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x42\n\ncontractID\x18\x03 \x01(\x0b\x32..dataclay.communication.grpc.common.ContractID\"\xf8\x01\n*RegisterToPublicContractOfNamespaceRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\"\xb5\x01\n+RegisterToPublicContractOfNamespaceResponse\x12\x42\n\ncontractID\x18\x01 \x01(\x0b\x32..dataclay.communication.grpc.common.ContractID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xaa\x01\n GetContractIDsOfApplicantRequest\x12\x42\n\x0b\x61pplicantID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\"\x87\x02\n!GetContractIDsOfApplicantResponse\x12l\n\tcontracts\x18\x01 \x03(\x0b\x32Y.dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantResponse.ContractsEntry\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\x1a\x30\n\x0e\x43ontractsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xf8\x01\n\x1fGetContractIDsOfProviderRequest\x12\x41\n\nproviderID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12N\n\x15namespaceIDOfProvider\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\"\x85\x02\n GetContractIDsOfProviderResponse\x12k\n\tcontracts\x18\x01 \x03(\x0b\x32X.dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderResponse.ContractsEntry\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\x1a\x30\n\x0e\x43ontractsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x80\x02\n&GetContractsOfApplicantWithProvRequest\x12\x42\n\x0b\x61pplicantID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12N\n\x15namespaceIDOfProvider\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\"\x93\x02\n\'GetContractsOfApplicantWithProvResponse\x12r\n\tcontracts\x18\x01 \x03(\x0b\x32_.dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvResponse.ContractsEntry\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\x1a\x30\n\x0e\x43ontractsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xb8\x01\n\x16NewDataContractRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x18\n\x10\x64\x61taContractYaml\x18\x03 \x01(\t\"\xa9\x01\n\x17NewDataContractResponse\x12J\n\x0e\x64\x61taContractID\x18\x01 \x01(\x0b\x32\x32.dataclay.communication.grpc.common.DataContractID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xf7\x01\n#RegisterToPublicDataContractRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12J\n\x0e\x64\x61taContractID\x18\x03 \x01(\x0b\x32\x32.dataclay.communication.grpc.common.DataContractID\"\xae\x01\n$GetDataContractIDsOfApplicantRequest\x12\x42\n\x0b\x61pplicantID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\"\x9b\x02\n%GetDataContractIDsOfApplicantResponse\x12x\n\rdatacontracts\x18\x01 \x03(\x0b\x32\x61.dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantResponse.DatacontractsEntry\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\x1a\x34\n\x12\x44\x61tacontractsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xf8\x01\n#GetDataContractIDsOfProviderRequest\x12\x41\n\nproviderID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12J\n\x13\x64\x61taSetIDOfProvider\x18\x03 \x01(\x0b\x32-.dataclay.communication.grpc.common.DataSetID\"\x99\x02\n$GetDataContractIDsOfProviderResponse\x12w\n\rdatacontracts\x18\x01 \x03(\x0b\x32`.dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderResponse.DatacontractsEntry\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\x1a\x34\n\x12\x44\x61tacontractsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x83\x02\n-GetDataContractInfoOfApplicantWithProvRequest\x12\x42\n\x0b\x61pplicantID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12J\n\x13\x64\x61taSetIDOfProvider\x18\x03 \x01(\x0b\x32-.dataclay.communication.grpc.common.DataSetID\"\x8e\x01\n.GetDataContractInfoOfApplicantWithProvResponse\x12\x18\n\x10\x64\x61taContractInfo\x18\x01 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xb4\x01\n\x13NewInterfaceRequest\x12\x42\n\x0b\x61pplicantID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x15\n\rinterfaceYaml\x18\x03 \x01(\t\"\xa0\x01\n\x14NewInterfaceResponse\x12\x44\n\x0binterfaceID\x18\x01 \x01(\x0b\x32/.dataclay.communication.grpc.common.InterfaceID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xe7\x01\n\x17GetInterfaceInfoRequest\x12\x42\n\x0b\x61pplicantID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0binterfaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.InterfaceID\"u\n\x18GetInterfaceInfoResponse\x12\x15\n\rinterfaceYaml\x18\x01 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xac\x02\n\x16RemoveInterfaceRequest\x12\x42\n\x0b\x61pplicantID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x44\n\x0bnamespaceID\x18\x03 \x01(\x0b\x32/.dataclay.communication.grpc.common.NamespaceID\x12\x44\n\x0binterfaceID\x18\x04 \x01(\x0b\x32/.dataclay.communication.grpc.common.InterfaceID\"r\n\x1eGetStorageLocationForDSRequest\x12P\n\x11storageLocationID\x18\x01 \x01(\x0b\x32\x35.dataclay.communication.grpc.common.StorageLocationID\"\x82\x01\n\x1fGetStorageLocationForDSResponse\x12\x1b\n\x13storageLocationYaml\x18\x01 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"t\n#GetExecutionEnvironmentForDSRequest\x12M\n\texecEnvID\x18\x01 \x01(\x0b\x32:.dataclay.communication.grpc.common.ExecutionEnvironmentID\"\x7f\n$GetExecutionEnvironmentForDSResponse\x12\x13\n\x0b\x65xecEnvYaml\x18\x01 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xa7\x01\n\x15GetDataClayIDResponse\x12J\n\ndataClayID\x18\x01 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"A\n\x1fRegisterExternalDataClayRequest\x12\x10\n\x08hostname\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\"\xb5\x01\n RegisterExternalDataClayResponse\x12M\n\rextDataClayID\x18\x01 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x9c\x01\n+NotifyRegistrationOfExternalDataClayRequest\x12M\n\rextDataClayID\x18\x01 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\x12\x10\n\x08hostname\x18\x02 \x01(\t\x12\x0c\n\x04port\x18\x03 \x01(\x05\"\xc1\x01\n,NotifyRegistrationOfExternalDataClayResponse\x12M\n\rextDataClayID\x18\x01 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\":\n\x1cGetExternalDataclayIDRequest\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\"\xb2\x01\n\x1dGetExternalDataclayIDResponse\x12M\n\rextDataClayID\x18\x01 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"j\n\x19GetExtDataClayInfoRequest\x12M\n\rextDataClayID\x18\x01 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\"y\n\x1aGetExtDataClayInfoResponse\x12\x17\n\x0f\x65xtDataClayYaml\x18\x01 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xfb\x01\n\x15\x46\x65\x64\x65rateObjectRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12M\n\rextDataClayID\x18\x03 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\x12\x11\n\trecursive\x18\x04 \x01(\x08\"\xfd\x01\n\x17UnfederateObjectRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12M\n\rextDataClayID\x18\x03 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\x12\x11\n\trecursive\x18\x04 \x01(\x08\"\xb8\x01\n!UnfederateObjectWithAllDCsRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12\x11\n\trecursive\x18\x03 \x01(\x08\"\x9b\x02\n\x1eMigrateFederatedObjectsRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12X\n\x18\x65xternalOriginDataClayID\x18\x02 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\x12]\n\x1d\x65xternalDestinationDataClayID\x18\x03 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\"\xbc\x01\n\x19\x46\x65\x64\x65rateAllObjectsRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12]\n\x1d\x65xternalDestinationDataClayID\x18\x03 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\"\xae\x01\n\x1bUnfederateAllObjectsRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12M\n\rextDataClayID\x18\x02 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\"i\n%UnfederateAllObjectsWithAllDCsRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\"\xa3\x04\n\x1dNotifyFederatedObjectsRequest\x12G\n\x07srcDcID\x18\x01 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\x12\x11\n\tsrcDcHost\x18\x02 \x01(\t\x12\x11\n\tsrcDcPort\x18\x03 \x01(\x05\x12l\n\x0bobjectsInfo\x18\x04 \x03(\x0b\x32W.dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.ObjectsInfoEntry\x12v\n\x10\x66\x65\x64\x65ratedObjects\x18\x05 \x03(\x0b\x32\\.dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.FederatedObjectsEntry\x1a\x32\n\x10ObjectsInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1ay\n\x15\x46\x65\x64\x65ratedObjectsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12O\n\x05value\x18\x02 \x01(\x0b\x32@.dataclay.communication.grpc.common.SerializedParametersOrReturn:\x02\x38\x01\"\xac\x01\n\x1fNotifyUnfederatedObjectsRequest\x12G\n\x07srcDcID\x18\x01 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\x12@\n\nobjectsIDs\x18\x02 \x03(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\"\xc0\x01\n/CheckObjectFederatedWithDataClayInstanceRequest\x12>\n\x08objectID\x18\x01 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12M\n\rextDataClayID\x18\x02 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\"\x8b\x01\n0CheckObjectFederatedWithDataClayInstanceResponse\x12\x13\n\x0bisFederated\x18\x01 \x01(\x08\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"]\n\x1b\x43heckObjectFederatedRequest\x12>\n\x08objectID\x18\x01 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\"w\n\x1c\x43heckObjectFederatedResponse\x12\x13\n\x0bisFederated\x18\x01 \x01(\x08\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"j\n(GetDataClaysObjectIsFederatedWithRequest\x12>\n\x08objectID\x18\x01 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\"\xbf\x01\n)GetDataClaysObjectIsFederatedWithResponse\x12N\n\x0e\x65xtDataClayIDs\x18\x01 \x03(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"j\n(GetExternalSourceDataClayOfObjectRequest\x12>\n\x08objectID\x18\x01 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\"\xbe\x01\n)GetExternalSourceDataClayOfObjectResponse\x12M\n\rextDataClayID\x18\x01 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xb2\x01\n\x1aRegisterObjectForGCRequest\x12\x45\n\x07regInfo\x18\x01 \x01(\x0b\x32\x34.dataclay.communication.grpc.common.RegistrationInfo\x12M\n\tbackendID\x18\x02 \x01(\x0b\x32:.dataclay.communication.grpc.common.ExecutionEnvironmentID\"\xf5\x01\n\x15RegisterObjectRequest\x12\x45\n\x07regInfo\x18\x01 \x01(\x0b\x32\x34.dataclay.communication.grpc.common.RegistrationInfo\x12M\n\tbackendID\x18\x02 \x01(\x0b\x32:.dataclay.communication.grpc.common.ExecutionEnvironmentID\x12\r\n\x05\x61lias\x18\x03 \x01(\t\x12\x37\n\x04lang\x18\x04 \x01(\x0e\x32).dataclay.communication.grpc.common.Langs\"\xbb\x01\n#GetExecutionEnvironmentsInfoRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x0b\x65xecEnvLang\x18\x02 \x01(\x0e\x32).dataclay.communication.grpc.common.Langs\x12\x12\n\nfromClient\x18\x03 \x01(\x08\"\x8a\x02\n$GetExecutionEnvironmentsInfoResponse\x12m\n\x08\x65xecEnvs\x18\x01 \x03(\x0b\x32[.dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoResponse.ExecEnvsEntry\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\x1a/\n\rExecEnvsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xec\x01\n$GetExecutionEnvironmentsNamesRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x42\n\ncredential\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12>\n\x0b\x65xecEnvLang\x18\x03 \x01(\x0e\x32).dataclay.communication.grpc.common.Langs\"}\n%GetExecutionEnvironmentsNamesResponse\x12\x10\n\x08\x65xecEnvs\x18\x01 \x03(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x98\x01\n\x14GetObjectInfoRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\"\x81\x01\n\x15GetObjectInfoResponse\x12\x11\n\tclassname\x18\x01 \x01(\t\x12\x11\n\tnamespace\x18\x02 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x03 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"l\n\x19GetObjectFromAliasRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12\r\n\x05\x61lias\x18\x02 \x01(\t\"\xac\x02\n\x1aGetObjectFromAliasResponse\x12>\n\x08objectID\x18\x01 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12@\n\x07\x63lassID\x18\x02 \x01(\x0b\x32/.dataclay.communication.grpc.common.MetaClassID\x12H\n\x04hint\x18\x03 \x01(\x0b\x32:.dataclay.communication.grpc.common.ExecutionEnvironmentID\x12\x42\n\x07\x65xcInfo\x18\x04 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"k\n\x0f\x41\x64\x64\x41liasRequest\x12I\n\x13objectIDToHaveAlias\x18\x01 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12\r\n\x05\x61lias\x18\x02 \x01(\t\"e\n\x12\x44\x65leteAliasRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12\r\n\x05\x61lias\x18\x02 \x01(\t\"m\n)GetObjectsMetaDataInfoOfClassForNMRequest\x12@\n\x07\x63lassID\x18\x01 \x01(\x0b\x32/.dataclay.communication.grpc.common.MetaClassID\"\x99\x02\n*GetObjectsMetaDataInfoOfClassForNMResponse\x12u\n\tmdataInfo\x18\x01 \x03(\x0b\x32\x62.dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMResponse.MdataInfoEntry\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\x1a\x30\n\x0eMdataInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xd9\x01\n\x13SetDataSetIDRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12@\n\tdatasetID\x18\x03 \x01(\x0b\x32-.dataclay.communication.grpc.common.DataSetID\"\xab\x01\n\'SetDataSetIDFromGarbageCollectorRequest\x12>\n\x08objectID\x18\x01 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12@\n\tdatasetID\x18\x02 \x01(\x0b\x32-.dataclay.communication.grpc.common.DataSetID\"\xeb\x01\n\x11NewVersionRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12T\n\x10optDestBackendID\x18\x03 \x01(\x0b\x32:.dataclay.communication.grpc.common.ExecutionEnvironmentID\"q\n\x12NewVersionResponse\x12\x17\n\x0fversionInfoYaml\x18\x01 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"v\n\x19\x43onsolidateVersionRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12\x17\n\x0fversionInfoYaml\x18\x02 \x01(\t\"\xfb\x01\n\x11NewReplicaRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12Q\n\rdestBackendID\x18\x03 \x01(\x0b\x32:.dataclay.communication.grpc.common.ExecutionEnvironmentID\x12\x11\n\trecursive\x18\x04 \x01(\x08\"\xab\x01\n\x12NewReplicaResponse\x12Q\n\rdestBackendID\x18\x01 \x01(\x0b\x32:.dataclay.communication.grpc.common.ExecutionEnvironmentID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xcd\x02\n\x11MoveObjectRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12P\n\x0csrcBackendID\x18\x03 \x01(\x0b\x32:.dataclay.communication.grpc.common.ExecutionEnvironmentID\x12Q\n\rdestBackendID\x18\x04 \x01(\x0b\x32:.dataclay.communication.grpc.common.ExecutionEnvironmentID\x12\x11\n\trecursive\x18\x05 \x01(\x08\"\x99\x01\n\x12MoveObjectResponse\x12?\n\tobjectIDs\x18\x01 \x03(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x9c\x01\n\x18SetObjectReadOnlyRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\"\x9d\x01\n\x19SetObjectReadWriteRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\"\x9b\x01\n\x17GetMetadataByOIDRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\"t\n\x18GetMetadataByOIDResponse\x12\x14\n\x0cobjMdataYaml\x18\x01 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"^\n\x1cGetMetadataByOIDForDSRequest\x12>\n\x08objectID\x18\x01 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\"y\n\x1dGetMetadataByOIDForDSResponse\x12\x14\n\x0cobjMdataYaml\x18\x01 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\x92\x04\n\x1c\x45xecuteImplementationRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12\x44\n\x0boperationID\x18\x02 \x01(\x0b\x32/.dataclay.communication.grpc.common.OperationID\x12N\n\x10implementationID\x18\x03 \x01(\x0b\x32\x34.dataclay.communication.grpc.common.ImplementationID\x12\x42\n\ncontractID\x18\x04 \x01(\x0b\x32..dataclay.communication.grpc.common.ContractID\x12\x44\n\x0binterfaceID\x18\x05 \x01(\x0b\x32/.dataclay.communication.grpc.common.InterfaceID\x12>\n\x08objectID\x18\x06 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12P\n\x06params\x18\x07 \x01(\x0b\x32@.dataclay.communication.grpc.common.SerializedParametersOrReturn\"\xb2\x01\n\x1d\x45xecuteImplementationResponse\x12M\n\x03ret\x18\x01 \x01(\x0b\x32@.dataclay.communication.grpc.common.SerializedParametersOrReturn\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xea\x02\n\x1c\x45xecuteMethodOnTargetRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12!\n\x19operationNameAndSignature\x18\x03 \x01(\t\x12P\n\x06params\x18\x04 \x01(\x0b\x32@.dataclay.communication.grpc.common.SerializedParametersOrReturn\x12S\n\x0ftargetBackendID\x18\x05 \x01(\x0b\x32:.dataclay.communication.grpc.common.ExecutionEnvironmentID\"\xb2\x01\n\x1d\x45xecuteMethodOnTargetResponse\x12M\n\x03ret\x18\x01 \x01(\x0b\x32@.dataclay.communication.grpc.common.SerializedParametersOrReturn\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xe9\x02\n!SynchronizeFederatedObjectRequest\x12M\n\rextDataClayID\x18\x01 \x01(\x0b\x32\x36.dataclay.communication.grpc.common.DataClayInstanceID\x12>\n\x08objectID\x18\x02 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\x12N\n\x10implementationID\x18\x03 \x01(\x0b\x32\x34.dataclay.communication.grpc.common.ImplementationID\x12P\n\x06params\x18\x04 \x01(\x0b\x32@.dataclay.communication.grpc.common.SerializedParametersOrReturn\x12\x13\n\x0b\x61llBackends\x18\x05 \x01(\x08\"\xa3\x02\n\x0fGetStubsRequest\x12I\n\x12\x61pplicantAccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x43\n\x0b\x63redentials\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12;\n\x08language\x18\x03 \x01(\x0e\x32).dataclay.communication.grpc.common.Langs\x12\x43\n\x0b\x63ontractIDs\x18\x04 \x03(\x0b\x32..dataclay.communication.grpc.common.ContractID\"\xd9\x01\n\x10GetStubsResponse\x12S\n\x05stubs\x18\x01 \x03(\x0b\x32\x44.dataclay.communication.grpc.logicmodule.GetStubsResponse.StubsEntry\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\x1a,\n\nStubsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01\"\xe2\x01\n\x14GetBabelStubsRequest\x12@\n\taccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x43\n\x0b\x63redentials\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x43\n\x0b\x63ontractIDs\x18\x04 \x03(\x0b\x32..dataclay.communication.grpc.common.ContractID\"m\n\x15GetBabelStubsResponse\x12\x10\n\x08yamlStub\x18\x01 \x01(\x0c\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\\\n\x18GetClassNameForDSRequest\x12@\n\x07\x63lassID\x18\x01 \x01(\x0b\x32/.dataclay.communication.grpc.common.MetaClassID\"r\n\x19GetClassNameForDSResponse\x12\x11\n\tclassName\x18\x01 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"h\n$GetClassNameAndNamespaceForDSRequest\x12@\n\x07\x63lassID\x18\x01 \x01(\x0b\x32/.dataclay.communication.grpc.common.MetaClassID\"\x91\x01\n%GetClassNameAndNamespaceForDSResponse\x12\x11\n\tclassName\x18\x01 \x01(\t\x12\x11\n\tnamespace\x18\x02 \x01(\t\x12\x42\n\x07\x65xcInfo\x18\x03 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"\xb1\x01\n\x12RegisterECARequest\x12I\n\x12\x61pplicantAccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x43\n\x0b\x63redentials\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\x12\x0b\n\x03\x65\x63\x61\x18\x03 \x01(\t\"\'\n\x12\x41\x64viseEventRequest\x12\x11\n\teventYaml\x18\x01 \x01(\t\"\xb8\x01\n&GetContractIDOfDataClayProviderRequest\x12I\n\x12\x61pplicantAccountID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.AccountID\x12\x43\n\x0b\x63redentials\x18\x02 \x01(\x0b\x32..dataclay.communication.grpc.common.Credential\"\xb1\x01\n\'GetContractIDOfDataClayProviderResponse\x12\x42\n\ncontractID\x18\x01 \x01(\x0b\x32..dataclay.communication.grpc.common.ContractID\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"(\n\x16\x41\x63tivateTracingRequest\x12\x0e\n\x06taskid\x18\x01 \x01(\x05\"W\n\x13\x43loseSessionRequest\x12@\n\tsessionID\x18\x01 \x01(\x0b\x32-.dataclay.communication.grpc.common.SessionID\"e\n\x18UnregisterObjectsRequest\x12I\n\x13objectsToUnregister\x18\x01 \x03(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\"s\n\x1cIsPrefetchingEnabledResponse\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfo\"_\n\x1dObjectExistsInDataClayRequest\x12>\n\x08objectID\x18\x01 \x01(\x0b\x32,.dataclay.communication.grpc.common.ObjectID\"t\n\x1eObjectExistsInDataClayResponse\x12\x0e\n\x06\x65xists\x18\x01 \x01(\x08\x12\x42\n\x07\x65xcInfo\x18\x02 \x01(\x0b\x32\x31.dataclay.communication.grpc.common.ExceptionInfoB4\n0dataclay.communication.grpc.messages.logicmoduleP\x00\x62\x06proto3')
  ,
  dependencies=[dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2.DESCRIPTOR,])




_NEWACCOUNTNOADMINREQUEST = _descriptor.Descriptor(
  name='NewAccountNoAdminRequest',
  full_name='dataclay.communication.grpc.logicmodule.NewAccountNoAdminRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='yamlNewAccount', full_name='dataclay.communication.grpc.logicmodule.NewAccountNoAdminRequest.yamlNewAccount', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=187,
  serialized_end=237,
)


_NEWACCOUNTREQUEST = _descriptor.Descriptor(
  name='NewAccountRequest',
  full_name='dataclay.communication.grpc.logicmodule.NewAccountRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='adminID', full_name='dataclay.communication.grpc.logicmodule.NewAccountRequest.adminID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='admincredential', full_name='dataclay.communication.grpc.logicmodule.NewAccountRequest.admincredential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='yamlNewAccount', full_name='dataclay.communication.grpc.logicmodule.NewAccountRequest.yamlNewAccount', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=240,
  serialized_end=420,
)


_NEWACCOUNTRESPONSE = _descriptor.Descriptor(
  name='NewAccountResponse',
  full_name='dataclay.communication.grpc.logicmodule.NewAccountResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='newAccountID', full_name='dataclay.communication.grpc.logicmodule.NewAccountResponse.newAccountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.NewAccountResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=423,
  serialized_end=580,
)


_PUBLISHADDRESSREQUEST = _descriptor.Descriptor(
  name='PublishAddressRequest',
  full_name='dataclay.communication.grpc.logicmodule.PublishAddressRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hostname', full_name='dataclay.communication.grpc.logicmodule.PublishAddressRequest.hostname', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port', full_name='dataclay.communication.grpc.logicmodule.PublishAddressRequest.port', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=582,
  serialized_end=637,
)


_GETACCOUNTIDREQUEST = _descriptor.Descriptor(
  name='GetAccountIDRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetAccountIDRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountName', full_name='dataclay.communication.grpc.logicmodule.GetAccountIDRequest.accountName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=639,
  serialized_end=681,
)


_GETACCOUNTIDRESPONSE = _descriptor.Descriptor(
  name='GetAccountIDResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetAccountIDResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='newAccountID', full_name='dataclay.communication.grpc.logicmodule.GetAccountIDResponse.newAccountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetAccountIDResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=684,
  serialized_end=843,
)


_AUTOREGISTERSLREQUEST = _descriptor.Descriptor(
  name='AutoRegisterSLRequest',
  full_name='dataclay.communication.grpc.logicmodule.AutoRegisterSLRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='storageLocationID', full_name='dataclay.communication.grpc.logicmodule.AutoRegisterSLRequest.storageLocationID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dsName', full_name='dataclay.communication.grpc.logicmodule.AutoRegisterSLRequest.dsName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dsHostname', full_name='dataclay.communication.grpc.logicmodule.AutoRegisterSLRequest.dsHostname', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dsPort', full_name='dataclay.communication.grpc.logicmodule.AutoRegisterSLRequest.dsPort', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=846,
  serialized_end=1003,
)


_AUTOREGISTEREEREQUEST = _descriptor.Descriptor(
  name='AutoRegisterEERequest',
  full_name='dataclay.communication.grpc.logicmodule.AutoRegisterEERequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='executionEnvironmentID', full_name='dataclay.communication.grpc.logicmodule.AutoRegisterEERequest.executionEnvironmentID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='eeName', full_name='dataclay.communication.grpc.logicmodule.AutoRegisterEERequest.eeName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='eeHostname', full_name='dataclay.communication.grpc.logicmodule.AutoRegisterEERequest.eeHostname', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='eePort', full_name='dataclay.communication.grpc.logicmodule.AutoRegisterEERequest.eePort', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lang', full_name='dataclay.communication.grpc.logicmodule.AutoRegisterEERequest.lang', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1006,
  serialized_end=1230,
)


_AUTOREGISTEREERESPONSE = _descriptor.Descriptor(
  name='AutoRegisterEEResponse',
  full_name='dataclay.communication.grpc.logicmodule.AutoRegisterEEResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='storageLocationID', full_name='dataclay.communication.grpc.logicmodule.AutoRegisterEEResponse.storageLocationID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.AutoRegisterEEResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1233,
  serialized_end=1407,
)


_UNREGISTERSTORAGELOCATIONREQUEST = _descriptor.Descriptor(
  name='UnregisterStorageLocationRequest',
  full_name='dataclay.communication.grpc.logicmodule.UnregisterStorageLocationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='storageLocationID', full_name='dataclay.communication.grpc.logicmodule.UnregisterStorageLocationRequest.storageLocationID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1409,
  serialized_end=1525,
)


_UNREGISTEREXECUTIONENVIRONMENTREQUEST = _descriptor.Descriptor(
  name='UnregisterExecutionEnvironmentRequest',
  full_name='dataclay.communication.grpc.logicmodule.UnregisterExecutionEnvironmentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='executionEnvironmentID', full_name='dataclay.communication.grpc.logicmodule.UnregisterExecutionEnvironmentRequest.executionEnvironmentID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1528,
  serialized_end=1659,
)


_PERFORMSETACCOUNTSREQUEST = _descriptor.Descriptor(
  name='PerformSetAccountsRequest',
  full_name='dataclay.communication.grpc.logicmodule.PerformSetAccountsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.PerformSetAccountsRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.PerformSetAccountsRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='yaml', full_name='dataclay.communication.grpc.logicmodule.PerformSetAccountsRequest.yaml', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1662,
  serialized_end=1837,
)


_PERFORMSETACCOUNTSRESPONSE = _descriptor.Descriptor(
  name='PerformSetAccountsResponse',
  full_name='dataclay.communication.grpc.logicmodule.PerformSetAccountsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='resultYaml', full_name='dataclay.communication.grpc.logicmodule.PerformSetAccountsResponse.resultYaml', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.PerformSetAccountsResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1839,
  serialized_end=1955,
)


_PERFORMSETOPERATIONSREQUEST = _descriptor.Descriptor(
  name='PerformSetOperationsRequest',
  full_name='dataclay.communication.grpc.logicmodule.PerformSetOperationsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.PerformSetOperationsRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.PerformSetOperationsRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='yaml', full_name='dataclay.communication.grpc.logicmodule.PerformSetOperationsRequest.yaml', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1958,
  serialized_end=2135,
)


_PERFORMSETOPERATIONSRESPONSE = _descriptor.Descriptor(
  name='PerformSetOperationsResponse',
  full_name='dataclay.communication.grpc.logicmodule.PerformSetOperationsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='resultYaml', full_name='dataclay.communication.grpc.logicmodule.PerformSetOperationsResponse.resultYaml', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.PerformSetOperationsResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2137,
  serialized_end=2255,
)


_GETACCOUNTLISTREQUEST = _descriptor.Descriptor(
  name='GetAccountListRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetAccountListRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='adminID', full_name='dataclay.communication.grpc.logicmodule.GetAccountListRequest.adminID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='admincredential', full_name='dataclay.communication.grpc.logicmodule.GetAccountListRequest.admincredential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2258,
  serialized_end=2418,
)


_GETACCOUNTLISTRESPONSE = _descriptor.Descriptor(
  name='GetAccountListResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetAccountListResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountIDs', full_name='dataclay.communication.grpc.logicmodule.GetAccountListResponse.accountIDs', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetAccountListResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2420,
  serialized_end=2532,
)


_NEWSESSIONREQUEST = _descriptor.Descriptor(
  name='NewSessionRequest',
  full_name='dataclay.communication.grpc.logicmodule.NewSessionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.NewSessionRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.NewSessionRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contractIDs', full_name='dataclay.communication.grpc.logicmodule.NewSessionRequest.contractIDs', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataSetIDs', full_name='dataclay.communication.grpc.logicmodule.NewSessionRequest.dataSetIDs', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='storeDataSet', full_name='dataclay.communication.grpc.logicmodule.NewSessionRequest.storeDataSet', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sessionLang', full_name='dataclay.communication.grpc.logicmodule.NewSessionRequest.sessionLang', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2535,
  serialized_end=2957,
)


_NEWSESSIONRESPONSE = _descriptor.Descriptor(
  name='NewSessionResponse',
  full_name='dataclay.communication.grpc.logicmodule.NewSessionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionInfo', full_name='dataclay.communication.grpc.logicmodule.NewSessionResponse.sessionInfo', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.NewSessionResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2959,
  serialized_end=3068,
)


_GETINFOOFSESSIONFORDSREQUEST = _descriptor.Descriptor(
  name='GetInfoOfSessionForDSRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetInfoOfSessionForDSRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.GetInfoOfSessionForDSRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3070,
  serialized_end=3166,
)


_GETINFOOFSESSIONFORDSRESPONSE = _descriptor.Descriptor(
  name='GetInfoOfSessionForDSResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetInfoOfSessionForDSResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataSetID', full_name='dataclay.communication.grpc.logicmodule.GetInfoOfSessionForDSResponse.dataSetID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='date', full_name='dataclay.communication.grpc.logicmodule.GetInfoOfSessionForDSResponse.date', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataSetIDs', full_name='dataclay.communication.grpc.logicmodule.GetInfoOfSessionForDSResponse.dataSetIDs', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetInfoOfSessionForDSResponse.excInfo', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3169,
  serialized_end=3415,
)


_NEWNAMESPACEREQUEST = _descriptor.Descriptor(
  name='NewNamespaceRequest',
  full_name='dataclay.communication.grpc.logicmodule.NewNamespaceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.NewNamespaceRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.NewNamespaceRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='newNamespaceYaml', full_name='dataclay.communication.grpc.logicmodule.NewNamespaceRequest.newNamespaceYaml', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3418,
  serialized_end=3599,
)


_NEWNAMESPACERESPONSE = _descriptor.Descriptor(
  name='NewNamespaceResponse',
  full_name='dataclay.communication.grpc.logicmodule.NewNamespaceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.NewNamespaceResponse.namespaceID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.NewNamespaceResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3602,
  serialized_end=3762,
)


_GETNAMESPACESREQUEST = _descriptor.Descriptor(
  name='GetNamespacesRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetNamespacesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetNamespacesRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetNamespacesRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3765,
  serialized_end=3921,
)


_GETNAMESPACESRESPONSE = _descriptor.Descriptor(
  name='GetNamespacesResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetNamespacesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespaces', full_name='dataclay.communication.grpc.logicmodule.GetNamespacesResponse.namespaces', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetNamespacesResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3923,
  serialized_end=4034,
)


_REMOVENAMESPACEREQUEST = _descriptor.Descriptor(
  name='RemoveNamespaceRequest',
  full_name='dataclay.communication.grpc.logicmodule.RemoveNamespaceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.RemoveNamespaceRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.RemoveNamespaceRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceName', full_name='dataclay.communication.grpc.logicmodule.RemoveNamespaceRequest.namespaceName', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4037,
  serialized_end=4218,
)


_GETNAMESPACEIDREQUEST = _descriptor.Descriptor(
  name='GetNamespaceIDRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetNamespaceIDRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetNamespaceIDRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetNamespaceIDRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceName', full_name='dataclay.communication.grpc.logicmodule.GetNamespaceIDRequest.namespaceName', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4221,
  serialized_end=4401,
)


_GETNAMESPACEIDRESPONSE = _descriptor.Descriptor(
  name='GetNamespaceIDResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetNamespaceIDResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.GetNamespaceIDResponse.namespaceID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetNamespaceIDResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4404,
  serialized_end=4566,
)


_GETNAMESPACELANGREQUEST = _descriptor.Descriptor(
  name='GetNamespaceLangRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetNamespaceLangRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetNamespaceLangRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetNamespaceLangRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceName', full_name='dataclay.communication.grpc.logicmodule.GetNamespaceLangRequest.namespaceName', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4569,
  serialized_end=4751,
)


_GETNAMESPACELANGRESPONSE = _descriptor.Descriptor(
  name='GetNamespaceLangResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetNamespaceLangResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='language', full_name='dataclay.communication.grpc.logicmodule.GetNamespaceLangResponse.language', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetNamespaceLangResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4754,
  serialized_end=4909,
)


_GETOBJECTDATASETIDREQUEST = _descriptor.Descriptor(
  name='GetObjectDataSetIDRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetObjectDataSetIDRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.GetObjectDataSetIDRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.GetObjectDataSetIDRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4912,
  serialized_end=5069,
)


_GETOBJECTDATASETIDRESPONSE = _descriptor.Descriptor(
  name='GetObjectDataSetIDResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetObjectDataSetIDResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataSetID', full_name='dataclay.communication.grpc.logicmodule.GetObjectDataSetIDResponse.dataSetID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetObjectDataSetIDResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5072,
  serialized_end=5234,
)


_IMPORTINTERFACEREQUEST = _descriptor.Descriptor(
  name='ImportInterfaceRequest',
  full_name='dataclay.communication.grpc.logicmodule.ImportInterfaceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.ImportInterfaceRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.ImportInterfaceRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.ImportInterfaceRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contractID', full_name='dataclay.communication.grpc.logicmodule.ImportInterfaceRequest.contractID', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='interfaceID', full_name='dataclay.communication.grpc.logicmodule.ImportInterfaceRequest.interfaceID', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5237,
  serialized_end=5603,
)


_IMPORTCONTRACTREQUEST = _descriptor.Descriptor(
  name='ImportContractRequest',
  full_name='dataclay.communication.grpc.logicmodule.ImportContractRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.ImportContractRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.ImportContractRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.ImportContractRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contractID', full_name='dataclay.communication.grpc.logicmodule.ImportContractRequest.contractID', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5606,
  serialized_end=5901,
)


_NEWDATASETREQUEST = _descriptor.Descriptor(
  name='NewDataSetRequest',
  full_name='dataclay.communication.grpc.logicmodule.NewDataSetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.NewDataSetRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.NewDataSetRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataSetYaml', full_name='dataclay.communication.grpc.logicmodule.NewDataSetRequest.dataSetYaml', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5904,
  serialized_end=6078,
)


_NEWDATASETRESPONSE = _descriptor.Descriptor(
  name='NewDataSetResponse',
  full_name='dataclay.communication.grpc.logicmodule.NewDataSetResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataSetID', full_name='dataclay.communication.grpc.logicmodule.NewDataSetResponse.dataSetID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.NewDataSetResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6081,
  serialized_end=6235,
)


_REMOVEDATASETREQUEST = _descriptor.Descriptor(
  name='RemoveDataSetRequest',
  full_name='dataclay.communication.grpc.logicmodule.RemoveDataSetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.RemoveDataSetRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.RemoveDataSetRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataSetName', full_name='dataclay.communication.grpc.logicmodule.RemoveDataSetRequest.dataSetName', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6238,
  serialized_end=6415,
)


_GETDATASETIDREQUEST = _descriptor.Descriptor(
  name='GetDataSetIDRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetDataSetIDRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetDataSetIDRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetDataSetIDRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataSetName', full_name='dataclay.communication.grpc.logicmodule.GetDataSetIDRequest.dataSetName', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6418,
  serialized_end=6594,
)


_GETDATASETIDRESPONSE = _descriptor.Descriptor(
  name='GetDataSetIDResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetDataSetIDResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataSetID', full_name='dataclay.communication.grpc.logicmodule.GetDataSetIDResponse.dataSetID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetDataSetIDResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6597,
  serialized_end=6753,
)


_GETPUBLICDATASETSREQUEST = _descriptor.Descriptor(
  name='GetPublicDataSetsRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetPublicDataSetsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetPublicDataSetsRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetPublicDataSetsRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6756,
  serialized_end=6916,
)


_GETPUBLICDATASETSRESPONSE = _descriptor.Descriptor(
  name='GetPublicDataSetsResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetPublicDataSetsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataSets', full_name='dataclay.communication.grpc.logicmodule.GetPublicDataSetsResponse.dataSets', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetPublicDataSetsResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6918,
  serialized_end=7031,
)


_GETACCOUNTDATASETSREQUEST = _descriptor.Descriptor(
  name='GetAccountDataSetsRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetAccountDataSetsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetAccountDataSetsRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetAccountDataSetsRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7034,
  serialized_end=7195,
)


_GETACCOUNTDATASETSRESPONSE = _descriptor.Descriptor(
  name='GetAccountDataSetsResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetAccountDataSetsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataSets', full_name='dataclay.communication.grpc.logicmodule.GetAccountDataSetsResponse.dataSets', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetAccountDataSetsResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7197,
  serialized_end=7311,
)


_CHECKDATASETISPUBLICREQUEST = _descriptor.Descriptor(
  name='CheckDataSetIsPublicRequest',
  full_name='dataclay.communication.grpc.logicmodule.CheckDataSetIsPublicRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataSetID', full_name='dataclay.communication.grpc.logicmodule.CheckDataSetIsPublicRequest.dataSetID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7313,
  serialized_end=7408,
)


_CHECKDATASETISPUBLICRESPONSE = _descriptor.Descriptor(
  name='CheckDataSetIsPublicResponse',
  full_name='dataclay.communication.grpc.logicmodule.CheckDataSetIsPublicResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='isPublic', full_name='dataclay.communication.grpc.logicmodule.CheckDataSetIsPublicResponse.isPublic', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.CheckDataSetIsPublicResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7410,
  serialized_end=7526,
)


_NEWCLASSREQUEST_NEWCLASSESENTRY = _descriptor.Descriptor(
  name='NewClassesEntry',
  full_name='dataclay.communication.grpc.logicmodule.NewClassRequest.NewClassesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.NewClassRequest.NewClassesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.NewClassRequest.NewClassesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7837,
  serialized_end=7886,
)

_NEWCLASSREQUEST = _descriptor.Descriptor(
  name='NewClassRequest',
  full_name='dataclay.communication.grpc.logicmodule.NewClassRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.NewClassRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.NewClassRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='language', full_name='dataclay.communication.grpc.logicmodule.NewClassRequest.language', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='newClasses', full_name='dataclay.communication.grpc.logicmodule.NewClassRequest.newClasses', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_NEWCLASSREQUEST_NEWCLASSESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7529,
  serialized_end=7886,
)


_NEWCLASSRESPONSE_NEWCLASSESENTRY = _descriptor.Descriptor(
  name='NewClassesEntry',
  full_name='dataclay.communication.grpc.logicmodule.NewClassResponse.NewClassesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.NewClassResponse.NewClassesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.NewClassResponse.NewClassesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7837,
  serialized_end=7886,
)

_NEWCLASSRESPONSE = _descriptor.Descriptor(
  name='NewClassResponse',
  full_name='dataclay.communication.grpc.logicmodule.NewClassResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='newClasses', full_name='dataclay.communication.grpc.logicmodule.NewClassResponse.newClasses', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.NewClassResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_NEWCLASSRESPONSE_NEWCLASSESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7889,
  serialized_end=8121,
)


_NEWCLASSIDREQUEST_NEWCLASSESENTRY = _descriptor.Descriptor(
  name='NewClassesEntry',
  full_name='dataclay.communication.grpc.logicmodule.NewClassIDRequest.NewClassesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.NewClassIDRequest.NewClassesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.NewClassIDRequest.NewClassesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7837,
  serialized_end=7886,
)

_NEWCLASSIDREQUEST = _descriptor.Descriptor(
  name='NewClassIDRequest',
  full_name='dataclay.communication.grpc.logicmodule.NewClassIDRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.NewClassIDRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.NewClassIDRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='language', full_name='dataclay.communication.grpc.logicmodule.NewClassIDRequest.language', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='className', full_name='dataclay.communication.grpc.logicmodule.NewClassIDRequest.className', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='newClasses', full_name='dataclay.communication.grpc.logicmodule.NewClassIDRequest.newClasses', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_NEWCLASSIDREQUEST_NEWCLASSESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8124,
  serialized_end=8504,
)


_NEWCLASSIDRESPONSE = _descriptor.Descriptor(
  name='NewClassIDResponse',
  full_name='dataclay.communication.grpc.logicmodule.NewClassIDResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='classID', full_name='dataclay.communication.grpc.logicmodule.NewClassIDResponse.classID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.NewClassIDResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8507,
  serialized_end=8661,
)


_REMOVECLASSREQUEST = _descriptor.Descriptor(
  name='RemoveClassRequest',
  full_name='dataclay.communication.grpc.logicmodule.RemoveClassRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.RemoveClassRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.RemoveClassRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.RemoveClassRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='className', full_name='dataclay.communication.grpc.logicmodule.RemoveClassRequest.className', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8664,
  serialized_end=8907,
)


_REMOVEOPERATIONREQUEST = _descriptor.Descriptor(
  name='RemoveOperationRequest',
  full_name='dataclay.communication.grpc.logicmodule.RemoveOperationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.RemoveOperationRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.RemoveOperationRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.RemoveOperationRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='className', full_name='dataclay.communication.grpc.logicmodule.RemoveOperationRequest.className', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='operationNameAndSignature', full_name='dataclay.communication.grpc.logicmodule.RemoveOperationRequest.operationNameAndSignature', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8910,
  serialized_end=9192,
)


_REMOVEIMPLEMENTATIONREQUEST = _descriptor.Descriptor(
  name='RemoveImplementationRequest',
  full_name='dataclay.communication.grpc.logicmodule.RemoveImplementationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.RemoveImplementationRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.RemoveImplementationRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.RemoveImplementationRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='className', full_name='dataclay.communication.grpc.logicmodule.RemoveImplementationRequest.className', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='implementationID', full_name='dataclay.communication.grpc.logicmodule.RemoveImplementationRequest.implementationID', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9195,
  serialized_end=9527,
)


_GETOPERATIONIDREQUEST = _descriptor.Descriptor(
  name='GetOperationIDRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetOperationIDRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetOperationIDRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetOperationIDRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.GetOperationIDRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='className', full_name='dataclay.communication.grpc.logicmodule.GetOperationIDRequest.className', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='operationNameAndSignature', full_name='dataclay.communication.grpc.logicmodule.GetOperationIDRequest.operationNameAndSignature', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9530,
  serialized_end=9811,
)


_GETOPERATIONIDRESPONSE = _descriptor.Descriptor(
  name='GetOperationIDResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetOperationIDResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='operationID', full_name='dataclay.communication.grpc.logicmodule.GetOperationIDResponse.operationID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetOperationIDResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9814,
  serialized_end=9976,
)


_GETPROPERTYIDREQUEST = _descriptor.Descriptor(
  name='GetPropertyIDRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetPropertyIDRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetPropertyIDRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetPropertyIDRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.GetPropertyIDRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='className', full_name='dataclay.communication.grpc.logicmodule.GetPropertyIDRequest.className', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='propertyName', full_name='dataclay.communication.grpc.logicmodule.GetPropertyIDRequest.propertyName', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9979,
  serialized_end=10246,
)


_GETPROPERTYIDRESPONSE = _descriptor.Descriptor(
  name='GetPropertyIDResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetPropertyIDResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='propertyID', full_name='dataclay.communication.grpc.logicmodule.GetPropertyIDResponse.propertyID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetPropertyIDResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10249,
  serialized_end=10408,
)


_GETCLASSIDREQUEST = _descriptor.Descriptor(
  name='GetClassIDRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetClassIDRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetClassIDRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetClassIDRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.GetClassIDRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='className', full_name='dataclay.communication.grpc.logicmodule.GetClassIDRequest.className', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10411,
  serialized_end=10653,
)


_GETCLASSIDRESPONSE = _descriptor.Descriptor(
  name='GetClassIDResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetClassIDResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='classID', full_name='dataclay.communication.grpc.logicmodule.GetClassIDResponse.classID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetClassIDResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10656,
  serialized_end=10810,
)


_GETCLASSINFOREQUEST = _descriptor.Descriptor(
  name='GetClassInfoRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetClassInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetClassInfoRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetClassInfoRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.GetClassInfoRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='className', full_name='dataclay.communication.grpc.logicmodule.GetClassInfoRequest.className', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10813,
  serialized_end=11057,
)


_GETCLASSINFORESPONSE = _descriptor.Descriptor(
  name='GetClassInfoResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetClassInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='metaClassYaml', full_name='dataclay.communication.grpc.logicmodule.GetClassInfoResponse.metaClassYaml', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetClassInfoResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11059,
  serialized_end=11172,
)


_GETINFOOFCLASSESINNAMESPACEREQUEST = _descriptor.Descriptor(
  name='GetInfoOfClassesInNamespaceRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11175,
  serialized_end=11415,
)


_GETINFOOFCLASSESINNAMESPACERESPONSE_CLASSESINFOENTRY = _descriptor.Descriptor(
  name='ClassesInfoEntry',
  full_name='dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceResponse.ClassesInfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceResponse.ClassesInfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceResponse.ClassesInfoEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11641,
  serialized_end=11691,
)

_GETINFOOFCLASSESINNAMESPACERESPONSE = _descriptor.Descriptor(
  name='GetInfoOfClassesInNamespaceResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='classesInfo', full_name='dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceResponse.classesInfo', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETINFOOFCLASSESINNAMESPACERESPONSE_CLASSESINFOENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11418,
  serialized_end=11691,
)


_GETIMPORTEDCLASSESINFOINNAMESPACEREQUEST = _descriptor.Descriptor(
  name='GetImportedClassesInfoInNamespaceRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11694,
  serialized_end=11940,
)


_GETIMPORTEDCLASSESINFOINNAMESPACERESPONSE_IMPORTEDCLASSESENTRY = _descriptor.Descriptor(
  name='ImportedClassesEntry',
  full_name='dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceResponse.ImportedClassesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceResponse.ImportedClassesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceResponse.ImportedClassesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12187,
  serialized_end=12241,
)

_GETIMPORTEDCLASSESINFOINNAMESPACERESPONSE = _descriptor.Descriptor(
  name='GetImportedClassesInfoInNamespaceResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='importedClasses', full_name='dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceResponse.importedClasses', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETIMPORTEDCLASSESINFOINNAMESPACERESPONSE_IMPORTEDCLASSESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11943,
  serialized_end=12241,
)


_GETCLASSIDFROMIMPORTREQUEST = _descriptor.Descriptor(
  name='GetClassIDFromImportRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetClassIDFromImportRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetClassIDFromImportRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetClassIDFromImportRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.GetClassIDFromImportRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='className', full_name='dataclay.communication.grpc.logicmodule.GetClassIDFromImportRequest.className', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12244,
  serialized_end=12496,
)


_GETCLASSIDFROMIMPORTRESPONSE = _descriptor.Descriptor(
  name='GetClassIDFromImportResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetClassIDFromImportResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='classID', full_name='dataclay.communication.grpc.logicmodule.GetClassIDFromImportResponse.classID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetClassIDFromImportResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12499,
  serialized_end=12663,
)


_NEWCONTRACTREQUEST = _descriptor.Descriptor(
  name='NewContractRequest',
  full_name='dataclay.communication.grpc.logicmodule.NewContractRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.NewContractRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.NewContractRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='newContractYaml', full_name='dataclay.communication.grpc.logicmodule.NewContractRequest.newContractYaml', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12666,
  serialized_end=12845,
)


_NEWCONTRACTRESPONSE = _descriptor.Descriptor(
  name='NewContractResponse',
  full_name='dataclay.communication.grpc.logicmodule.NewContractResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='contractID', full_name='dataclay.communication.grpc.logicmodule.NewContractResponse.contractID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.NewContractResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12848,
  serialized_end=13005,
)


_REGISTERTOPUBLICCONTRACTREQUEST = _descriptor.Descriptor(
  name='RegisterToPublicContractRequest',
  full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicContractRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicContractRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicContractRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contractID', full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicContractRequest.contractID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13008,
  serialized_end=13243,
)


_REGISTERTOPUBLICCONTRACTOFNAMESPACEREQUEST = _descriptor.Descriptor(
  name='RegisterToPublicContractOfNamespaceRequest',
  full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicContractOfNamespaceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicContractOfNamespaceRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicContractOfNamespaceRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicContractOfNamespaceRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13246,
  serialized_end=13494,
)


_REGISTERTOPUBLICCONTRACTOFNAMESPACERESPONSE = _descriptor.Descriptor(
  name='RegisterToPublicContractOfNamespaceResponse',
  full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicContractOfNamespaceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='contractID', full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicContractOfNamespaceResponse.contractID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicContractOfNamespaceResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13497,
  serialized_end=13678,
)


_GETCONTRACTIDSOFAPPLICANTREQUEST = _descriptor.Descriptor(
  name='GetContractIDsOfApplicantRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='applicantID', full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantRequest.applicantID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13681,
  serialized_end=13851,
)


_GETCONTRACTIDSOFAPPLICANTRESPONSE_CONTRACTSENTRY = _descriptor.Descriptor(
  name='ContractsEntry',
  full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantResponse.ContractsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantResponse.ContractsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantResponse.ContractsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14069,
  serialized_end=14117,
)

_GETCONTRACTIDSOFAPPLICANTRESPONSE = _descriptor.Descriptor(
  name='GetContractIDsOfApplicantResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='contracts', full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantResponse.contracts', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETCONTRACTIDSOFAPPLICANTRESPONSE_CONTRACTSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13854,
  serialized_end=14117,
)


_GETCONTRACTIDSOFPROVIDERREQUEST = _descriptor.Descriptor(
  name='GetContractIDsOfProviderRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='providerID', full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderRequest.providerID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceIDOfProvider', full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderRequest.namespaceIDOfProvider', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14120,
  serialized_end=14368,
)


_GETCONTRACTIDSOFPROVIDERRESPONSE_CONTRACTSENTRY = _descriptor.Descriptor(
  name='ContractsEntry',
  full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderResponse.ContractsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderResponse.ContractsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderResponse.ContractsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14069,
  serialized_end=14117,
)

_GETCONTRACTIDSOFPROVIDERRESPONSE = _descriptor.Descriptor(
  name='GetContractIDsOfProviderResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='contracts', full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderResponse.contracts', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETCONTRACTIDSOFPROVIDERRESPONSE_CONTRACTSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14371,
  serialized_end=14632,
)


_GETCONTRACTSOFAPPLICANTWITHPROVREQUEST = _descriptor.Descriptor(
  name='GetContractsOfApplicantWithProvRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='applicantID', full_name='dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvRequest.applicantID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceIDOfProvider', full_name='dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvRequest.namespaceIDOfProvider', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14635,
  serialized_end=14891,
)


_GETCONTRACTSOFAPPLICANTWITHPROVRESPONSE_CONTRACTSENTRY = _descriptor.Descriptor(
  name='ContractsEntry',
  full_name='dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvResponse.ContractsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvResponse.ContractsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvResponse.ContractsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14069,
  serialized_end=14117,
)

_GETCONTRACTSOFAPPLICANTWITHPROVRESPONSE = _descriptor.Descriptor(
  name='GetContractsOfApplicantWithProvResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='contracts', full_name='dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvResponse.contracts', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETCONTRACTSOFAPPLICANTWITHPROVRESPONSE_CONTRACTSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14894,
  serialized_end=15169,
)


_NEWDATACONTRACTREQUEST = _descriptor.Descriptor(
  name='NewDataContractRequest',
  full_name='dataclay.communication.grpc.logicmodule.NewDataContractRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.NewDataContractRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.NewDataContractRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataContractYaml', full_name='dataclay.communication.grpc.logicmodule.NewDataContractRequest.dataContractYaml', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15172,
  serialized_end=15356,
)


_NEWDATACONTRACTRESPONSE = _descriptor.Descriptor(
  name='NewDataContractResponse',
  full_name='dataclay.communication.grpc.logicmodule.NewDataContractResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataContractID', full_name='dataclay.communication.grpc.logicmodule.NewDataContractResponse.dataContractID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.NewDataContractResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15359,
  serialized_end=15528,
)


_REGISTERTOPUBLICDATACONTRACTREQUEST = _descriptor.Descriptor(
  name='RegisterToPublicDataContractRequest',
  full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicDataContractRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicDataContractRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicDataContractRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataContractID', full_name='dataclay.communication.grpc.logicmodule.RegisterToPublicDataContractRequest.dataContractID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15531,
  serialized_end=15778,
)


_GETDATACONTRACTIDSOFAPPLICANTREQUEST = _descriptor.Descriptor(
  name='GetDataContractIDsOfApplicantRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='applicantID', full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantRequest.applicantID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15781,
  serialized_end=15955,
)


_GETDATACONTRACTIDSOFAPPLICANTRESPONSE_DATACONTRACTSENTRY = _descriptor.Descriptor(
  name='DatacontractsEntry',
  full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantResponse.DatacontractsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantResponse.DatacontractsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantResponse.DatacontractsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16189,
  serialized_end=16241,
)

_GETDATACONTRACTIDSOFAPPLICANTRESPONSE = _descriptor.Descriptor(
  name='GetDataContractIDsOfApplicantResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='datacontracts', full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantResponse.datacontracts', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETDATACONTRACTIDSOFAPPLICANTRESPONSE_DATACONTRACTSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15958,
  serialized_end=16241,
)


_GETDATACONTRACTIDSOFPROVIDERREQUEST = _descriptor.Descriptor(
  name='GetDataContractIDsOfProviderRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='providerID', full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderRequest.providerID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataSetIDOfProvider', full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderRequest.dataSetIDOfProvider', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16244,
  serialized_end=16492,
)


_GETDATACONTRACTIDSOFPROVIDERRESPONSE_DATACONTRACTSENTRY = _descriptor.Descriptor(
  name='DatacontractsEntry',
  full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderResponse.DatacontractsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderResponse.DatacontractsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderResponse.DatacontractsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16189,
  serialized_end=16241,
)

_GETDATACONTRACTIDSOFPROVIDERRESPONSE = _descriptor.Descriptor(
  name='GetDataContractIDsOfProviderResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='datacontracts', full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderResponse.datacontracts', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETDATACONTRACTIDSOFPROVIDERRESPONSE_DATACONTRACTSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16495,
  serialized_end=16776,
)


_GETDATACONTRACTINFOOFAPPLICANTWITHPROVREQUEST = _descriptor.Descriptor(
  name='GetDataContractInfoOfApplicantWithProvRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetDataContractInfoOfApplicantWithProvRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='applicantID', full_name='dataclay.communication.grpc.logicmodule.GetDataContractInfoOfApplicantWithProvRequest.applicantID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetDataContractInfoOfApplicantWithProvRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataSetIDOfProvider', full_name='dataclay.communication.grpc.logicmodule.GetDataContractInfoOfApplicantWithProvRequest.dataSetIDOfProvider', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=16779,
  serialized_end=17038,
)


_GETDATACONTRACTINFOOFAPPLICANTWITHPROVRESPONSE = _descriptor.Descriptor(
  name='GetDataContractInfoOfApplicantWithProvResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetDataContractInfoOfApplicantWithProvResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataContractInfo', full_name='dataclay.communication.grpc.logicmodule.GetDataContractInfoOfApplicantWithProvResponse.dataContractInfo', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetDataContractInfoOfApplicantWithProvResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17041,
  serialized_end=17183,
)


_NEWINTERFACEREQUEST = _descriptor.Descriptor(
  name='NewInterfaceRequest',
  full_name='dataclay.communication.grpc.logicmodule.NewInterfaceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='applicantID', full_name='dataclay.communication.grpc.logicmodule.NewInterfaceRequest.applicantID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.NewInterfaceRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='interfaceYaml', full_name='dataclay.communication.grpc.logicmodule.NewInterfaceRequest.interfaceYaml', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17186,
  serialized_end=17366,
)


_NEWINTERFACERESPONSE = _descriptor.Descriptor(
  name='NewInterfaceResponse',
  full_name='dataclay.communication.grpc.logicmodule.NewInterfaceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='interfaceID', full_name='dataclay.communication.grpc.logicmodule.NewInterfaceResponse.interfaceID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.NewInterfaceResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17369,
  serialized_end=17529,
)


_GETINTERFACEINFOREQUEST = _descriptor.Descriptor(
  name='GetInterfaceInfoRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetInterfaceInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='applicantID', full_name='dataclay.communication.grpc.logicmodule.GetInterfaceInfoRequest.applicantID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetInterfaceInfoRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='interfaceID', full_name='dataclay.communication.grpc.logicmodule.GetInterfaceInfoRequest.interfaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17532,
  serialized_end=17763,
)


_GETINTERFACEINFORESPONSE = _descriptor.Descriptor(
  name='GetInterfaceInfoResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetInterfaceInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='interfaceYaml', full_name='dataclay.communication.grpc.logicmodule.GetInterfaceInfoResponse.interfaceYaml', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetInterfaceInfoResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17765,
  serialized_end=17882,
)


_REMOVEINTERFACEREQUEST = _descriptor.Descriptor(
  name='RemoveInterfaceRequest',
  full_name='dataclay.communication.grpc.logicmodule.RemoveInterfaceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='applicantID', full_name='dataclay.communication.grpc.logicmodule.RemoveInterfaceRequest.applicantID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.RemoveInterfaceRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespaceID', full_name='dataclay.communication.grpc.logicmodule.RemoveInterfaceRequest.namespaceID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='interfaceID', full_name='dataclay.communication.grpc.logicmodule.RemoveInterfaceRequest.interfaceID', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=17885,
  serialized_end=18185,
)


_GETSTORAGELOCATIONFORDSREQUEST = _descriptor.Descriptor(
  name='GetStorageLocationForDSRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetStorageLocationForDSRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='storageLocationID', full_name='dataclay.communication.grpc.logicmodule.GetStorageLocationForDSRequest.storageLocationID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18187,
  serialized_end=18301,
)


_GETSTORAGELOCATIONFORDSRESPONSE = _descriptor.Descriptor(
  name='GetStorageLocationForDSResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetStorageLocationForDSResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='storageLocationYaml', full_name='dataclay.communication.grpc.logicmodule.GetStorageLocationForDSResponse.storageLocationYaml', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetStorageLocationForDSResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18304,
  serialized_end=18434,
)


_GETEXECUTIONENVIRONMENTFORDSREQUEST = _descriptor.Descriptor(
  name='GetExecutionEnvironmentForDSRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentForDSRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='execEnvID', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentForDSRequest.execEnvID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18436,
  serialized_end=18552,
)


_GETEXECUTIONENVIRONMENTFORDSRESPONSE = _descriptor.Descriptor(
  name='GetExecutionEnvironmentForDSResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentForDSResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='execEnvYaml', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentForDSResponse.execEnvYaml', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentForDSResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18554,
  serialized_end=18681,
)


_GETDATACLAYIDRESPONSE = _descriptor.Descriptor(
  name='GetDataClayIDResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetDataClayIDResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataClayID', full_name='dataclay.communication.grpc.logicmodule.GetDataClayIDResponse.dataClayID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetDataClayIDResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18684,
  serialized_end=18851,
)


_REGISTEREXTERNALDATACLAYREQUEST = _descriptor.Descriptor(
  name='RegisterExternalDataClayRequest',
  full_name='dataclay.communication.grpc.logicmodule.RegisterExternalDataClayRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hostname', full_name='dataclay.communication.grpc.logicmodule.RegisterExternalDataClayRequest.hostname', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port', full_name='dataclay.communication.grpc.logicmodule.RegisterExternalDataClayRequest.port', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18853,
  serialized_end=18918,
)


_REGISTEREXTERNALDATACLAYRESPONSE = _descriptor.Descriptor(
  name='RegisterExternalDataClayResponse',
  full_name='dataclay.communication.grpc.logicmodule.RegisterExternalDataClayResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='extDataClayID', full_name='dataclay.communication.grpc.logicmodule.RegisterExternalDataClayResponse.extDataClayID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.RegisterExternalDataClayResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18921,
  serialized_end=19102,
)


_NOTIFYREGISTRATIONOFEXTERNALDATACLAYREQUEST = _descriptor.Descriptor(
  name='NotifyRegistrationOfExternalDataClayRequest',
  full_name='dataclay.communication.grpc.logicmodule.NotifyRegistrationOfExternalDataClayRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='extDataClayID', full_name='dataclay.communication.grpc.logicmodule.NotifyRegistrationOfExternalDataClayRequest.extDataClayID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hostname', full_name='dataclay.communication.grpc.logicmodule.NotifyRegistrationOfExternalDataClayRequest.hostname', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port', full_name='dataclay.communication.grpc.logicmodule.NotifyRegistrationOfExternalDataClayRequest.port', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19105,
  serialized_end=19261,
)


_NOTIFYREGISTRATIONOFEXTERNALDATACLAYRESPONSE = _descriptor.Descriptor(
  name='NotifyRegistrationOfExternalDataClayResponse',
  full_name='dataclay.communication.grpc.logicmodule.NotifyRegistrationOfExternalDataClayResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='extDataClayID', full_name='dataclay.communication.grpc.logicmodule.NotifyRegistrationOfExternalDataClayResponse.extDataClayID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.NotifyRegistrationOfExternalDataClayResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19264,
  serialized_end=19457,
)


_GETEXTERNALDATACLAYIDREQUEST = _descriptor.Descriptor(
  name='GetExternalDataclayIDRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetExternalDataclayIDRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='host', full_name='dataclay.communication.grpc.logicmodule.GetExternalDataclayIDRequest.host', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port', full_name='dataclay.communication.grpc.logicmodule.GetExternalDataclayIDRequest.port', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19459,
  serialized_end=19517,
)


_GETEXTERNALDATACLAYIDRESPONSE = _descriptor.Descriptor(
  name='GetExternalDataclayIDResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetExternalDataclayIDResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='extDataClayID', full_name='dataclay.communication.grpc.logicmodule.GetExternalDataclayIDResponse.extDataClayID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetExternalDataclayIDResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19520,
  serialized_end=19698,
)


_GETEXTDATACLAYINFOREQUEST = _descriptor.Descriptor(
  name='GetExtDataClayInfoRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetExtDataClayInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='extDataClayID', full_name='dataclay.communication.grpc.logicmodule.GetExtDataClayInfoRequest.extDataClayID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19700,
  serialized_end=19806,
)


_GETEXTDATACLAYINFORESPONSE = _descriptor.Descriptor(
  name='GetExtDataClayInfoResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetExtDataClayInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='extDataClayYaml', full_name='dataclay.communication.grpc.logicmodule.GetExtDataClayInfoResponse.extDataClayYaml', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetExtDataClayInfoResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19808,
  serialized_end=19929,
)


_FEDERATEOBJECTREQUEST = _descriptor.Descriptor(
  name='FederateObjectRequest',
  full_name='dataclay.communication.grpc.logicmodule.FederateObjectRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.FederateObjectRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.FederateObjectRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extDataClayID', full_name='dataclay.communication.grpc.logicmodule.FederateObjectRequest.extDataClayID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recursive', full_name='dataclay.communication.grpc.logicmodule.FederateObjectRequest.recursive', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19932,
  serialized_end=20183,
)


_UNFEDERATEOBJECTREQUEST = _descriptor.Descriptor(
  name='UnfederateObjectRequest',
  full_name='dataclay.communication.grpc.logicmodule.UnfederateObjectRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.UnfederateObjectRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.UnfederateObjectRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extDataClayID', full_name='dataclay.communication.grpc.logicmodule.UnfederateObjectRequest.extDataClayID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recursive', full_name='dataclay.communication.grpc.logicmodule.UnfederateObjectRequest.recursive', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20186,
  serialized_end=20439,
)


_UNFEDERATEOBJECTWITHALLDCSREQUEST = _descriptor.Descriptor(
  name='UnfederateObjectWithAllDCsRequest',
  full_name='dataclay.communication.grpc.logicmodule.UnfederateObjectWithAllDCsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.UnfederateObjectWithAllDCsRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.UnfederateObjectWithAllDCsRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recursive', full_name='dataclay.communication.grpc.logicmodule.UnfederateObjectWithAllDCsRequest.recursive', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20442,
  serialized_end=20626,
)


_MIGRATEFEDERATEDOBJECTSREQUEST = _descriptor.Descriptor(
  name='MigrateFederatedObjectsRequest',
  full_name='dataclay.communication.grpc.logicmodule.MigrateFederatedObjectsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.MigrateFederatedObjectsRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='externalOriginDataClayID', full_name='dataclay.communication.grpc.logicmodule.MigrateFederatedObjectsRequest.externalOriginDataClayID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='externalDestinationDataClayID', full_name='dataclay.communication.grpc.logicmodule.MigrateFederatedObjectsRequest.externalDestinationDataClayID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20629,
  serialized_end=20912,
)


_FEDERATEALLOBJECTSREQUEST = _descriptor.Descriptor(
  name='FederateAllObjectsRequest',
  full_name='dataclay.communication.grpc.logicmodule.FederateAllObjectsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.FederateAllObjectsRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='externalDestinationDataClayID', full_name='dataclay.communication.grpc.logicmodule.FederateAllObjectsRequest.externalDestinationDataClayID', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20915,
  serialized_end=21103,
)


_UNFEDERATEALLOBJECTSREQUEST = _descriptor.Descriptor(
  name='UnfederateAllObjectsRequest',
  full_name='dataclay.communication.grpc.logicmodule.UnfederateAllObjectsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.UnfederateAllObjectsRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extDataClayID', full_name='dataclay.communication.grpc.logicmodule.UnfederateAllObjectsRequest.extDataClayID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21106,
  serialized_end=21280,
)


_UNFEDERATEALLOBJECTSWITHALLDCSREQUEST = _descriptor.Descriptor(
  name='UnfederateAllObjectsWithAllDCsRequest',
  full_name='dataclay.communication.grpc.logicmodule.UnfederateAllObjectsWithAllDCsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.UnfederateAllObjectsWithAllDCsRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21282,
  serialized_end=21387,
)


_NOTIFYFEDERATEDOBJECTSREQUEST_OBJECTSINFOENTRY = _descriptor.Descriptor(
  name='ObjectsInfoEntry',
  full_name='dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.ObjectsInfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.ObjectsInfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.ObjectsInfoEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21764,
  serialized_end=21814,
)

_NOTIFYFEDERATEDOBJECTSREQUEST_FEDERATEDOBJECTSENTRY = _descriptor.Descriptor(
  name='FederatedObjectsEntry',
  full_name='dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.FederatedObjectsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.FederatedObjectsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.FederatedObjectsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21816,
  serialized_end=21937,
)

_NOTIFYFEDERATEDOBJECTSREQUEST = _descriptor.Descriptor(
  name='NotifyFederatedObjectsRequest',
  full_name='dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='srcDcID', full_name='dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.srcDcID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='srcDcHost', full_name='dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.srcDcHost', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='srcDcPort', full_name='dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.srcDcPort', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectsInfo', full_name='dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.objectsInfo', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='federatedObjects', full_name='dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.federatedObjects', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_NOTIFYFEDERATEDOBJECTSREQUEST_OBJECTSINFOENTRY, _NOTIFYFEDERATEDOBJECTSREQUEST_FEDERATEDOBJECTSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21390,
  serialized_end=21937,
)


_NOTIFYUNFEDERATEDOBJECTSREQUEST = _descriptor.Descriptor(
  name='NotifyUnfederatedObjectsRequest',
  full_name='dataclay.communication.grpc.logicmodule.NotifyUnfederatedObjectsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='srcDcID', full_name='dataclay.communication.grpc.logicmodule.NotifyUnfederatedObjectsRequest.srcDcID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectsIDs', full_name='dataclay.communication.grpc.logicmodule.NotifyUnfederatedObjectsRequest.objectsIDs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21940,
  serialized_end=22112,
)


_CHECKOBJECTFEDERATEDWITHDATACLAYINSTANCEREQUEST = _descriptor.Descriptor(
  name='CheckObjectFederatedWithDataClayInstanceRequest',
  full_name='dataclay.communication.grpc.logicmodule.CheckObjectFederatedWithDataClayInstanceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.CheckObjectFederatedWithDataClayInstanceRequest.objectID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extDataClayID', full_name='dataclay.communication.grpc.logicmodule.CheckObjectFederatedWithDataClayInstanceRequest.extDataClayID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22115,
  serialized_end=22307,
)


_CHECKOBJECTFEDERATEDWITHDATACLAYINSTANCERESPONSE = _descriptor.Descriptor(
  name='CheckObjectFederatedWithDataClayInstanceResponse',
  full_name='dataclay.communication.grpc.logicmodule.CheckObjectFederatedWithDataClayInstanceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='isFederated', full_name='dataclay.communication.grpc.logicmodule.CheckObjectFederatedWithDataClayInstanceResponse.isFederated', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.CheckObjectFederatedWithDataClayInstanceResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22310,
  serialized_end=22449,
)


_CHECKOBJECTFEDERATEDREQUEST = _descriptor.Descriptor(
  name='CheckObjectFederatedRequest',
  full_name='dataclay.communication.grpc.logicmodule.CheckObjectFederatedRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.CheckObjectFederatedRequest.objectID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22451,
  serialized_end=22544,
)


_CHECKOBJECTFEDERATEDRESPONSE = _descriptor.Descriptor(
  name='CheckObjectFederatedResponse',
  full_name='dataclay.communication.grpc.logicmodule.CheckObjectFederatedResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='isFederated', full_name='dataclay.communication.grpc.logicmodule.CheckObjectFederatedResponse.isFederated', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.CheckObjectFederatedResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22546,
  serialized_end=22665,
)


_GETDATACLAYSOBJECTISFEDERATEDWITHREQUEST = _descriptor.Descriptor(
  name='GetDataClaysObjectIsFederatedWithRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetDataClaysObjectIsFederatedWithRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.GetDataClaysObjectIsFederatedWithRequest.objectID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22667,
  serialized_end=22773,
)


_GETDATACLAYSOBJECTISFEDERATEDWITHRESPONSE = _descriptor.Descriptor(
  name='GetDataClaysObjectIsFederatedWithResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetDataClaysObjectIsFederatedWithResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='extDataClayIDs', full_name='dataclay.communication.grpc.logicmodule.GetDataClaysObjectIsFederatedWithResponse.extDataClayIDs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetDataClaysObjectIsFederatedWithResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22776,
  serialized_end=22967,
)


_GETEXTERNALSOURCEDATACLAYOFOBJECTREQUEST = _descriptor.Descriptor(
  name='GetExternalSourceDataClayOfObjectRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetExternalSourceDataClayOfObjectRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.GetExternalSourceDataClayOfObjectRequest.objectID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22969,
  serialized_end=23075,
)


_GETEXTERNALSOURCEDATACLAYOFOBJECTRESPONSE = _descriptor.Descriptor(
  name='GetExternalSourceDataClayOfObjectResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetExternalSourceDataClayOfObjectResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='extDataClayID', full_name='dataclay.communication.grpc.logicmodule.GetExternalSourceDataClayOfObjectResponse.extDataClayID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetExternalSourceDataClayOfObjectResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=23078,
  serialized_end=23268,
)


_REGISTEROBJECTFORGCREQUEST = _descriptor.Descriptor(
  name='RegisterObjectForGCRequest',
  full_name='dataclay.communication.grpc.logicmodule.RegisterObjectForGCRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='regInfo', full_name='dataclay.communication.grpc.logicmodule.RegisterObjectForGCRequest.regInfo', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='backendID', full_name='dataclay.communication.grpc.logicmodule.RegisterObjectForGCRequest.backendID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=23271,
  serialized_end=23449,
)


_REGISTEROBJECTREQUEST = _descriptor.Descriptor(
  name='RegisterObjectRequest',
  full_name='dataclay.communication.grpc.logicmodule.RegisterObjectRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='regInfo', full_name='dataclay.communication.grpc.logicmodule.RegisterObjectRequest.regInfo', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='backendID', full_name='dataclay.communication.grpc.logicmodule.RegisterObjectRequest.backendID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alias', full_name='dataclay.communication.grpc.logicmodule.RegisterObjectRequest.alias', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lang', full_name='dataclay.communication.grpc.logicmodule.RegisterObjectRequest.lang', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=23452,
  serialized_end=23697,
)


_GETEXECUTIONENVIRONMENTSINFOREQUEST = _descriptor.Descriptor(
  name='GetExecutionEnvironmentsInfoRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='execEnvLang', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoRequest.execEnvLang', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fromClient', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoRequest.fromClient', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=23700,
  serialized_end=23887,
)


_GETEXECUTIONENVIRONMENTSINFORESPONSE_EXECENVSENTRY = _descriptor.Descriptor(
  name='ExecEnvsEntry',
  full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoResponse.ExecEnvsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoResponse.ExecEnvsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoResponse.ExecEnvsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24109,
  serialized_end=24156,
)

_GETEXECUTIONENVIRONMENTSINFORESPONSE = _descriptor.Descriptor(
  name='GetExecutionEnvironmentsInfoResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='execEnvs', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoResponse.execEnvs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETEXECUTIONENVIRONMENTSINFORESPONSE_EXECENVSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=23890,
  serialized_end=24156,
)


_GETEXECUTIONENVIRONMENTSNAMESREQUEST = _descriptor.Descriptor(
  name='GetExecutionEnvironmentsNamesRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsNamesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsNamesRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsNamesRequest.credential', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='execEnvLang', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsNamesRequest.execEnvLang', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24159,
  serialized_end=24395,
)


_GETEXECUTIONENVIRONMENTSNAMESRESPONSE = _descriptor.Descriptor(
  name='GetExecutionEnvironmentsNamesResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsNamesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='execEnvs', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsNamesResponse.execEnvs', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsNamesResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24397,
  serialized_end=24522,
)


_GETOBJECTINFOREQUEST = _descriptor.Descriptor(
  name='GetObjectInfoRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetObjectInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.GetObjectInfoRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.GetObjectInfoRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24525,
  serialized_end=24677,
)


_GETOBJECTINFORESPONSE = _descriptor.Descriptor(
  name='GetObjectInfoResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetObjectInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='classname', full_name='dataclay.communication.grpc.logicmodule.GetObjectInfoResponse.classname', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespace', full_name='dataclay.communication.grpc.logicmodule.GetObjectInfoResponse.namespace', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetObjectInfoResponse.excInfo', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24680,
  serialized_end=24809,
)


_GETOBJECTFROMALIASREQUEST = _descriptor.Descriptor(
  name='GetObjectFromAliasRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetObjectFromAliasRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.GetObjectFromAliasRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alias', full_name='dataclay.communication.grpc.logicmodule.GetObjectFromAliasRequest.alias', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24811,
  serialized_end=24919,
)


_GETOBJECTFROMALIASRESPONSE = _descriptor.Descriptor(
  name='GetObjectFromAliasResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetObjectFromAliasResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.GetObjectFromAliasResponse.objectID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='classID', full_name='dataclay.communication.grpc.logicmodule.GetObjectFromAliasResponse.classID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hint', full_name='dataclay.communication.grpc.logicmodule.GetObjectFromAliasResponse.hint', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetObjectFromAliasResponse.excInfo', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24922,
  serialized_end=25222,
)


_ADDALIASREQUEST = _descriptor.Descriptor(
  name='AddAliasRequest',
  full_name='dataclay.communication.grpc.logicmodule.AddAliasRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objectIDToHaveAlias', full_name='dataclay.communication.grpc.logicmodule.AddAliasRequest.objectIDToHaveAlias', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alias', full_name='dataclay.communication.grpc.logicmodule.AddAliasRequest.alias', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=25224,
  serialized_end=25331,
)


_DELETEALIASREQUEST = _descriptor.Descriptor(
  name='DeleteAliasRequest',
  full_name='dataclay.communication.grpc.logicmodule.DeleteAliasRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.DeleteAliasRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alias', full_name='dataclay.communication.grpc.logicmodule.DeleteAliasRequest.alias', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=25333,
  serialized_end=25434,
)


_GETOBJECTSMETADATAINFOOFCLASSFORNMREQUEST = _descriptor.Descriptor(
  name='GetObjectsMetaDataInfoOfClassForNMRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='classID', full_name='dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMRequest.classID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=25436,
  serialized_end=25545,
)


_GETOBJECTSMETADATAINFOOFCLASSFORNMRESPONSE_MDATAINFOENTRY = _descriptor.Descriptor(
  name='MdataInfoEntry',
  full_name='dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMResponse.MdataInfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMResponse.MdataInfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMResponse.MdataInfoEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=25781,
  serialized_end=25829,
)

_GETOBJECTSMETADATAINFOOFCLASSFORNMRESPONSE = _descriptor.Descriptor(
  name='GetObjectsMetaDataInfoOfClassForNMResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mdataInfo', full_name='dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMResponse.mdataInfo', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETOBJECTSMETADATAINFOOFCLASSFORNMRESPONSE_MDATAINFOENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=25548,
  serialized_end=25829,
)


_SETDATASETIDREQUEST = _descriptor.Descriptor(
  name='SetDataSetIDRequest',
  full_name='dataclay.communication.grpc.logicmodule.SetDataSetIDRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.SetDataSetIDRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.SetDataSetIDRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='datasetID', full_name='dataclay.communication.grpc.logicmodule.SetDataSetIDRequest.datasetID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=25832,
  serialized_end=26049,
)


_SETDATASETIDFROMGARBAGECOLLECTORREQUEST = _descriptor.Descriptor(
  name='SetDataSetIDFromGarbageCollectorRequest',
  full_name='dataclay.communication.grpc.logicmodule.SetDataSetIDFromGarbageCollectorRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.SetDataSetIDFromGarbageCollectorRequest.objectID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='datasetID', full_name='dataclay.communication.grpc.logicmodule.SetDataSetIDFromGarbageCollectorRequest.datasetID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26052,
  serialized_end=26223,
)


_NEWVERSIONREQUEST = _descriptor.Descriptor(
  name='NewVersionRequest',
  full_name='dataclay.communication.grpc.logicmodule.NewVersionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.NewVersionRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.NewVersionRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='optDestBackendID', full_name='dataclay.communication.grpc.logicmodule.NewVersionRequest.optDestBackendID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26226,
  serialized_end=26461,
)


_NEWVERSIONRESPONSE = _descriptor.Descriptor(
  name='NewVersionResponse',
  full_name='dataclay.communication.grpc.logicmodule.NewVersionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='versionInfoYaml', full_name='dataclay.communication.grpc.logicmodule.NewVersionResponse.versionInfoYaml', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.NewVersionResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26463,
  serialized_end=26576,
)


_CONSOLIDATEVERSIONREQUEST = _descriptor.Descriptor(
  name='ConsolidateVersionRequest',
  full_name='dataclay.communication.grpc.logicmodule.ConsolidateVersionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.ConsolidateVersionRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='versionInfoYaml', full_name='dataclay.communication.grpc.logicmodule.ConsolidateVersionRequest.versionInfoYaml', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26578,
  serialized_end=26696,
)


_NEWREPLICAREQUEST = _descriptor.Descriptor(
  name='NewReplicaRequest',
  full_name='dataclay.communication.grpc.logicmodule.NewReplicaRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.NewReplicaRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.NewReplicaRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='destBackendID', full_name='dataclay.communication.grpc.logicmodule.NewReplicaRequest.destBackendID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recursive', full_name='dataclay.communication.grpc.logicmodule.NewReplicaRequest.recursive', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26699,
  serialized_end=26950,
)


_NEWREPLICARESPONSE = _descriptor.Descriptor(
  name='NewReplicaResponse',
  full_name='dataclay.communication.grpc.logicmodule.NewReplicaResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='destBackendID', full_name='dataclay.communication.grpc.logicmodule.NewReplicaResponse.destBackendID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.NewReplicaResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26953,
  serialized_end=27124,
)


_MOVEOBJECTREQUEST = _descriptor.Descriptor(
  name='MoveObjectRequest',
  full_name='dataclay.communication.grpc.logicmodule.MoveObjectRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.MoveObjectRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.MoveObjectRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='srcBackendID', full_name='dataclay.communication.grpc.logicmodule.MoveObjectRequest.srcBackendID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='destBackendID', full_name='dataclay.communication.grpc.logicmodule.MoveObjectRequest.destBackendID', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recursive', full_name='dataclay.communication.grpc.logicmodule.MoveObjectRequest.recursive', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=27127,
  serialized_end=27460,
)


_MOVEOBJECTRESPONSE = _descriptor.Descriptor(
  name='MoveObjectResponse',
  full_name='dataclay.communication.grpc.logicmodule.MoveObjectResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objectIDs', full_name='dataclay.communication.grpc.logicmodule.MoveObjectResponse.objectIDs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.MoveObjectResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=27463,
  serialized_end=27616,
)


_SETOBJECTREADONLYREQUEST = _descriptor.Descriptor(
  name='SetObjectReadOnlyRequest',
  full_name='dataclay.communication.grpc.logicmodule.SetObjectReadOnlyRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.SetObjectReadOnlyRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.SetObjectReadOnlyRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=27619,
  serialized_end=27775,
)


_SETOBJECTREADWRITEREQUEST = _descriptor.Descriptor(
  name='SetObjectReadWriteRequest',
  full_name='dataclay.communication.grpc.logicmodule.SetObjectReadWriteRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.SetObjectReadWriteRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.SetObjectReadWriteRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=27778,
  serialized_end=27935,
)


_GETMETADATABYOIDREQUEST = _descriptor.Descriptor(
  name='GetMetadataByOIDRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetMetadataByOIDRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.GetMetadataByOIDRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.GetMetadataByOIDRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=27938,
  serialized_end=28093,
)


_GETMETADATABYOIDRESPONSE = _descriptor.Descriptor(
  name='GetMetadataByOIDResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetMetadataByOIDResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objMdataYaml', full_name='dataclay.communication.grpc.logicmodule.GetMetadataByOIDResponse.objMdataYaml', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetMetadataByOIDResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28095,
  serialized_end=28211,
)


_GETMETADATABYOIDFORDSREQUEST = _descriptor.Descriptor(
  name='GetMetadataByOIDForDSRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetMetadataByOIDForDSRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.GetMetadataByOIDForDSRequest.objectID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28213,
  serialized_end=28307,
)


_GETMETADATABYOIDFORDSRESPONSE = _descriptor.Descriptor(
  name='GetMetadataByOIDForDSResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetMetadataByOIDForDSResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objMdataYaml', full_name='dataclay.communication.grpc.logicmodule.GetMetadataByOIDForDSResponse.objMdataYaml', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetMetadataByOIDForDSResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28309,
  serialized_end=28430,
)


_EXECUTEIMPLEMENTATIONREQUEST = _descriptor.Descriptor(
  name='ExecuteImplementationRequest',
  full_name='dataclay.communication.grpc.logicmodule.ExecuteImplementationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.ExecuteImplementationRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='operationID', full_name='dataclay.communication.grpc.logicmodule.ExecuteImplementationRequest.operationID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='implementationID', full_name='dataclay.communication.grpc.logicmodule.ExecuteImplementationRequest.implementationID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contractID', full_name='dataclay.communication.grpc.logicmodule.ExecuteImplementationRequest.contractID', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='interfaceID', full_name='dataclay.communication.grpc.logicmodule.ExecuteImplementationRequest.interfaceID', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.ExecuteImplementationRequest.objectID', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='params', full_name='dataclay.communication.grpc.logicmodule.ExecuteImplementationRequest.params', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28433,
  serialized_end=28963,
)


_EXECUTEIMPLEMENTATIONRESPONSE = _descriptor.Descriptor(
  name='ExecuteImplementationResponse',
  full_name='dataclay.communication.grpc.logicmodule.ExecuteImplementationResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ret', full_name='dataclay.communication.grpc.logicmodule.ExecuteImplementationResponse.ret', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.ExecuteImplementationResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28966,
  serialized_end=29144,
)


_EXECUTEMETHODONTARGETREQUEST = _descriptor.Descriptor(
  name='ExecuteMethodOnTargetRequest',
  full_name='dataclay.communication.grpc.logicmodule.ExecuteMethodOnTargetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.ExecuteMethodOnTargetRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.ExecuteMethodOnTargetRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='operationNameAndSignature', full_name='dataclay.communication.grpc.logicmodule.ExecuteMethodOnTargetRequest.operationNameAndSignature', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='params', full_name='dataclay.communication.grpc.logicmodule.ExecuteMethodOnTargetRequest.params', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='targetBackendID', full_name='dataclay.communication.grpc.logicmodule.ExecuteMethodOnTargetRequest.targetBackendID', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=29147,
  serialized_end=29509,
)


_EXECUTEMETHODONTARGETRESPONSE = _descriptor.Descriptor(
  name='ExecuteMethodOnTargetResponse',
  full_name='dataclay.communication.grpc.logicmodule.ExecuteMethodOnTargetResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ret', full_name='dataclay.communication.grpc.logicmodule.ExecuteMethodOnTargetResponse.ret', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.ExecuteMethodOnTargetResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=29512,
  serialized_end=29690,
)


_SYNCHRONIZEFEDERATEDOBJECTREQUEST = _descriptor.Descriptor(
  name='SynchronizeFederatedObjectRequest',
  full_name='dataclay.communication.grpc.logicmodule.SynchronizeFederatedObjectRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='extDataClayID', full_name='dataclay.communication.grpc.logicmodule.SynchronizeFederatedObjectRequest.extDataClayID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.SynchronizeFederatedObjectRequest.objectID', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='implementationID', full_name='dataclay.communication.grpc.logicmodule.SynchronizeFederatedObjectRequest.implementationID', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='params', full_name='dataclay.communication.grpc.logicmodule.SynchronizeFederatedObjectRequest.params', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='allBackends', full_name='dataclay.communication.grpc.logicmodule.SynchronizeFederatedObjectRequest.allBackends', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=29693,
  serialized_end=30054,
)


_GETSTUBSREQUEST = _descriptor.Descriptor(
  name='GetStubsRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetStubsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='applicantAccountID', full_name='dataclay.communication.grpc.logicmodule.GetStubsRequest.applicantAccountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credentials', full_name='dataclay.communication.grpc.logicmodule.GetStubsRequest.credentials', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='language', full_name='dataclay.communication.grpc.logicmodule.GetStubsRequest.language', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contractIDs', full_name='dataclay.communication.grpc.logicmodule.GetStubsRequest.contractIDs', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=30057,
  serialized_end=30348,
)


_GETSTUBSRESPONSE_STUBSENTRY = _descriptor.Descriptor(
  name='StubsEntry',
  full_name='dataclay.communication.grpc.logicmodule.GetStubsResponse.StubsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='dataclay.communication.grpc.logicmodule.GetStubsResponse.StubsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='dataclay.communication.grpc.logicmodule.GetStubsResponse.StubsEntry.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=30524,
  serialized_end=30568,
)

_GETSTUBSRESPONSE = _descriptor.Descriptor(
  name='GetStubsResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetStubsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stubs', full_name='dataclay.communication.grpc.logicmodule.GetStubsResponse.stubs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetStubsResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETSTUBSRESPONSE_STUBSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=30351,
  serialized_end=30568,
)


_GETBABELSTUBSREQUEST = _descriptor.Descriptor(
  name='GetBabelStubsRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetBabelStubsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accountID', full_name='dataclay.communication.grpc.logicmodule.GetBabelStubsRequest.accountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credentials', full_name='dataclay.communication.grpc.logicmodule.GetBabelStubsRequest.credentials', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contractIDs', full_name='dataclay.communication.grpc.logicmodule.GetBabelStubsRequest.contractIDs', index=2,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=30571,
  serialized_end=30797,
)


_GETBABELSTUBSRESPONSE = _descriptor.Descriptor(
  name='GetBabelStubsResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetBabelStubsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='yamlStub', full_name='dataclay.communication.grpc.logicmodule.GetBabelStubsResponse.yamlStub', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetBabelStubsResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=30799,
  serialized_end=30908,
)


_GETCLASSNAMEFORDSREQUEST = _descriptor.Descriptor(
  name='GetClassNameForDSRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetClassNameForDSRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='classID', full_name='dataclay.communication.grpc.logicmodule.GetClassNameForDSRequest.classID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=30910,
  serialized_end=31002,
)


_GETCLASSNAMEFORDSRESPONSE = _descriptor.Descriptor(
  name='GetClassNameForDSResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetClassNameForDSResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='className', full_name='dataclay.communication.grpc.logicmodule.GetClassNameForDSResponse.className', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetClassNameForDSResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31004,
  serialized_end=31118,
)


_GETCLASSNAMEANDNAMESPACEFORDSREQUEST = _descriptor.Descriptor(
  name='GetClassNameAndNamespaceForDSRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetClassNameAndNamespaceForDSRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='classID', full_name='dataclay.communication.grpc.logicmodule.GetClassNameAndNamespaceForDSRequest.classID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31120,
  serialized_end=31224,
)


_GETCLASSNAMEANDNAMESPACEFORDSRESPONSE = _descriptor.Descriptor(
  name='GetClassNameAndNamespaceForDSResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetClassNameAndNamespaceForDSResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='className', full_name='dataclay.communication.grpc.logicmodule.GetClassNameAndNamespaceForDSResponse.className', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespace', full_name='dataclay.communication.grpc.logicmodule.GetClassNameAndNamespaceForDSResponse.namespace', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetClassNameAndNamespaceForDSResponse.excInfo', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31227,
  serialized_end=31372,
)


_REGISTERECAREQUEST = _descriptor.Descriptor(
  name='RegisterECARequest',
  full_name='dataclay.communication.grpc.logicmodule.RegisterECARequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='applicantAccountID', full_name='dataclay.communication.grpc.logicmodule.RegisterECARequest.applicantAccountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credentials', full_name='dataclay.communication.grpc.logicmodule.RegisterECARequest.credentials', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='eca', full_name='dataclay.communication.grpc.logicmodule.RegisterECARequest.eca', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31375,
  serialized_end=31552,
)


_ADVISEEVENTREQUEST = _descriptor.Descriptor(
  name='AdviseEventRequest',
  full_name='dataclay.communication.grpc.logicmodule.AdviseEventRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='eventYaml', full_name='dataclay.communication.grpc.logicmodule.AdviseEventRequest.eventYaml', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31554,
  serialized_end=31593,
)


_GETCONTRACTIDOFDATACLAYPROVIDERREQUEST = _descriptor.Descriptor(
  name='GetContractIDOfDataClayProviderRequest',
  full_name='dataclay.communication.grpc.logicmodule.GetContractIDOfDataClayProviderRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='applicantAccountID', full_name='dataclay.communication.grpc.logicmodule.GetContractIDOfDataClayProviderRequest.applicantAccountID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credentials', full_name='dataclay.communication.grpc.logicmodule.GetContractIDOfDataClayProviderRequest.credentials', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31596,
  serialized_end=31780,
)


_GETCONTRACTIDOFDATACLAYPROVIDERRESPONSE = _descriptor.Descriptor(
  name='GetContractIDOfDataClayProviderResponse',
  full_name='dataclay.communication.grpc.logicmodule.GetContractIDOfDataClayProviderResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='contractID', full_name='dataclay.communication.grpc.logicmodule.GetContractIDOfDataClayProviderResponse.contractID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.GetContractIDOfDataClayProviderResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31783,
  serialized_end=31960,
)


_ACTIVATETRACINGREQUEST = _descriptor.Descriptor(
  name='ActivateTracingRequest',
  full_name='dataclay.communication.grpc.logicmodule.ActivateTracingRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='taskid', full_name='dataclay.communication.grpc.logicmodule.ActivateTracingRequest.taskid', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31962,
  serialized_end=32002,
)


_CLOSESESSIONREQUEST = _descriptor.Descriptor(
  name='CloseSessionRequest',
  full_name='dataclay.communication.grpc.logicmodule.CloseSessionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='dataclay.communication.grpc.logicmodule.CloseSessionRequest.sessionID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=32004,
  serialized_end=32091,
)


_UNREGISTEROBJECTSREQUEST = _descriptor.Descriptor(
  name='UnregisterObjectsRequest',
  full_name='dataclay.communication.grpc.logicmodule.UnregisterObjectsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objectsToUnregister', full_name='dataclay.communication.grpc.logicmodule.UnregisterObjectsRequest.objectsToUnregister', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=32093,
  serialized_end=32194,
)


_ISPREFETCHINGENABLEDRESPONSE = _descriptor.Descriptor(
  name='IsPrefetchingEnabledResponse',
  full_name='dataclay.communication.grpc.logicmodule.IsPrefetchingEnabledResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='enabled', full_name='dataclay.communication.grpc.logicmodule.IsPrefetchingEnabledResponse.enabled', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.IsPrefetchingEnabledResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=32196,
  serialized_end=32311,
)


_OBJECTEXISTSINDATACLAYREQUEST = _descriptor.Descriptor(
  name='ObjectExistsInDataClayRequest',
  full_name='dataclay.communication.grpc.logicmodule.ObjectExistsInDataClayRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='objectID', full_name='dataclay.communication.grpc.logicmodule.ObjectExistsInDataClayRequest.objectID', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=32313,
  serialized_end=32408,
)


_OBJECTEXISTSINDATACLAYRESPONSE = _descriptor.Descriptor(
  name='ObjectExistsInDataClayResponse',
  full_name='dataclay.communication.grpc.logicmodule.ObjectExistsInDataClayResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='exists', full_name='dataclay.communication.grpc.logicmodule.ObjectExistsInDataClayResponse.exists', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excInfo', full_name='dataclay.communication.grpc.logicmodule.ObjectExistsInDataClayResponse.excInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=32410,
  serialized_end=32526,
)

_NEWACCOUNTREQUEST.fields_by_name['adminID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_NEWACCOUNTREQUEST.fields_by_name['admincredential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_NEWACCOUNTRESPONSE.fields_by_name['newAccountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_NEWACCOUNTRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETACCOUNTIDRESPONSE.fields_by_name['newAccountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETACCOUNTIDRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_AUTOREGISTERSLREQUEST.fields_by_name['storageLocationID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._STORAGELOCATIONID
_AUTOREGISTEREEREQUEST.fields_by_name['executionEnvironmentID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXECUTIONENVIRONMENTID
_AUTOREGISTEREEREQUEST.fields_by_name['lang'].enum_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._LANGS
_AUTOREGISTEREERESPONSE.fields_by_name['storageLocationID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._STORAGELOCATIONID
_AUTOREGISTEREERESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_UNREGISTERSTORAGELOCATIONREQUEST.fields_by_name['storageLocationID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._STORAGELOCATIONID
_UNREGISTEREXECUTIONENVIRONMENTREQUEST.fields_by_name['executionEnvironmentID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXECUTIONENVIRONMENTID
_PERFORMSETACCOUNTSREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_PERFORMSETACCOUNTSREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_PERFORMSETACCOUNTSRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_PERFORMSETOPERATIONSREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_PERFORMSETOPERATIONSREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_PERFORMSETOPERATIONSRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETACCOUNTLISTREQUEST.fields_by_name['adminID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETACCOUNTLISTREQUEST.fields_by_name['admincredential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETACCOUNTLISTRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_NEWSESSIONREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_NEWSESSIONREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_NEWSESSIONREQUEST.fields_by_name['contractIDs'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CONTRACTID
_NEWSESSIONREQUEST.fields_by_name['dataSetIDs'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATASETID
_NEWSESSIONREQUEST.fields_by_name['storeDataSet'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATASETID
_NEWSESSIONREQUEST.fields_by_name['sessionLang'].enum_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._LANGS
_NEWSESSIONRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETINFOOFSESSIONFORDSREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_GETINFOOFSESSIONFORDSRESPONSE.fields_by_name['dataSetID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATASETID
_GETINFOOFSESSIONFORDSRESPONSE.fields_by_name['dataSetIDs'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATASETID
_GETINFOOFSESSIONFORDSRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_NEWNAMESPACEREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_NEWNAMESPACEREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_NEWNAMESPACERESPONSE.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_NEWNAMESPACERESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETNAMESPACESREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETNAMESPACESREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETNAMESPACESRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_REMOVENAMESPACEREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_REMOVENAMESPACEREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETNAMESPACEIDREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETNAMESPACEIDREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETNAMESPACEIDRESPONSE.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_GETNAMESPACEIDRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETNAMESPACELANGREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETNAMESPACELANGREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETNAMESPACELANGRESPONSE.fields_by_name['language'].enum_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._LANGS
_GETNAMESPACELANGRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETOBJECTDATASETIDREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_GETOBJECTDATASETIDREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_GETOBJECTDATASETIDRESPONSE.fields_by_name['dataSetID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATASETID
_GETOBJECTDATASETIDRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_IMPORTINTERFACEREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_IMPORTINTERFACEREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_IMPORTINTERFACEREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_IMPORTINTERFACEREQUEST.fields_by_name['contractID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CONTRACTID
_IMPORTINTERFACEREQUEST.fields_by_name['interfaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._INTERFACEID
_IMPORTCONTRACTREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_IMPORTCONTRACTREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_IMPORTCONTRACTREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_IMPORTCONTRACTREQUEST.fields_by_name['contractID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CONTRACTID
_NEWDATASETREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_NEWDATASETREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_NEWDATASETRESPONSE.fields_by_name['dataSetID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATASETID
_NEWDATASETRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_REMOVEDATASETREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_REMOVEDATASETREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETDATASETIDREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETDATASETIDREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETDATASETIDRESPONSE.fields_by_name['dataSetID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATASETID
_GETDATASETIDRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETPUBLICDATASETSREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETPUBLICDATASETSREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETPUBLICDATASETSRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETACCOUNTDATASETSREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETACCOUNTDATASETSREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETACCOUNTDATASETSRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_CHECKDATASETISPUBLICREQUEST.fields_by_name['dataSetID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATASETID
_CHECKDATASETISPUBLICRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_NEWCLASSREQUEST_NEWCLASSESENTRY.containing_type = _NEWCLASSREQUEST
_NEWCLASSREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_NEWCLASSREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_NEWCLASSREQUEST.fields_by_name['language'].enum_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._LANGS
_NEWCLASSREQUEST.fields_by_name['newClasses'].message_type = _NEWCLASSREQUEST_NEWCLASSESENTRY
_NEWCLASSRESPONSE_NEWCLASSESENTRY.containing_type = _NEWCLASSRESPONSE
_NEWCLASSRESPONSE.fields_by_name['newClasses'].message_type = _NEWCLASSRESPONSE_NEWCLASSESENTRY
_NEWCLASSRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_NEWCLASSIDREQUEST_NEWCLASSESENTRY.containing_type = _NEWCLASSIDREQUEST
_NEWCLASSIDREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_NEWCLASSIDREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_NEWCLASSIDREQUEST.fields_by_name['language'].enum_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._LANGS
_NEWCLASSIDREQUEST.fields_by_name['newClasses'].message_type = _NEWCLASSIDREQUEST_NEWCLASSESENTRY
_NEWCLASSIDRESPONSE.fields_by_name['classID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._METACLASSID
_NEWCLASSIDRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_REMOVECLASSREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_REMOVECLASSREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_REMOVECLASSREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_REMOVEOPERATIONREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_REMOVEOPERATIONREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_REMOVEOPERATIONREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_REMOVEIMPLEMENTATIONREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_REMOVEIMPLEMENTATIONREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_REMOVEIMPLEMENTATIONREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_REMOVEIMPLEMENTATIONREQUEST.fields_by_name['implementationID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._IMPLEMENTATIONID
_GETOPERATIONIDREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETOPERATIONIDREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETOPERATIONIDREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_GETOPERATIONIDRESPONSE.fields_by_name['operationID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OPERATIONID
_GETOPERATIONIDRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETPROPERTYIDREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETPROPERTYIDREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETPROPERTYIDREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_GETPROPERTYIDRESPONSE.fields_by_name['propertyID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._PROPERTYID
_GETPROPERTYIDRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETCLASSIDREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETCLASSIDREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETCLASSIDREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_GETCLASSIDRESPONSE.fields_by_name['classID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._METACLASSID
_GETCLASSIDRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETCLASSINFOREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETCLASSINFOREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETCLASSINFOREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_GETCLASSINFORESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETINFOOFCLASSESINNAMESPACEREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETINFOOFCLASSESINNAMESPACEREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETINFOOFCLASSESINNAMESPACEREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_GETINFOOFCLASSESINNAMESPACERESPONSE_CLASSESINFOENTRY.containing_type = _GETINFOOFCLASSESINNAMESPACERESPONSE
_GETINFOOFCLASSESINNAMESPACERESPONSE.fields_by_name['classesInfo'].message_type = _GETINFOOFCLASSESINNAMESPACERESPONSE_CLASSESINFOENTRY
_GETINFOOFCLASSESINNAMESPACERESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETIMPORTEDCLASSESINFOINNAMESPACEREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETIMPORTEDCLASSESINFOINNAMESPACEREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETIMPORTEDCLASSESINFOINNAMESPACEREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_GETIMPORTEDCLASSESINFOINNAMESPACERESPONSE_IMPORTEDCLASSESENTRY.containing_type = _GETIMPORTEDCLASSESINFOINNAMESPACERESPONSE
_GETIMPORTEDCLASSESINFOINNAMESPACERESPONSE.fields_by_name['importedClasses'].message_type = _GETIMPORTEDCLASSESINFOINNAMESPACERESPONSE_IMPORTEDCLASSESENTRY
_GETIMPORTEDCLASSESINFOINNAMESPACERESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETCLASSIDFROMIMPORTREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETCLASSIDFROMIMPORTREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETCLASSIDFROMIMPORTREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_GETCLASSIDFROMIMPORTRESPONSE.fields_by_name['classID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._METACLASSID
_GETCLASSIDFROMIMPORTRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_NEWCONTRACTREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_NEWCONTRACTREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_NEWCONTRACTRESPONSE.fields_by_name['contractID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CONTRACTID
_NEWCONTRACTRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_REGISTERTOPUBLICCONTRACTREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_REGISTERTOPUBLICCONTRACTREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_REGISTERTOPUBLICCONTRACTREQUEST.fields_by_name['contractID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CONTRACTID
_REGISTERTOPUBLICCONTRACTOFNAMESPACEREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_REGISTERTOPUBLICCONTRACTOFNAMESPACEREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_REGISTERTOPUBLICCONTRACTOFNAMESPACEREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_REGISTERTOPUBLICCONTRACTOFNAMESPACERESPONSE.fields_by_name['contractID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CONTRACTID
_REGISTERTOPUBLICCONTRACTOFNAMESPACERESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETCONTRACTIDSOFAPPLICANTREQUEST.fields_by_name['applicantID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETCONTRACTIDSOFAPPLICANTREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETCONTRACTIDSOFAPPLICANTRESPONSE_CONTRACTSENTRY.containing_type = _GETCONTRACTIDSOFAPPLICANTRESPONSE
_GETCONTRACTIDSOFAPPLICANTRESPONSE.fields_by_name['contracts'].message_type = _GETCONTRACTIDSOFAPPLICANTRESPONSE_CONTRACTSENTRY
_GETCONTRACTIDSOFAPPLICANTRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETCONTRACTIDSOFPROVIDERREQUEST.fields_by_name['providerID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETCONTRACTIDSOFPROVIDERREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETCONTRACTIDSOFPROVIDERREQUEST.fields_by_name['namespaceIDOfProvider'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_GETCONTRACTIDSOFPROVIDERRESPONSE_CONTRACTSENTRY.containing_type = _GETCONTRACTIDSOFPROVIDERRESPONSE
_GETCONTRACTIDSOFPROVIDERRESPONSE.fields_by_name['contracts'].message_type = _GETCONTRACTIDSOFPROVIDERRESPONSE_CONTRACTSENTRY
_GETCONTRACTIDSOFPROVIDERRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETCONTRACTSOFAPPLICANTWITHPROVREQUEST.fields_by_name['applicantID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETCONTRACTSOFAPPLICANTWITHPROVREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETCONTRACTSOFAPPLICANTWITHPROVREQUEST.fields_by_name['namespaceIDOfProvider'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_GETCONTRACTSOFAPPLICANTWITHPROVRESPONSE_CONTRACTSENTRY.containing_type = _GETCONTRACTSOFAPPLICANTWITHPROVRESPONSE
_GETCONTRACTSOFAPPLICANTWITHPROVRESPONSE.fields_by_name['contracts'].message_type = _GETCONTRACTSOFAPPLICANTWITHPROVRESPONSE_CONTRACTSENTRY
_GETCONTRACTSOFAPPLICANTWITHPROVRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_NEWDATACONTRACTREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_NEWDATACONTRACTREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_NEWDATACONTRACTRESPONSE.fields_by_name['dataContractID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACONTRACTID
_NEWDATACONTRACTRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_REGISTERTOPUBLICDATACONTRACTREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_REGISTERTOPUBLICDATACONTRACTREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_REGISTERTOPUBLICDATACONTRACTREQUEST.fields_by_name['dataContractID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACONTRACTID
_GETDATACONTRACTIDSOFAPPLICANTREQUEST.fields_by_name['applicantID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETDATACONTRACTIDSOFAPPLICANTREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETDATACONTRACTIDSOFAPPLICANTRESPONSE_DATACONTRACTSENTRY.containing_type = _GETDATACONTRACTIDSOFAPPLICANTRESPONSE
_GETDATACONTRACTIDSOFAPPLICANTRESPONSE.fields_by_name['datacontracts'].message_type = _GETDATACONTRACTIDSOFAPPLICANTRESPONSE_DATACONTRACTSENTRY
_GETDATACONTRACTIDSOFAPPLICANTRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETDATACONTRACTIDSOFPROVIDERREQUEST.fields_by_name['providerID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETDATACONTRACTIDSOFPROVIDERREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETDATACONTRACTIDSOFPROVIDERREQUEST.fields_by_name['dataSetIDOfProvider'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATASETID
_GETDATACONTRACTIDSOFPROVIDERRESPONSE_DATACONTRACTSENTRY.containing_type = _GETDATACONTRACTIDSOFPROVIDERRESPONSE
_GETDATACONTRACTIDSOFPROVIDERRESPONSE.fields_by_name['datacontracts'].message_type = _GETDATACONTRACTIDSOFPROVIDERRESPONSE_DATACONTRACTSENTRY
_GETDATACONTRACTIDSOFPROVIDERRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETDATACONTRACTINFOOFAPPLICANTWITHPROVREQUEST.fields_by_name['applicantID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETDATACONTRACTINFOOFAPPLICANTWITHPROVREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETDATACONTRACTINFOOFAPPLICANTWITHPROVREQUEST.fields_by_name['dataSetIDOfProvider'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATASETID
_GETDATACONTRACTINFOOFAPPLICANTWITHPROVRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_NEWINTERFACEREQUEST.fields_by_name['applicantID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_NEWINTERFACEREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_NEWINTERFACERESPONSE.fields_by_name['interfaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._INTERFACEID
_NEWINTERFACERESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETINTERFACEINFOREQUEST.fields_by_name['applicantID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETINTERFACEINFOREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETINTERFACEINFOREQUEST.fields_by_name['interfaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._INTERFACEID
_GETINTERFACEINFORESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_REMOVEINTERFACEREQUEST.fields_by_name['applicantID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_REMOVEINTERFACEREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_REMOVEINTERFACEREQUEST.fields_by_name['namespaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._NAMESPACEID
_REMOVEINTERFACEREQUEST.fields_by_name['interfaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._INTERFACEID
_GETSTORAGELOCATIONFORDSREQUEST.fields_by_name['storageLocationID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._STORAGELOCATIONID
_GETSTORAGELOCATIONFORDSRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETEXECUTIONENVIRONMENTFORDSREQUEST.fields_by_name['execEnvID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXECUTIONENVIRONMENTID
_GETEXECUTIONENVIRONMENTFORDSRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETDATACLAYIDRESPONSE.fields_by_name['dataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_GETDATACLAYIDRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_REGISTEREXTERNALDATACLAYRESPONSE.fields_by_name['extDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_REGISTEREXTERNALDATACLAYRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_NOTIFYREGISTRATIONOFEXTERNALDATACLAYREQUEST.fields_by_name['extDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_NOTIFYREGISTRATIONOFEXTERNALDATACLAYRESPONSE.fields_by_name['extDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_NOTIFYREGISTRATIONOFEXTERNALDATACLAYRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETEXTERNALDATACLAYIDRESPONSE.fields_by_name['extDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_GETEXTERNALDATACLAYIDRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETEXTDATACLAYINFOREQUEST.fields_by_name['extDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_GETEXTDATACLAYINFORESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_FEDERATEOBJECTREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_FEDERATEOBJECTREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_FEDERATEOBJECTREQUEST.fields_by_name['extDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_UNFEDERATEOBJECTREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_UNFEDERATEOBJECTREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_UNFEDERATEOBJECTREQUEST.fields_by_name['extDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_UNFEDERATEOBJECTWITHALLDCSREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_UNFEDERATEOBJECTWITHALLDCSREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_MIGRATEFEDERATEDOBJECTSREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_MIGRATEFEDERATEDOBJECTSREQUEST.fields_by_name['externalOriginDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_MIGRATEFEDERATEDOBJECTSREQUEST.fields_by_name['externalDestinationDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_FEDERATEALLOBJECTSREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_FEDERATEALLOBJECTSREQUEST.fields_by_name['externalDestinationDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_UNFEDERATEALLOBJECTSREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_UNFEDERATEALLOBJECTSREQUEST.fields_by_name['extDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_UNFEDERATEALLOBJECTSWITHALLDCSREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_NOTIFYFEDERATEDOBJECTSREQUEST_OBJECTSINFOENTRY.containing_type = _NOTIFYFEDERATEDOBJECTSREQUEST
_NOTIFYFEDERATEDOBJECTSREQUEST_FEDERATEDOBJECTSENTRY.fields_by_name['value'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SERIALIZEDPARAMETERSORRETURN
_NOTIFYFEDERATEDOBJECTSREQUEST_FEDERATEDOBJECTSENTRY.containing_type = _NOTIFYFEDERATEDOBJECTSREQUEST
_NOTIFYFEDERATEDOBJECTSREQUEST.fields_by_name['srcDcID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_NOTIFYFEDERATEDOBJECTSREQUEST.fields_by_name['objectsInfo'].message_type = _NOTIFYFEDERATEDOBJECTSREQUEST_OBJECTSINFOENTRY
_NOTIFYFEDERATEDOBJECTSREQUEST.fields_by_name['federatedObjects'].message_type = _NOTIFYFEDERATEDOBJECTSREQUEST_FEDERATEDOBJECTSENTRY
_NOTIFYUNFEDERATEDOBJECTSREQUEST.fields_by_name['srcDcID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_NOTIFYUNFEDERATEDOBJECTSREQUEST.fields_by_name['objectsIDs'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_CHECKOBJECTFEDERATEDWITHDATACLAYINSTANCEREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_CHECKOBJECTFEDERATEDWITHDATACLAYINSTANCEREQUEST.fields_by_name['extDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_CHECKOBJECTFEDERATEDWITHDATACLAYINSTANCERESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_CHECKOBJECTFEDERATEDREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_CHECKOBJECTFEDERATEDRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETDATACLAYSOBJECTISFEDERATEDWITHREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_GETDATACLAYSOBJECTISFEDERATEDWITHRESPONSE.fields_by_name['extDataClayIDs'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_GETDATACLAYSOBJECTISFEDERATEDWITHRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETEXTERNALSOURCEDATACLAYOFOBJECTREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_GETEXTERNALSOURCEDATACLAYOFOBJECTRESPONSE.fields_by_name['extDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_GETEXTERNALSOURCEDATACLAYOFOBJECTRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_REGISTEROBJECTFORGCREQUEST.fields_by_name['regInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._REGISTRATIONINFO
_REGISTEROBJECTFORGCREQUEST.fields_by_name['backendID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXECUTIONENVIRONMENTID
_REGISTEROBJECTREQUEST.fields_by_name['regInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._REGISTRATIONINFO
_REGISTEROBJECTREQUEST.fields_by_name['backendID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXECUTIONENVIRONMENTID
_REGISTEROBJECTREQUEST.fields_by_name['lang'].enum_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._LANGS
_GETEXECUTIONENVIRONMENTSINFOREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_GETEXECUTIONENVIRONMENTSINFOREQUEST.fields_by_name['execEnvLang'].enum_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._LANGS
_GETEXECUTIONENVIRONMENTSINFORESPONSE_EXECENVSENTRY.containing_type = _GETEXECUTIONENVIRONMENTSINFORESPONSE
_GETEXECUTIONENVIRONMENTSINFORESPONSE.fields_by_name['execEnvs'].message_type = _GETEXECUTIONENVIRONMENTSINFORESPONSE_EXECENVSENTRY
_GETEXECUTIONENVIRONMENTSINFORESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETEXECUTIONENVIRONMENTSNAMESREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETEXECUTIONENVIRONMENTSNAMESREQUEST.fields_by_name['credential'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETEXECUTIONENVIRONMENTSNAMESREQUEST.fields_by_name['execEnvLang'].enum_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._LANGS
_GETEXECUTIONENVIRONMENTSNAMESRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETOBJECTINFOREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_GETOBJECTINFOREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_GETOBJECTINFORESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETOBJECTFROMALIASREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_GETOBJECTFROMALIASRESPONSE.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_GETOBJECTFROMALIASRESPONSE.fields_by_name['classID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._METACLASSID
_GETOBJECTFROMALIASRESPONSE.fields_by_name['hint'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXECUTIONENVIRONMENTID
_GETOBJECTFROMALIASRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_ADDALIASREQUEST.fields_by_name['objectIDToHaveAlias'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_DELETEALIASREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_GETOBJECTSMETADATAINFOOFCLASSFORNMREQUEST.fields_by_name['classID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._METACLASSID
_GETOBJECTSMETADATAINFOOFCLASSFORNMRESPONSE_MDATAINFOENTRY.containing_type = _GETOBJECTSMETADATAINFOOFCLASSFORNMRESPONSE
_GETOBJECTSMETADATAINFOOFCLASSFORNMRESPONSE.fields_by_name['mdataInfo'].message_type = _GETOBJECTSMETADATAINFOOFCLASSFORNMRESPONSE_MDATAINFOENTRY
_GETOBJECTSMETADATAINFOOFCLASSFORNMRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_SETDATASETIDREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_SETDATASETIDREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_SETDATASETIDREQUEST.fields_by_name['datasetID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATASETID
_SETDATASETIDFROMGARBAGECOLLECTORREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_SETDATASETIDFROMGARBAGECOLLECTORREQUEST.fields_by_name['datasetID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATASETID
_NEWVERSIONREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_NEWVERSIONREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_NEWVERSIONREQUEST.fields_by_name['optDestBackendID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXECUTIONENVIRONMENTID
_NEWVERSIONRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_CONSOLIDATEVERSIONREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_NEWREPLICAREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_NEWREPLICAREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_NEWREPLICAREQUEST.fields_by_name['destBackendID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXECUTIONENVIRONMENTID
_NEWREPLICARESPONSE.fields_by_name['destBackendID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXECUTIONENVIRONMENTID
_NEWREPLICARESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_MOVEOBJECTREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_MOVEOBJECTREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_MOVEOBJECTREQUEST.fields_by_name['srcBackendID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXECUTIONENVIRONMENTID
_MOVEOBJECTREQUEST.fields_by_name['destBackendID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXECUTIONENVIRONMENTID
_MOVEOBJECTRESPONSE.fields_by_name['objectIDs'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_MOVEOBJECTRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_SETOBJECTREADONLYREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_SETOBJECTREADONLYREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_SETOBJECTREADWRITEREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_SETOBJECTREADWRITEREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_GETMETADATABYOIDREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_GETMETADATABYOIDREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_GETMETADATABYOIDRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETMETADATABYOIDFORDSREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_GETMETADATABYOIDFORDSRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_EXECUTEIMPLEMENTATIONREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_EXECUTEIMPLEMENTATIONREQUEST.fields_by_name['operationID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OPERATIONID
_EXECUTEIMPLEMENTATIONREQUEST.fields_by_name['implementationID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._IMPLEMENTATIONID
_EXECUTEIMPLEMENTATIONREQUEST.fields_by_name['contractID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CONTRACTID
_EXECUTEIMPLEMENTATIONREQUEST.fields_by_name['interfaceID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._INTERFACEID
_EXECUTEIMPLEMENTATIONREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_EXECUTEIMPLEMENTATIONREQUEST.fields_by_name['params'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SERIALIZEDPARAMETERSORRETURN
_EXECUTEIMPLEMENTATIONRESPONSE.fields_by_name['ret'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SERIALIZEDPARAMETERSORRETURN
_EXECUTEIMPLEMENTATIONRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_EXECUTEMETHODONTARGETREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_EXECUTEMETHODONTARGETREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_EXECUTEMETHODONTARGETREQUEST.fields_by_name['params'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SERIALIZEDPARAMETERSORRETURN
_EXECUTEMETHODONTARGETREQUEST.fields_by_name['targetBackendID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXECUTIONENVIRONMENTID
_EXECUTEMETHODONTARGETRESPONSE.fields_by_name['ret'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SERIALIZEDPARAMETERSORRETURN
_EXECUTEMETHODONTARGETRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_SYNCHRONIZEFEDERATEDOBJECTREQUEST.fields_by_name['extDataClayID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._DATACLAYINSTANCEID
_SYNCHRONIZEFEDERATEDOBJECTREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_SYNCHRONIZEFEDERATEDOBJECTREQUEST.fields_by_name['implementationID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._IMPLEMENTATIONID
_SYNCHRONIZEFEDERATEDOBJECTREQUEST.fields_by_name['params'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SERIALIZEDPARAMETERSORRETURN
_GETSTUBSREQUEST.fields_by_name['applicantAccountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETSTUBSREQUEST.fields_by_name['credentials'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETSTUBSREQUEST.fields_by_name['language'].enum_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._LANGS
_GETSTUBSREQUEST.fields_by_name['contractIDs'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CONTRACTID
_GETSTUBSRESPONSE_STUBSENTRY.containing_type = _GETSTUBSRESPONSE
_GETSTUBSRESPONSE.fields_by_name['stubs'].message_type = _GETSTUBSRESPONSE_STUBSENTRY
_GETSTUBSRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETBABELSTUBSREQUEST.fields_by_name['accountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETBABELSTUBSREQUEST.fields_by_name['credentials'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETBABELSTUBSREQUEST.fields_by_name['contractIDs'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CONTRACTID
_GETBABELSTUBSRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETCLASSNAMEFORDSREQUEST.fields_by_name['classID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._METACLASSID
_GETCLASSNAMEFORDSRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_GETCLASSNAMEANDNAMESPACEFORDSREQUEST.fields_by_name['classID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._METACLASSID
_GETCLASSNAMEANDNAMESPACEFORDSRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_REGISTERECAREQUEST.fields_by_name['applicantAccountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_REGISTERECAREQUEST.fields_by_name['credentials'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETCONTRACTIDOFDATACLAYPROVIDERREQUEST.fields_by_name['applicantAccountID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._ACCOUNTID
_GETCONTRACTIDOFDATACLAYPROVIDERREQUEST.fields_by_name['credentials'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CREDENTIAL
_GETCONTRACTIDOFDATACLAYPROVIDERRESPONSE.fields_by_name['contractID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._CONTRACTID
_GETCONTRACTIDOFDATACLAYPROVIDERRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_CLOSESESSIONREQUEST.fields_by_name['sessionID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._SESSIONID
_UNREGISTEROBJECTSREQUEST.fields_by_name['objectsToUnregister'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_ISPREFETCHINGENABLEDRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
_OBJECTEXISTSINDATACLAYREQUEST.fields_by_name['objectID'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._OBJECTID
_OBJECTEXISTSINDATACLAYRESPONSE.fields_by_name['excInfo'].message_type = dataclay_dot_communication_dot_grpc_dot_messages_dot_common_dot_common__messages__pb2._EXCEPTIONINFO
DESCRIPTOR.message_types_by_name['NewAccountNoAdminRequest'] = _NEWACCOUNTNOADMINREQUEST
DESCRIPTOR.message_types_by_name['NewAccountRequest'] = _NEWACCOUNTREQUEST
DESCRIPTOR.message_types_by_name['NewAccountResponse'] = _NEWACCOUNTRESPONSE
DESCRIPTOR.message_types_by_name['PublishAddressRequest'] = _PUBLISHADDRESSREQUEST
DESCRIPTOR.message_types_by_name['GetAccountIDRequest'] = _GETACCOUNTIDREQUEST
DESCRIPTOR.message_types_by_name['GetAccountIDResponse'] = _GETACCOUNTIDRESPONSE
DESCRIPTOR.message_types_by_name['AutoRegisterSLRequest'] = _AUTOREGISTERSLREQUEST
DESCRIPTOR.message_types_by_name['AutoRegisterEERequest'] = _AUTOREGISTEREEREQUEST
DESCRIPTOR.message_types_by_name['AutoRegisterEEResponse'] = _AUTOREGISTEREERESPONSE
DESCRIPTOR.message_types_by_name['UnregisterStorageLocationRequest'] = _UNREGISTERSTORAGELOCATIONREQUEST
DESCRIPTOR.message_types_by_name['UnregisterExecutionEnvironmentRequest'] = _UNREGISTEREXECUTIONENVIRONMENTREQUEST
DESCRIPTOR.message_types_by_name['PerformSetAccountsRequest'] = _PERFORMSETACCOUNTSREQUEST
DESCRIPTOR.message_types_by_name['PerformSetAccountsResponse'] = _PERFORMSETACCOUNTSRESPONSE
DESCRIPTOR.message_types_by_name['PerformSetOperationsRequest'] = _PERFORMSETOPERATIONSREQUEST
DESCRIPTOR.message_types_by_name['PerformSetOperationsResponse'] = _PERFORMSETOPERATIONSRESPONSE
DESCRIPTOR.message_types_by_name['GetAccountListRequest'] = _GETACCOUNTLISTREQUEST
DESCRIPTOR.message_types_by_name['GetAccountListResponse'] = _GETACCOUNTLISTRESPONSE
DESCRIPTOR.message_types_by_name['NewSessionRequest'] = _NEWSESSIONREQUEST
DESCRIPTOR.message_types_by_name['NewSessionResponse'] = _NEWSESSIONRESPONSE
DESCRIPTOR.message_types_by_name['GetInfoOfSessionForDSRequest'] = _GETINFOOFSESSIONFORDSREQUEST
DESCRIPTOR.message_types_by_name['GetInfoOfSessionForDSResponse'] = _GETINFOOFSESSIONFORDSRESPONSE
DESCRIPTOR.message_types_by_name['NewNamespaceRequest'] = _NEWNAMESPACEREQUEST
DESCRIPTOR.message_types_by_name['NewNamespaceResponse'] = _NEWNAMESPACERESPONSE
DESCRIPTOR.message_types_by_name['GetNamespacesRequest'] = _GETNAMESPACESREQUEST
DESCRIPTOR.message_types_by_name['GetNamespacesResponse'] = _GETNAMESPACESRESPONSE
DESCRIPTOR.message_types_by_name['RemoveNamespaceRequest'] = _REMOVENAMESPACEREQUEST
DESCRIPTOR.message_types_by_name['GetNamespaceIDRequest'] = _GETNAMESPACEIDREQUEST
DESCRIPTOR.message_types_by_name['GetNamespaceIDResponse'] = _GETNAMESPACEIDRESPONSE
DESCRIPTOR.message_types_by_name['GetNamespaceLangRequest'] = _GETNAMESPACELANGREQUEST
DESCRIPTOR.message_types_by_name['GetNamespaceLangResponse'] = _GETNAMESPACELANGRESPONSE
DESCRIPTOR.message_types_by_name['GetObjectDataSetIDRequest'] = _GETOBJECTDATASETIDREQUEST
DESCRIPTOR.message_types_by_name['GetObjectDataSetIDResponse'] = _GETOBJECTDATASETIDRESPONSE
DESCRIPTOR.message_types_by_name['ImportInterfaceRequest'] = _IMPORTINTERFACEREQUEST
DESCRIPTOR.message_types_by_name['ImportContractRequest'] = _IMPORTCONTRACTREQUEST
DESCRIPTOR.message_types_by_name['NewDataSetRequest'] = _NEWDATASETREQUEST
DESCRIPTOR.message_types_by_name['NewDataSetResponse'] = _NEWDATASETRESPONSE
DESCRIPTOR.message_types_by_name['RemoveDataSetRequest'] = _REMOVEDATASETREQUEST
DESCRIPTOR.message_types_by_name['GetDataSetIDRequest'] = _GETDATASETIDREQUEST
DESCRIPTOR.message_types_by_name['GetDataSetIDResponse'] = _GETDATASETIDRESPONSE
DESCRIPTOR.message_types_by_name['GetPublicDataSetsRequest'] = _GETPUBLICDATASETSREQUEST
DESCRIPTOR.message_types_by_name['GetPublicDataSetsResponse'] = _GETPUBLICDATASETSRESPONSE
DESCRIPTOR.message_types_by_name['GetAccountDataSetsRequest'] = _GETACCOUNTDATASETSREQUEST
DESCRIPTOR.message_types_by_name['GetAccountDataSetsResponse'] = _GETACCOUNTDATASETSRESPONSE
DESCRIPTOR.message_types_by_name['CheckDataSetIsPublicRequest'] = _CHECKDATASETISPUBLICREQUEST
DESCRIPTOR.message_types_by_name['CheckDataSetIsPublicResponse'] = _CHECKDATASETISPUBLICRESPONSE
DESCRIPTOR.message_types_by_name['NewClassRequest'] = _NEWCLASSREQUEST
DESCRIPTOR.message_types_by_name['NewClassResponse'] = _NEWCLASSRESPONSE
DESCRIPTOR.message_types_by_name['NewClassIDRequest'] = _NEWCLASSIDREQUEST
DESCRIPTOR.message_types_by_name['NewClassIDResponse'] = _NEWCLASSIDRESPONSE
DESCRIPTOR.message_types_by_name['RemoveClassRequest'] = _REMOVECLASSREQUEST
DESCRIPTOR.message_types_by_name['RemoveOperationRequest'] = _REMOVEOPERATIONREQUEST
DESCRIPTOR.message_types_by_name['RemoveImplementationRequest'] = _REMOVEIMPLEMENTATIONREQUEST
DESCRIPTOR.message_types_by_name['GetOperationIDRequest'] = _GETOPERATIONIDREQUEST
DESCRIPTOR.message_types_by_name['GetOperationIDResponse'] = _GETOPERATIONIDRESPONSE
DESCRIPTOR.message_types_by_name['GetPropertyIDRequest'] = _GETPROPERTYIDREQUEST
DESCRIPTOR.message_types_by_name['GetPropertyIDResponse'] = _GETPROPERTYIDRESPONSE
DESCRIPTOR.message_types_by_name['GetClassIDRequest'] = _GETCLASSIDREQUEST
DESCRIPTOR.message_types_by_name['GetClassIDResponse'] = _GETCLASSIDRESPONSE
DESCRIPTOR.message_types_by_name['GetClassInfoRequest'] = _GETCLASSINFOREQUEST
DESCRIPTOR.message_types_by_name['GetClassInfoResponse'] = _GETCLASSINFORESPONSE
DESCRIPTOR.message_types_by_name['GetInfoOfClassesInNamespaceRequest'] = _GETINFOOFCLASSESINNAMESPACEREQUEST
DESCRIPTOR.message_types_by_name['GetInfoOfClassesInNamespaceResponse'] = _GETINFOOFCLASSESINNAMESPACERESPONSE
DESCRIPTOR.message_types_by_name['GetImportedClassesInfoInNamespaceRequest'] = _GETIMPORTEDCLASSESINFOINNAMESPACEREQUEST
DESCRIPTOR.message_types_by_name['GetImportedClassesInfoInNamespaceResponse'] = _GETIMPORTEDCLASSESINFOINNAMESPACERESPONSE
DESCRIPTOR.message_types_by_name['GetClassIDFromImportRequest'] = _GETCLASSIDFROMIMPORTREQUEST
DESCRIPTOR.message_types_by_name['GetClassIDFromImportResponse'] = _GETCLASSIDFROMIMPORTRESPONSE
DESCRIPTOR.message_types_by_name['NewContractRequest'] = _NEWCONTRACTREQUEST
DESCRIPTOR.message_types_by_name['NewContractResponse'] = _NEWCONTRACTRESPONSE
DESCRIPTOR.message_types_by_name['RegisterToPublicContractRequest'] = _REGISTERTOPUBLICCONTRACTREQUEST
DESCRIPTOR.message_types_by_name['RegisterToPublicContractOfNamespaceRequest'] = _REGISTERTOPUBLICCONTRACTOFNAMESPACEREQUEST
DESCRIPTOR.message_types_by_name['RegisterToPublicContractOfNamespaceResponse'] = _REGISTERTOPUBLICCONTRACTOFNAMESPACERESPONSE
DESCRIPTOR.message_types_by_name['GetContractIDsOfApplicantRequest'] = _GETCONTRACTIDSOFAPPLICANTREQUEST
DESCRIPTOR.message_types_by_name['GetContractIDsOfApplicantResponse'] = _GETCONTRACTIDSOFAPPLICANTRESPONSE
DESCRIPTOR.message_types_by_name['GetContractIDsOfProviderRequest'] = _GETCONTRACTIDSOFPROVIDERREQUEST
DESCRIPTOR.message_types_by_name['GetContractIDsOfProviderResponse'] = _GETCONTRACTIDSOFPROVIDERRESPONSE
DESCRIPTOR.message_types_by_name['GetContractsOfApplicantWithProvRequest'] = _GETCONTRACTSOFAPPLICANTWITHPROVREQUEST
DESCRIPTOR.message_types_by_name['GetContractsOfApplicantWithProvResponse'] = _GETCONTRACTSOFAPPLICANTWITHPROVRESPONSE
DESCRIPTOR.message_types_by_name['NewDataContractRequest'] = _NEWDATACONTRACTREQUEST
DESCRIPTOR.message_types_by_name['NewDataContractResponse'] = _NEWDATACONTRACTRESPONSE
DESCRIPTOR.message_types_by_name['RegisterToPublicDataContractRequest'] = _REGISTERTOPUBLICDATACONTRACTREQUEST
DESCRIPTOR.message_types_by_name['GetDataContractIDsOfApplicantRequest'] = _GETDATACONTRACTIDSOFAPPLICANTREQUEST
DESCRIPTOR.message_types_by_name['GetDataContractIDsOfApplicantResponse'] = _GETDATACONTRACTIDSOFAPPLICANTRESPONSE
DESCRIPTOR.message_types_by_name['GetDataContractIDsOfProviderRequest'] = _GETDATACONTRACTIDSOFPROVIDERREQUEST
DESCRIPTOR.message_types_by_name['GetDataContractIDsOfProviderResponse'] = _GETDATACONTRACTIDSOFPROVIDERRESPONSE
DESCRIPTOR.message_types_by_name['GetDataContractInfoOfApplicantWithProvRequest'] = _GETDATACONTRACTINFOOFAPPLICANTWITHPROVREQUEST
DESCRIPTOR.message_types_by_name['GetDataContractInfoOfApplicantWithProvResponse'] = _GETDATACONTRACTINFOOFAPPLICANTWITHPROVRESPONSE
DESCRIPTOR.message_types_by_name['NewInterfaceRequest'] = _NEWINTERFACEREQUEST
DESCRIPTOR.message_types_by_name['NewInterfaceResponse'] = _NEWINTERFACERESPONSE
DESCRIPTOR.message_types_by_name['GetInterfaceInfoRequest'] = _GETINTERFACEINFOREQUEST
DESCRIPTOR.message_types_by_name['GetInterfaceInfoResponse'] = _GETINTERFACEINFORESPONSE
DESCRIPTOR.message_types_by_name['RemoveInterfaceRequest'] = _REMOVEINTERFACEREQUEST
DESCRIPTOR.message_types_by_name['GetStorageLocationForDSRequest'] = _GETSTORAGELOCATIONFORDSREQUEST
DESCRIPTOR.message_types_by_name['GetStorageLocationForDSResponse'] = _GETSTORAGELOCATIONFORDSRESPONSE
DESCRIPTOR.message_types_by_name['GetExecutionEnvironmentForDSRequest'] = _GETEXECUTIONENVIRONMENTFORDSREQUEST
DESCRIPTOR.message_types_by_name['GetExecutionEnvironmentForDSResponse'] = _GETEXECUTIONENVIRONMENTFORDSRESPONSE
DESCRIPTOR.message_types_by_name['GetDataClayIDResponse'] = _GETDATACLAYIDRESPONSE
DESCRIPTOR.message_types_by_name['RegisterExternalDataClayRequest'] = _REGISTEREXTERNALDATACLAYREQUEST
DESCRIPTOR.message_types_by_name['RegisterExternalDataClayResponse'] = _REGISTEREXTERNALDATACLAYRESPONSE
DESCRIPTOR.message_types_by_name['NotifyRegistrationOfExternalDataClayRequest'] = _NOTIFYREGISTRATIONOFEXTERNALDATACLAYREQUEST
DESCRIPTOR.message_types_by_name['NotifyRegistrationOfExternalDataClayResponse'] = _NOTIFYREGISTRATIONOFEXTERNALDATACLAYRESPONSE
DESCRIPTOR.message_types_by_name['GetExternalDataclayIDRequest'] = _GETEXTERNALDATACLAYIDREQUEST
DESCRIPTOR.message_types_by_name['GetExternalDataclayIDResponse'] = _GETEXTERNALDATACLAYIDRESPONSE
DESCRIPTOR.message_types_by_name['GetExtDataClayInfoRequest'] = _GETEXTDATACLAYINFOREQUEST
DESCRIPTOR.message_types_by_name['GetExtDataClayInfoResponse'] = _GETEXTDATACLAYINFORESPONSE
DESCRIPTOR.message_types_by_name['FederateObjectRequest'] = _FEDERATEOBJECTREQUEST
DESCRIPTOR.message_types_by_name['UnfederateObjectRequest'] = _UNFEDERATEOBJECTREQUEST
DESCRIPTOR.message_types_by_name['UnfederateObjectWithAllDCsRequest'] = _UNFEDERATEOBJECTWITHALLDCSREQUEST
DESCRIPTOR.message_types_by_name['MigrateFederatedObjectsRequest'] = _MIGRATEFEDERATEDOBJECTSREQUEST
DESCRIPTOR.message_types_by_name['FederateAllObjectsRequest'] = _FEDERATEALLOBJECTSREQUEST
DESCRIPTOR.message_types_by_name['UnfederateAllObjectsRequest'] = _UNFEDERATEALLOBJECTSREQUEST
DESCRIPTOR.message_types_by_name['UnfederateAllObjectsWithAllDCsRequest'] = _UNFEDERATEALLOBJECTSWITHALLDCSREQUEST
DESCRIPTOR.message_types_by_name['NotifyFederatedObjectsRequest'] = _NOTIFYFEDERATEDOBJECTSREQUEST
DESCRIPTOR.message_types_by_name['NotifyUnfederatedObjectsRequest'] = _NOTIFYUNFEDERATEDOBJECTSREQUEST
DESCRIPTOR.message_types_by_name['CheckObjectFederatedWithDataClayInstanceRequest'] = _CHECKOBJECTFEDERATEDWITHDATACLAYINSTANCEREQUEST
DESCRIPTOR.message_types_by_name['CheckObjectFederatedWithDataClayInstanceResponse'] = _CHECKOBJECTFEDERATEDWITHDATACLAYINSTANCERESPONSE
DESCRIPTOR.message_types_by_name['CheckObjectFederatedRequest'] = _CHECKOBJECTFEDERATEDREQUEST
DESCRIPTOR.message_types_by_name['CheckObjectFederatedResponse'] = _CHECKOBJECTFEDERATEDRESPONSE
DESCRIPTOR.message_types_by_name['GetDataClaysObjectIsFederatedWithRequest'] = _GETDATACLAYSOBJECTISFEDERATEDWITHREQUEST
DESCRIPTOR.message_types_by_name['GetDataClaysObjectIsFederatedWithResponse'] = _GETDATACLAYSOBJECTISFEDERATEDWITHRESPONSE
DESCRIPTOR.message_types_by_name['GetExternalSourceDataClayOfObjectRequest'] = _GETEXTERNALSOURCEDATACLAYOFOBJECTREQUEST
DESCRIPTOR.message_types_by_name['GetExternalSourceDataClayOfObjectResponse'] = _GETEXTERNALSOURCEDATACLAYOFOBJECTRESPONSE
DESCRIPTOR.message_types_by_name['RegisterObjectForGCRequest'] = _REGISTEROBJECTFORGCREQUEST
DESCRIPTOR.message_types_by_name['RegisterObjectRequest'] = _REGISTEROBJECTREQUEST
DESCRIPTOR.message_types_by_name['GetExecutionEnvironmentsInfoRequest'] = _GETEXECUTIONENVIRONMENTSINFOREQUEST
DESCRIPTOR.message_types_by_name['GetExecutionEnvironmentsInfoResponse'] = _GETEXECUTIONENVIRONMENTSINFORESPONSE
DESCRIPTOR.message_types_by_name['GetExecutionEnvironmentsNamesRequest'] = _GETEXECUTIONENVIRONMENTSNAMESREQUEST
DESCRIPTOR.message_types_by_name['GetExecutionEnvironmentsNamesResponse'] = _GETEXECUTIONENVIRONMENTSNAMESRESPONSE
DESCRIPTOR.message_types_by_name['GetObjectInfoRequest'] = _GETOBJECTINFOREQUEST
DESCRIPTOR.message_types_by_name['GetObjectInfoResponse'] = _GETOBJECTINFORESPONSE
DESCRIPTOR.message_types_by_name['GetObjectFromAliasRequest'] = _GETOBJECTFROMALIASREQUEST
DESCRIPTOR.message_types_by_name['GetObjectFromAliasResponse'] = _GETOBJECTFROMALIASRESPONSE
DESCRIPTOR.message_types_by_name['AddAliasRequest'] = _ADDALIASREQUEST
DESCRIPTOR.message_types_by_name['DeleteAliasRequest'] = _DELETEALIASREQUEST
DESCRIPTOR.message_types_by_name['GetObjectsMetaDataInfoOfClassForNMRequest'] = _GETOBJECTSMETADATAINFOOFCLASSFORNMREQUEST
DESCRIPTOR.message_types_by_name['GetObjectsMetaDataInfoOfClassForNMResponse'] = _GETOBJECTSMETADATAINFOOFCLASSFORNMRESPONSE
DESCRIPTOR.message_types_by_name['SetDataSetIDRequest'] = _SETDATASETIDREQUEST
DESCRIPTOR.message_types_by_name['SetDataSetIDFromGarbageCollectorRequest'] = _SETDATASETIDFROMGARBAGECOLLECTORREQUEST
DESCRIPTOR.message_types_by_name['NewVersionRequest'] = _NEWVERSIONREQUEST
DESCRIPTOR.message_types_by_name['NewVersionResponse'] = _NEWVERSIONRESPONSE
DESCRIPTOR.message_types_by_name['ConsolidateVersionRequest'] = _CONSOLIDATEVERSIONREQUEST
DESCRIPTOR.message_types_by_name['NewReplicaRequest'] = _NEWREPLICAREQUEST
DESCRIPTOR.message_types_by_name['NewReplicaResponse'] = _NEWREPLICARESPONSE
DESCRIPTOR.message_types_by_name['MoveObjectRequest'] = _MOVEOBJECTREQUEST
DESCRIPTOR.message_types_by_name['MoveObjectResponse'] = _MOVEOBJECTRESPONSE
DESCRIPTOR.message_types_by_name['SetObjectReadOnlyRequest'] = _SETOBJECTREADONLYREQUEST
DESCRIPTOR.message_types_by_name['SetObjectReadWriteRequest'] = _SETOBJECTREADWRITEREQUEST
DESCRIPTOR.message_types_by_name['GetMetadataByOIDRequest'] = _GETMETADATABYOIDREQUEST
DESCRIPTOR.message_types_by_name['GetMetadataByOIDResponse'] = _GETMETADATABYOIDRESPONSE
DESCRIPTOR.message_types_by_name['GetMetadataByOIDForDSRequest'] = _GETMETADATABYOIDFORDSREQUEST
DESCRIPTOR.message_types_by_name['GetMetadataByOIDForDSResponse'] = _GETMETADATABYOIDFORDSRESPONSE
DESCRIPTOR.message_types_by_name['ExecuteImplementationRequest'] = _EXECUTEIMPLEMENTATIONREQUEST
DESCRIPTOR.message_types_by_name['ExecuteImplementationResponse'] = _EXECUTEIMPLEMENTATIONRESPONSE
DESCRIPTOR.message_types_by_name['ExecuteMethodOnTargetRequest'] = _EXECUTEMETHODONTARGETREQUEST
DESCRIPTOR.message_types_by_name['ExecuteMethodOnTargetResponse'] = _EXECUTEMETHODONTARGETRESPONSE
DESCRIPTOR.message_types_by_name['SynchronizeFederatedObjectRequest'] = _SYNCHRONIZEFEDERATEDOBJECTREQUEST
DESCRIPTOR.message_types_by_name['GetStubsRequest'] = _GETSTUBSREQUEST
DESCRIPTOR.message_types_by_name['GetStubsResponse'] = _GETSTUBSRESPONSE
DESCRIPTOR.message_types_by_name['GetBabelStubsRequest'] = _GETBABELSTUBSREQUEST
DESCRIPTOR.message_types_by_name['GetBabelStubsResponse'] = _GETBABELSTUBSRESPONSE
DESCRIPTOR.message_types_by_name['GetClassNameForDSRequest'] = _GETCLASSNAMEFORDSREQUEST
DESCRIPTOR.message_types_by_name['GetClassNameForDSResponse'] = _GETCLASSNAMEFORDSRESPONSE
DESCRIPTOR.message_types_by_name['GetClassNameAndNamespaceForDSRequest'] = _GETCLASSNAMEANDNAMESPACEFORDSREQUEST
DESCRIPTOR.message_types_by_name['GetClassNameAndNamespaceForDSResponse'] = _GETCLASSNAMEANDNAMESPACEFORDSRESPONSE
DESCRIPTOR.message_types_by_name['RegisterECARequest'] = _REGISTERECAREQUEST
DESCRIPTOR.message_types_by_name['AdviseEventRequest'] = _ADVISEEVENTREQUEST
DESCRIPTOR.message_types_by_name['GetContractIDOfDataClayProviderRequest'] = _GETCONTRACTIDOFDATACLAYPROVIDERREQUEST
DESCRIPTOR.message_types_by_name['GetContractIDOfDataClayProviderResponse'] = _GETCONTRACTIDOFDATACLAYPROVIDERRESPONSE
DESCRIPTOR.message_types_by_name['ActivateTracingRequest'] = _ACTIVATETRACINGREQUEST
DESCRIPTOR.message_types_by_name['CloseSessionRequest'] = _CLOSESESSIONREQUEST
DESCRIPTOR.message_types_by_name['UnregisterObjectsRequest'] = _UNREGISTEROBJECTSREQUEST
DESCRIPTOR.message_types_by_name['IsPrefetchingEnabledResponse'] = _ISPREFETCHINGENABLEDRESPONSE
DESCRIPTOR.message_types_by_name['ObjectExistsInDataClayRequest'] = _OBJECTEXISTSINDATACLAYREQUEST
DESCRIPTOR.message_types_by_name['ObjectExistsInDataClayResponse'] = _OBJECTEXISTSINDATACLAYRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

NewAccountNoAdminRequest = _reflection.GeneratedProtocolMessageType('NewAccountNoAdminRequest', (_message.Message,), dict(
  DESCRIPTOR = _NEWACCOUNTNOADMINREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewAccountNoAdminRequest)
  ))
_sym_db.RegisterMessage(NewAccountNoAdminRequest)

NewAccountRequest = _reflection.GeneratedProtocolMessageType('NewAccountRequest', (_message.Message,), dict(
  DESCRIPTOR = _NEWACCOUNTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewAccountRequest)
  ))
_sym_db.RegisterMessage(NewAccountRequest)

NewAccountResponse = _reflection.GeneratedProtocolMessageType('NewAccountResponse', (_message.Message,), dict(
  DESCRIPTOR = _NEWACCOUNTRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewAccountResponse)
  ))
_sym_db.RegisterMessage(NewAccountResponse)

PublishAddressRequest = _reflection.GeneratedProtocolMessageType('PublishAddressRequest', (_message.Message,), dict(
  DESCRIPTOR = _PUBLISHADDRESSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.PublishAddressRequest)
  ))
_sym_db.RegisterMessage(PublishAddressRequest)

GetAccountIDRequest = _reflection.GeneratedProtocolMessageType('GetAccountIDRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETACCOUNTIDREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetAccountIDRequest)
  ))
_sym_db.RegisterMessage(GetAccountIDRequest)

GetAccountIDResponse = _reflection.GeneratedProtocolMessageType('GetAccountIDResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETACCOUNTIDRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetAccountIDResponse)
  ))
_sym_db.RegisterMessage(GetAccountIDResponse)

AutoRegisterSLRequest = _reflection.GeneratedProtocolMessageType('AutoRegisterSLRequest', (_message.Message,), dict(
  DESCRIPTOR = _AUTOREGISTERSLREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.AutoRegisterSLRequest)
  ))
_sym_db.RegisterMessage(AutoRegisterSLRequest)

AutoRegisterEERequest = _reflection.GeneratedProtocolMessageType('AutoRegisterEERequest', (_message.Message,), dict(
  DESCRIPTOR = _AUTOREGISTEREEREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.AutoRegisterEERequest)
  ))
_sym_db.RegisterMessage(AutoRegisterEERequest)

AutoRegisterEEResponse = _reflection.GeneratedProtocolMessageType('AutoRegisterEEResponse', (_message.Message,), dict(
  DESCRIPTOR = _AUTOREGISTEREERESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.AutoRegisterEEResponse)
  ))
_sym_db.RegisterMessage(AutoRegisterEEResponse)

UnregisterStorageLocationRequest = _reflection.GeneratedProtocolMessageType('UnregisterStorageLocationRequest', (_message.Message,), dict(
  DESCRIPTOR = _UNREGISTERSTORAGELOCATIONREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.UnregisterStorageLocationRequest)
  ))
_sym_db.RegisterMessage(UnregisterStorageLocationRequest)

UnregisterExecutionEnvironmentRequest = _reflection.GeneratedProtocolMessageType('UnregisterExecutionEnvironmentRequest', (_message.Message,), dict(
  DESCRIPTOR = _UNREGISTEREXECUTIONENVIRONMENTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.UnregisterExecutionEnvironmentRequest)
  ))
_sym_db.RegisterMessage(UnregisterExecutionEnvironmentRequest)

PerformSetAccountsRequest = _reflection.GeneratedProtocolMessageType('PerformSetAccountsRequest', (_message.Message,), dict(
  DESCRIPTOR = _PERFORMSETACCOUNTSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.PerformSetAccountsRequest)
  ))
_sym_db.RegisterMessage(PerformSetAccountsRequest)

PerformSetAccountsResponse = _reflection.GeneratedProtocolMessageType('PerformSetAccountsResponse', (_message.Message,), dict(
  DESCRIPTOR = _PERFORMSETACCOUNTSRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.PerformSetAccountsResponse)
  ))
_sym_db.RegisterMessage(PerformSetAccountsResponse)

PerformSetOperationsRequest = _reflection.GeneratedProtocolMessageType('PerformSetOperationsRequest', (_message.Message,), dict(
  DESCRIPTOR = _PERFORMSETOPERATIONSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.PerformSetOperationsRequest)
  ))
_sym_db.RegisterMessage(PerformSetOperationsRequest)

PerformSetOperationsResponse = _reflection.GeneratedProtocolMessageType('PerformSetOperationsResponse', (_message.Message,), dict(
  DESCRIPTOR = _PERFORMSETOPERATIONSRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.PerformSetOperationsResponse)
  ))
_sym_db.RegisterMessage(PerformSetOperationsResponse)

GetAccountListRequest = _reflection.GeneratedProtocolMessageType('GetAccountListRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETACCOUNTLISTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetAccountListRequest)
  ))
_sym_db.RegisterMessage(GetAccountListRequest)

GetAccountListResponse = _reflection.GeneratedProtocolMessageType('GetAccountListResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETACCOUNTLISTRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetAccountListResponse)
  ))
_sym_db.RegisterMessage(GetAccountListResponse)

NewSessionRequest = _reflection.GeneratedProtocolMessageType('NewSessionRequest', (_message.Message,), dict(
  DESCRIPTOR = _NEWSESSIONREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewSessionRequest)
  ))
_sym_db.RegisterMessage(NewSessionRequest)

NewSessionResponse = _reflection.GeneratedProtocolMessageType('NewSessionResponse', (_message.Message,), dict(
  DESCRIPTOR = _NEWSESSIONRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewSessionResponse)
  ))
_sym_db.RegisterMessage(NewSessionResponse)

GetInfoOfSessionForDSRequest = _reflection.GeneratedProtocolMessageType('GetInfoOfSessionForDSRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETINFOOFSESSIONFORDSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetInfoOfSessionForDSRequest)
  ))
_sym_db.RegisterMessage(GetInfoOfSessionForDSRequest)

GetInfoOfSessionForDSResponse = _reflection.GeneratedProtocolMessageType('GetInfoOfSessionForDSResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETINFOOFSESSIONFORDSRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetInfoOfSessionForDSResponse)
  ))
_sym_db.RegisterMessage(GetInfoOfSessionForDSResponse)

NewNamespaceRequest = _reflection.GeneratedProtocolMessageType('NewNamespaceRequest', (_message.Message,), dict(
  DESCRIPTOR = _NEWNAMESPACEREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewNamespaceRequest)
  ))
_sym_db.RegisterMessage(NewNamespaceRequest)

NewNamespaceResponse = _reflection.GeneratedProtocolMessageType('NewNamespaceResponse', (_message.Message,), dict(
  DESCRIPTOR = _NEWNAMESPACERESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewNamespaceResponse)
  ))
_sym_db.RegisterMessage(NewNamespaceResponse)

GetNamespacesRequest = _reflection.GeneratedProtocolMessageType('GetNamespacesRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETNAMESPACESREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetNamespacesRequest)
  ))
_sym_db.RegisterMessage(GetNamespacesRequest)

GetNamespacesResponse = _reflection.GeneratedProtocolMessageType('GetNamespacesResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETNAMESPACESRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetNamespacesResponse)
  ))
_sym_db.RegisterMessage(GetNamespacesResponse)

RemoveNamespaceRequest = _reflection.GeneratedProtocolMessageType('RemoveNamespaceRequest', (_message.Message,), dict(
  DESCRIPTOR = _REMOVENAMESPACEREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RemoveNamespaceRequest)
  ))
_sym_db.RegisterMessage(RemoveNamespaceRequest)

GetNamespaceIDRequest = _reflection.GeneratedProtocolMessageType('GetNamespaceIDRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETNAMESPACEIDREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetNamespaceIDRequest)
  ))
_sym_db.RegisterMessage(GetNamespaceIDRequest)

GetNamespaceIDResponse = _reflection.GeneratedProtocolMessageType('GetNamespaceIDResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETNAMESPACEIDRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetNamespaceIDResponse)
  ))
_sym_db.RegisterMessage(GetNamespaceIDResponse)

GetNamespaceLangRequest = _reflection.GeneratedProtocolMessageType('GetNamespaceLangRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETNAMESPACELANGREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetNamespaceLangRequest)
  ))
_sym_db.RegisterMessage(GetNamespaceLangRequest)

GetNamespaceLangResponse = _reflection.GeneratedProtocolMessageType('GetNamespaceLangResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETNAMESPACELANGRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetNamespaceLangResponse)
  ))
_sym_db.RegisterMessage(GetNamespaceLangResponse)

GetObjectDataSetIDRequest = _reflection.GeneratedProtocolMessageType('GetObjectDataSetIDRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETOBJECTDATASETIDREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetObjectDataSetIDRequest)
  ))
_sym_db.RegisterMessage(GetObjectDataSetIDRequest)

GetObjectDataSetIDResponse = _reflection.GeneratedProtocolMessageType('GetObjectDataSetIDResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETOBJECTDATASETIDRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetObjectDataSetIDResponse)
  ))
_sym_db.RegisterMessage(GetObjectDataSetIDResponse)

ImportInterfaceRequest = _reflection.GeneratedProtocolMessageType('ImportInterfaceRequest', (_message.Message,), dict(
  DESCRIPTOR = _IMPORTINTERFACEREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.ImportInterfaceRequest)
  ))
_sym_db.RegisterMessage(ImportInterfaceRequest)

ImportContractRequest = _reflection.GeneratedProtocolMessageType('ImportContractRequest', (_message.Message,), dict(
  DESCRIPTOR = _IMPORTCONTRACTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.ImportContractRequest)
  ))
_sym_db.RegisterMessage(ImportContractRequest)

NewDataSetRequest = _reflection.GeneratedProtocolMessageType('NewDataSetRequest', (_message.Message,), dict(
  DESCRIPTOR = _NEWDATASETREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewDataSetRequest)
  ))
_sym_db.RegisterMessage(NewDataSetRequest)

NewDataSetResponse = _reflection.GeneratedProtocolMessageType('NewDataSetResponse', (_message.Message,), dict(
  DESCRIPTOR = _NEWDATASETRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewDataSetResponse)
  ))
_sym_db.RegisterMessage(NewDataSetResponse)

RemoveDataSetRequest = _reflection.GeneratedProtocolMessageType('RemoveDataSetRequest', (_message.Message,), dict(
  DESCRIPTOR = _REMOVEDATASETREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RemoveDataSetRequest)
  ))
_sym_db.RegisterMessage(RemoveDataSetRequest)

GetDataSetIDRequest = _reflection.GeneratedProtocolMessageType('GetDataSetIDRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETDATASETIDREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetDataSetIDRequest)
  ))
_sym_db.RegisterMessage(GetDataSetIDRequest)

GetDataSetIDResponse = _reflection.GeneratedProtocolMessageType('GetDataSetIDResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETDATASETIDRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetDataSetIDResponse)
  ))
_sym_db.RegisterMessage(GetDataSetIDResponse)

GetPublicDataSetsRequest = _reflection.GeneratedProtocolMessageType('GetPublicDataSetsRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETPUBLICDATASETSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetPublicDataSetsRequest)
  ))
_sym_db.RegisterMessage(GetPublicDataSetsRequest)

GetPublicDataSetsResponse = _reflection.GeneratedProtocolMessageType('GetPublicDataSetsResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETPUBLICDATASETSRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetPublicDataSetsResponse)
  ))
_sym_db.RegisterMessage(GetPublicDataSetsResponse)

GetAccountDataSetsRequest = _reflection.GeneratedProtocolMessageType('GetAccountDataSetsRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETACCOUNTDATASETSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetAccountDataSetsRequest)
  ))
_sym_db.RegisterMessage(GetAccountDataSetsRequest)

GetAccountDataSetsResponse = _reflection.GeneratedProtocolMessageType('GetAccountDataSetsResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETACCOUNTDATASETSRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetAccountDataSetsResponse)
  ))
_sym_db.RegisterMessage(GetAccountDataSetsResponse)

CheckDataSetIsPublicRequest = _reflection.GeneratedProtocolMessageType('CheckDataSetIsPublicRequest', (_message.Message,), dict(
  DESCRIPTOR = _CHECKDATASETISPUBLICREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.CheckDataSetIsPublicRequest)
  ))
_sym_db.RegisterMessage(CheckDataSetIsPublicRequest)

CheckDataSetIsPublicResponse = _reflection.GeneratedProtocolMessageType('CheckDataSetIsPublicResponse', (_message.Message,), dict(
  DESCRIPTOR = _CHECKDATASETISPUBLICRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.CheckDataSetIsPublicResponse)
  ))
_sym_db.RegisterMessage(CheckDataSetIsPublicResponse)

NewClassRequest = _reflection.GeneratedProtocolMessageType('NewClassRequest', (_message.Message,), dict(

  NewClassesEntry = _reflection.GeneratedProtocolMessageType('NewClassesEntry', (_message.Message,), dict(
    DESCRIPTOR = _NEWCLASSREQUEST_NEWCLASSESENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewClassRequest.NewClassesEntry)
    ))
  ,
  DESCRIPTOR = _NEWCLASSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewClassRequest)
  ))
_sym_db.RegisterMessage(NewClassRequest)
_sym_db.RegisterMessage(NewClassRequest.NewClassesEntry)

NewClassResponse = _reflection.GeneratedProtocolMessageType('NewClassResponse', (_message.Message,), dict(

  NewClassesEntry = _reflection.GeneratedProtocolMessageType('NewClassesEntry', (_message.Message,), dict(
    DESCRIPTOR = _NEWCLASSRESPONSE_NEWCLASSESENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewClassResponse.NewClassesEntry)
    ))
  ,
  DESCRIPTOR = _NEWCLASSRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewClassResponse)
  ))
_sym_db.RegisterMessage(NewClassResponse)
_sym_db.RegisterMessage(NewClassResponse.NewClassesEntry)

NewClassIDRequest = _reflection.GeneratedProtocolMessageType('NewClassIDRequest', (_message.Message,), dict(

  NewClassesEntry = _reflection.GeneratedProtocolMessageType('NewClassesEntry', (_message.Message,), dict(
    DESCRIPTOR = _NEWCLASSIDREQUEST_NEWCLASSESENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewClassIDRequest.NewClassesEntry)
    ))
  ,
  DESCRIPTOR = _NEWCLASSIDREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewClassIDRequest)
  ))
_sym_db.RegisterMessage(NewClassIDRequest)
_sym_db.RegisterMessage(NewClassIDRequest.NewClassesEntry)

NewClassIDResponse = _reflection.GeneratedProtocolMessageType('NewClassIDResponse', (_message.Message,), dict(
  DESCRIPTOR = _NEWCLASSIDRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewClassIDResponse)
  ))
_sym_db.RegisterMessage(NewClassIDResponse)

RemoveClassRequest = _reflection.GeneratedProtocolMessageType('RemoveClassRequest', (_message.Message,), dict(
  DESCRIPTOR = _REMOVECLASSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RemoveClassRequest)
  ))
_sym_db.RegisterMessage(RemoveClassRequest)

RemoveOperationRequest = _reflection.GeneratedProtocolMessageType('RemoveOperationRequest', (_message.Message,), dict(
  DESCRIPTOR = _REMOVEOPERATIONREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RemoveOperationRequest)
  ))
_sym_db.RegisterMessage(RemoveOperationRequest)

RemoveImplementationRequest = _reflection.GeneratedProtocolMessageType('RemoveImplementationRequest', (_message.Message,), dict(
  DESCRIPTOR = _REMOVEIMPLEMENTATIONREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RemoveImplementationRequest)
  ))
_sym_db.RegisterMessage(RemoveImplementationRequest)

GetOperationIDRequest = _reflection.GeneratedProtocolMessageType('GetOperationIDRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETOPERATIONIDREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetOperationIDRequest)
  ))
_sym_db.RegisterMessage(GetOperationIDRequest)

GetOperationIDResponse = _reflection.GeneratedProtocolMessageType('GetOperationIDResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETOPERATIONIDRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetOperationIDResponse)
  ))
_sym_db.RegisterMessage(GetOperationIDResponse)

GetPropertyIDRequest = _reflection.GeneratedProtocolMessageType('GetPropertyIDRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETPROPERTYIDREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetPropertyIDRequest)
  ))
_sym_db.RegisterMessage(GetPropertyIDRequest)

GetPropertyIDResponse = _reflection.GeneratedProtocolMessageType('GetPropertyIDResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETPROPERTYIDRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetPropertyIDResponse)
  ))
_sym_db.RegisterMessage(GetPropertyIDResponse)

GetClassIDRequest = _reflection.GeneratedProtocolMessageType('GetClassIDRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETCLASSIDREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetClassIDRequest)
  ))
_sym_db.RegisterMessage(GetClassIDRequest)

GetClassIDResponse = _reflection.GeneratedProtocolMessageType('GetClassIDResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETCLASSIDRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetClassIDResponse)
  ))
_sym_db.RegisterMessage(GetClassIDResponse)

GetClassInfoRequest = _reflection.GeneratedProtocolMessageType('GetClassInfoRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETCLASSINFOREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetClassInfoRequest)
  ))
_sym_db.RegisterMessage(GetClassInfoRequest)

GetClassInfoResponse = _reflection.GeneratedProtocolMessageType('GetClassInfoResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETCLASSINFORESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetClassInfoResponse)
  ))
_sym_db.RegisterMessage(GetClassInfoResponse)

GetInfoOfClassesInNamespaceRequest = _reflection.GeneratedProtocolMessageType('GetInfoOfClassesInNamespaceRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETINFOOFCLASSESINNAMESPACEREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceRequest)
  ))
_sym_db.RegisterMessage(GetInfoOfClassesInNamespaceRequest)

GetInfoOfClassesInNamespaceResponse = _reflection.GeneratedProtocolMessageType('GetInfoOfClassesInNamespaceResponse', (_message.Message,), dict(

  ClassesInfoEntry = _reflection.GeneratedProtocolMessageType('ClassesInfoEntry', (_message.Message,), dict(
    DESCRIPTOR = _GETINFOOFCLASSESINNAMESPACERESPONSE_CLASSESINFOENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceResponse.ClassesInfoEntry)
    ))
  ,
  DESCRIPTOR = _GETINFOOFCLASSESINNAMESPACERESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetInfoOfClassesInNamespaceResponse)
  ))
_sym_db.RegisterMessage(GetInfoOfClassesInNamespaceResponse)
_sym_db.RegisterMessage(GetInfoOfClassesInNamespaceResponse.ClassesInfoEntry)

GetImportedClassesInfoInNamespaceRequest = _reflection.GeneratedProtocolMessageType('GetImportedClassesInfoInNamespaceRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETIMPORTEDCLASSESINFOINNAMESPACEREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceRequest)
  ))
_sym_db.RegisterMessage(GetImportedClassesInfoInNamespaceRequest)

GetImportedClassesInfoInNamespaceResponse = _reflection.GeneratedProtocolMessageType('GetImportedClassesInfoInNamespaceResponse', (_message.Message,), dict(

  ImportedClassesEntry = _reflection.GeneratedProtocolMessageType('ImportedClassesEntry', (_message.Message,), dict(
    DESCRIPTOR = _GETIMPORTEDCLASSESINFOINNAMESPACERESPONSE_IMPORTEDCLASSESENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceResponse.ImportedClassesEntry)
    ))
  ,
  DESCRIPTOR = _GETIMPORTEDCLASSESINFOINNAMESPACERESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetImportedClassesInfoInNamespaceResponse)
  ))
_sym_db.RegisterMessage(GetImportedClassesInfoInNamespaceResponse)
_sym_db.RegisterMessage(GetImportedClassesInfoInNamespaceResponse.ImportedClassesEntry)

GetClassIDFromImportRequest = _reflection.GeneratedProtocolMessageType('GetClassIDFromImportRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETCLASSIDFROMIMPORTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetClassIDFromImportRequest)
  ))
_sym_db.RegisterMessage(GetClassIDFromImportRequest)

GetClassIDFromImportResponse = _reflection.GeneratedProtocolMessageType('GetClassIDFromImportResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETCLASSIDFROMIMPORTRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetClassIDFromImportResponse)
  ))
_sym_db.RegisterMessage(GetClassIDFromImportResponse)

NewContractRequest = _reflection.GeneratedProtocolMessageType('NewContractRequest', (_message.Message,), dict(
  DESCRIPTOR = _NEWCONTRACTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewContractRequest)
  ))
_sym_db.RegisterMessage(NewContractRequest)

NewContractResponse = _reflection.GeneratedProtocolMessageType('NewContractResponse', (_message.Message,), dict(
  DESCRIPTOR = _NEWCONTRACTRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewContractResponse)
  ))
_sym_db.RegisterMessage(NewContractResponse)

RegisterToPublicContractRequest = _reflection.GeneratedProtocolMessageType('RegisterToPublicContractRequest', (_message.Message,), dict(
  DESCRIPTOR = _REGISTERTOPUBLICCONTRACTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RegisterToPublicContractRequest)
  ))
_sym_db.RegisterMessage(RegisterToPublicContractRequest)

RegisterToPublicContractOfNamespaceRequest = _reflection.GeneratedProtocolMessageType('RegisterToPublicContractOfNamespaceRequest', (_message.Message,), dict(
  DESCRIPTOR = _REGISTERTOPUBLICCONTRACTOFNAMESPACEREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RegisterToPublicContractOfNamespaceRequest)
  ))
_sym_db.RegisterMessage(RegisterToPublicContractOfNamespaceRequest)

RegisterToPublicContractOfNamespaceResponse = _reflection.GeneratedProtocolMessageType('RegisterToPublicContractOfNamespaceResponse', (_message.Message,), dict(
  DESCRIPTOR = _REGISTERTOPUBLICCONTRACTOFNAMESPACERESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RegisterToPublicContractOfNamespaceResponse)
  ))
_sym_db.RegisterMessage(RegisterToPublicContractOfNamespaceResponse)

GetContractIDsOfApplicantRequest = _reflection.GeneratedProtocolMessageType('GetContractIDsOfApplicantRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETCONTRACTIDSOFAPPLICANTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantRequest)
  ))
_sym_db.RegisterMessage(GetContractIDsOfApplicantRequest)

GetContractIDsOfApplicantResponse = _reflection.GeneratedProtocolMessageType('GetContractIDsOfApplicantResponse', (_message.Message,), dict(

  ContractsEntry = _reflection.GeneratedProtocolMessageType('ContractsEntry', (_message.Message,), dict(
    DESCRIPTOR = _GETCONTRACTIDSOFAPPLICANTRESPONSE_CONTRACTSENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantResponse.ContractsEntry)
    ))
  ,
  DESCRIPTOR = _GETCONTRACTIDSOFAPPLICANTRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetContractIDsOfApplicantResponse)
  ))
_sym_db.RegisterMessage(GetContractIDsOfApplicantResponse)
_sym_db.RegisterMessage(GetContractIDsOfApplicantResponse.ContractsEntry)

GetContractIDsOfProviderRequest = _reflection.GeneratedProtocolMessageType('GetContractIDsOfProviderRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETCONTRACTIDSOFPROVIDERREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderRequest)
  ))
_sym_db.RegisterMessage(GetContractIDsOfProviderRequest)

GetContractIDsOfProviderResponse = _reflection.GeneratedProtocolMessageType('GetContractIDsOfProviderResponse', (_message.Message,), dict(

  ContractsEntry = _reflection.GeneratedProtocolMessageType('ContractsEntry', (_message.Message,), dict(
    DESCRIPTOR = _GETCONTRACTIDSOFPROVIDERRESPONSE_CONTRACTSENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderResponse.ContractsEntry)
    ))
  ,
  DESCRIPTOR = _GETCONTRACTIDSOFPROVIDERRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetContractIDsOfProviderResponse)
  ))
_sym_db.RegisterMessage(GetContractIDsOfProviderResponse)
_sym_db.RegisterMessage(GetContractIDsOfProviderResponse.ContractsEntry)

GetContractsOfApplicantWithProvRequest = _reflection.GeneratedProtocolMessageType('GetContractsOfApplicantWithProvRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETCONTRACTSOFAPPLICANTWITHPROVREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvRequest)
  ))
_sym_db.RegisterMessage(GetContractsOfApplicantWithProvRequest)

GetContractsOfApplicantWithProvResponse = _reflection.GeneratedProtocolMessageType('GetContractsOfApplicantWithProvResponse', (_message.Message,), dict(

  ContractsEntry = _reflection.GeneratedProtocolMessageType('ContractsEntry', (_message.Message,), dict(
    DESCRIPTOR = _GETCONTRACTSOFAPPLICANTWITHPROVRESPONSE_CONTRACTSENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvResponse.ContractsEntry)
    ))
  ,
  DESCRIPTOR = _GETCONTRACTSOFAPPLICANTWITHPROVRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetContractsOfApplicantWithProvResponse)
  ))
_sym_db.RegisterMessage(GetContractsOfApplicantWithProvResponse)
_sym_db.RegisterMessage(GetContractsOfApplicantWithProvResponse.ContractsEntry)

NewDataContractRequest = _reflection.GeneratedProtocolMessageType('NewDataContractRequest', (_message.Message,), dict(
  DESCRIPTOR = _NEWDATACONTRACTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewDataContractRequest)
  ))
_sym_db.RegisterMessage(NewDataContractRequest)

NewDataContractResponse = _reflection.GeneratedProtocolMessageType('NewDataContractResponse', (_message.Message,), dict(
  DESCRIPTOR = _NEWDATACONTRACTRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewDataContractResponse)
  ))
_sym_db.RegisterMessage(NewDataContractResponse)

RegisterToPublicDataContractRequest = _reflection.GeneratedProtocolMessageType('RegisterToPublicDataContractRequest', (_message.Message,), dict(
  DESCRIPTOR = _REGISTERTOPUBLICDATACONTRACTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RegisterToPublicDataContractRequest)
  ))
_sym_db.RegisterMessage(RegisterToPublicDataContractRequest)

GetDataContractIDsOfApplicantRequest = _reflection.GeneratedProtocolMessageType('GetDataContractIDsOfApplicantRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETDATACONTRACTIDSOFAPPLICANTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantRequest)
  ))
_sym_db.RegisterMessage(GetDataContractIDsOfApplicantRequest)

GetDataContractIDsOfApplicantResponse = _reflection.GeneratedProtocolMessageType('GetDataContractIDsOfApplicantResponse', (_message.Message,), dict(

  DatacontractsEntry = _reflection.GeneratedProtocolMessageType('DatacontractsEntry', (_message.Message,), dict(
    DESCRIPTOR = _GETDATACONTRACTIDSOFAPPLICANTRESPONSE_DATACONTRACTSENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantResponse.DatacontractsEntry)
    ))
  ,
  DESCRIPTOR = _GETDATACONTRACTIDSOFAPPLICANTRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetDataContractIDsOfApplicantResponse)
  ))
_sym_db.RegisterMessage(GetDataContractIDsOfApplicantResponse)
_sym_db.RegisterMessage(GetDataContractIDsOfApplicantResponse.DatacontractsEntry)

GetDataContractIDsOfProviderRequest = _reflection.GeneratedProtocolMessageType('GetDataContractIDsOfProviderRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETDATACONTRACTIDSOFPROVIDERREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderRequest)
  ))
_sym_db.RegisterMessage(GetDataContractIDsOfProviderRequest)

GetDataContractIDsOfProviderResponse = _reflection.GeneratedProtocolMessageType('GetDataContractIDsOfProviderResponse', (_message.Message,), dict(

  DatacontractsEntry = _reflection.GeneratedProtocolMessageType('DatacontractsEntry', (_message.Message,), dict(
    DESCRIPTOR = _GETDATACONTRACTIDSOFPROVIDERRESPONSE_DATACONTRACTSENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderResponse.DatacontractsEntry)
    ))
  ,
  DESCRIPTOR = _GETDATACONTRACTIDSOFPROVIDERRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetDataContractIDsOfProviderResponse)
  ))
_sym_db.RegisterMessage(GetDataContractIDsOfProviderResponse)
_sym_db.RegisterMessage(GetDataContractIDsOfProviderResponse.DatacontractsEntry)

GetDataContractInfoOfApplicantWithProvRequest = _reflection.GeneratedProtocolMessageType('GetDataContractInfoOfApplicantWithProvRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETDATACONTRACTINFOOFAPPLICANTWITHPROVREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetDataContractInfoOfApplicantWithProvRequest)
  ))
_sym_db.RegisterMessage(GetDataContractInfoOfApplicantWithProvRequest)

GetDataContractInfoOfApplicantWithProvResponse = _reflection.GeneratedProtocolMessageType('GetDataContractInfoOfApplicantWithProvResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETDATACONTRACTINFOOFAPPLICANTWITHPROVRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetDataContractInfoOfApplicantWithProvResponse)
  ))
_sym_db.RegisterMessage(GetDataContractInfoOfApplicantWithProvResponse)

NewInterfaceRequest = _reflection.GeneratedProtocolMessageType('NewInterfaceRequest', (_message.Message,), dict(
  DESCRIPTOR = _NEWINTERFACEREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewInterfaceRequest)
  ))
_sym_db.RegisterMessage(NewInterfaceRequest)

NewInterfaceResponse = _reflection.GeneratedProtocolMessageType('NewInterfaceResponse', (_message.Message,), dict(
  DESCRIPTOR = _NEWINTERFACERESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewInterfaceResponse)
  ))
_sym_db.RegisterMessage(NewInterfaceResponse)

GetInterfaceInfoRequest = _reflection.GeneratedProtocolMessageType('GetInterfaceInfoRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETINTERFACEINFOREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetInterfaceInfoRequest)
  ))
_sym_db.RegisterMessage(GetInterfaceInfoRequest)

GetInterfaceInfoResponse = _reflection.GeneratedProtocolMessageType('GetInterfaceInfoResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETINTERFACEINFORESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetInterfaceInfoResponse)
  ))
_sym_db.RegisterMessage(GetInterfaceInfoResponse)

RemoveInterfaceRequest = _reflection.GeneratedProtocolMessageType('RemoveInterfaceRequest', (_message.Message,), dict(
  DESCRIPTOR = _REMOVEINTERFACEREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RemoveInterfaceRequest)
  ))
_sym_db.RegisterMessage(RemoveInterfaceRequest)

GetStorageLocationForDSRequest = _reflection.GeneratedProtocolMessageType('GetStorageLocationForDSRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETSTORAGELOCATIONFORDSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetStorageLocationForDSRequest)
  ))
_sym_db.RegisterMessage(GetStorageLocationForDSRequest)

GetStorageLocationForDSResponse = _reflection.GeneratedProtocolMessageType('GetStorageLocationForDSResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETSTORAGELOCATIONFORDSRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetStorageLocationForDSResponse)
  ))
_sym_db.RegisterMessage(GetStorageLocationForDSResponse)

GetExecutionEnvironmentForDSRequest = _reflection.GeneratedProtocolMessageType('GetExecutionEnvironmentForDSRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETEXECUTIONENVIRONMENTFORDSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentForDSRequest)
  ))
_sym_db.RegisterMessage(GetExecutionEnvironmentForDSRequest)

GetExecutionEnvironmentForDSResponse = _reflection.GeneratedProtocolMessageType('GetExecutionEnvironmentForDSResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETEXECUTIONENVIRONMENTFORDSRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentForDSResponse)
  ))
_sym_db.RegisterMessage(GetExecutionEnvironmentForDSResponse)

GetDataClayIDResponse = _reflection.GeneratedProtocolMessageType('GetDataClayIDResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETDATACLAYIDRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetDataClayIDResponse)
  ))
_sym_db.RegisterMessage(GetDataClayIDResponse)

RegisterExternalDataClayRequest = _reflection.GeneratedProtocolMessageType('RegisterExternalDataClayRequest', (_message.Message,), dict(
  DESCRIPTOR = _REGISTEREXTERNALDATACLAYREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RegisterExternalDataClayRequest)
  ))
_sym_db.RegisterMessage(RegisterExternalDataClayRequest)

RegisterExternalDataClayResponse = _reflection.GeneratedProtocolMessageType('RegisterExternalDataClayResponse', (_message.Message,), dict(
  DESCRIPTOR = _REGISTEREXTERNALDATACLAYRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RegisterExternalDataClayResponse)
  ))
_sym_db.RegisterMessage(RegisterExternalDataClayResponse)

NotifyRegistrationOfExternalDataClayRequest = _reflection.GeneratedProtocolMessageType('NotifyRegistrationOfExternalDataClayRequest', (_message.Message,), dict(
  DESCRIPTOR = _NOTIFYREGISTRATIONOFEXTERNALDATACLAYREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NotifyRegistrationOfExternalDataClayRequest)
  ))
_sym_db.RegisterMessage(NotifyRegistrationOfExternalDataClayRequest)

NotifyRegistrationOfExternalDataClayResponse = _reflection.GeneratedProtocolMessageType('NotifyRegistrationOfExternalDataClayResponse', (_message.Message,), dict(
  DESCRIPTOR = _NOTIFYREGISTRATIONOFEXTERNALDATACLAYRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NotifyRegistrationOfExternalDataClayResponse)
  ))
_sym_db.RegisterMessage(NotifyRegistrationOfExternalDataClayResponse)

GetExternalDataclayIDRequest = _reflection.GeneratedProtocolMessageType('GetExternalDataclayIDRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETEXTERNALDATACLAYIDREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetExternalDataclayIDRequest)
  ))
_sym_db.RegisterMessage(GetExternalDataclayIDRequest)

GetExternalDataclayIDResponse = _reflection.GeneratedProtocolMessageType('GetExternalDataclayIDResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETEXTERNALDATACLAYIDRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetExternalDataclayIDResponse)
  ))
_sym_db.RegisterMessage(GetExternalDataclayIDResponse)

GetExtDataClayInfoRequest = _reflection.GeneratedProtocolMessageType('GetExtDataClayInfoRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETEXTDATACLAYINFOREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetExtDataClayInfoRequest)
  ))
_sym_db.RegisterMessage(GetExtDataClayInfoRequest)

GetExtDataClayInfoResponse = _reflection.GeneratedProtocolMessageType('GetExtDataClayInfoResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETEXTDATACLAYINFORESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetExtDataClayInfoResponse)
  ))
_sym_db.RegisterMessage(GetExtDataClayInfoResponse)

FederateObjectRequest = _reflection.GeneratedProtocolMessageType('FederateObjectRequest', (_message.Message,), dict(
  DESCRIPTOR = _FEDERATEOBJECTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.FederateObjectRequest)
  ))
_sym_db.RegisterMessage(FederateObjectRequest)

UnfederateObjectRequest = _reflection.GeneratedProtocolMessageType('UnfederateObjectRequest', (_message.Message,), dict(
  DESCRIPTOR = _UNFEDERATEOBJECTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.UnfederateObjectRequest)
  ))
_sym_db.RegisterMessage(UnfederateObjectRequest)

UnfederateObjectWithAllDCsRequest = _reflection.GeneratedProtocolMessageType('UnfederateObjectWithAllDCsRequest', (_message.Message,), dict(
  DESCRIPTOR = _UNFEDERATEOBJECTWITHALLDCSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.UnfederateObjectWithAllDCsRequest)
  ))
_sym_db.RegisterMessage(UnfederateObjectWithAllDCsRequest)

MigrateFederatedObjectsRequest = _reflection.GeneratedProtocolMessageType('MigrateFederatedObjectsRequest', (_message.Message,), dict(
  DESCRIPTOR = _MIGRATEFEDERATEDOBJECTSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.MigrateFederatedObjectsRequest)
  ))
_sym_db.RegisterMessage(MigrateFederatedObjectsRequest)

FederateAllObjectsRequest = _reflection.GeneratedProtocolMessageType('FederateAllObjectsRequest', (_message.Message,), dict(
  DESCRIPTOR = _FEDERATEALLOBJECTSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.FederateAllObjectsRequest)
  ))
_sym_db.RegisterMessage(FederateAllObjectsRequest)

UnfederateAllObjectsRequest = _reflection.GeneratedProtocolMessageType('UnfederateAllObjectsRequest', (_message.Message,), dict(
  DESCRIPTOR = _UNFEDERATEALLOBJECTSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.UnfederateAllObjectsRequest)
  ))
_sym_db.RegisterMessage(UnfederateAllObjectsRequest)

UnfederateAllObjectsWithAllDCsRequest = _reflection.GeneratedProtocolMessageType('UnfederateAllObjectsWithAllDCsRequest', (_message.Message,), dict(
  DESCRIPTOR = _UNFEDERATEALLOBJECTSWITHALLDCSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.UnfederateAllObjectsWithAllDCsRequest)
  ))
_sym_db.RegisterMessage(UnfederateAllObjectsWithAllDCsRequest)

NotifyFederatedObjectsRequest = _reflection.GeneratedProtocolMessageType('NotifyFederatedObjectsRequest', (_message.Message,), dict(

  ObjectsInfoEntry = _reflection.GeneratedProtocolMessageType('ObjectsInfoEntry', (_message.Message,), dict(
    DESCRIPTOR = _NOTIFYFEDERATEDOBJECTSREQUEST_OBJECTSINFOENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.ObjectsInfoEntry)
    ))
  ,

  FederatedObjectsEntry = _reflection.GeneratedProtocolMessageType('FederatedObjectsEntry', (_message.Message,), dict(
    DESCRIPTOR = _NOTIFYFEDERATEDOBJECTSREQUEST_FEDERATEDOBJECTSENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest.FederatedObjectsEntry)
    ))
  ,
  DESCRIPTOR = _NOTIFYFEDERATEDOBJECTSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NotifyFederatedObjectsRequest)
  ))
_sym_db.RegisterMessage(NotifyFederatedObjectsRequest)
_sym_db.RegisterMessage(NotifyFederatedObjectsRequest.ObjectsInfoEntry)
_sym_db.RegisterMessage(NotifyFederatedObjectsRequest.FederatedObjectsEntry)

NotifyUnfederatedObjectsRequest = _reflection.GeneratedProtocolMessageType('NotifyUnfederatedObjectsRequest', (_message.Message,), dict(
  DESCRIPTOR = _NOTIFYUNFEDERATEDOBJECTSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NotifyUnfederatedObjectsRequest)
  ))
_sym_db.RegisterMessage(NotifyUnfederatedObjectsRequest)

CheckObjectFederatedWithDataClayInstanceRequest = _reflection.GeneratedProtocolMessageType('CheckObjectFederatedWithDataClayInstanceRequest', (_message.Message,), dict(
  DESCRIPTOR = _CHECKOBJECTFEDERATEDWITHDATACLAYINSTANCEREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.CheckObjectFederatedWithDataClayInstanceRequest)
  ))
_sym_db.RegisterMessage(CheckObjectFederatedWithDataClayInstanceRequest)

CheckObjectFederatedWithDataClayInstanceResponse = _reflection.GeneratedProtocolMessageType('CheckObjectFederatedWithDataClayInstanceResponse', (_message.Message,), dict(
  DESCRIPTOR = _CHECKOBJECTFEDERATEDWITHDATACLAYINSTANCERESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.CheckObjectFederatedWithDataClayInstanceResponse)
  ))
_sym_db.RegisterMessage(CheckObjectFederatedWithDataClayInstanceResponse)

CheckObjectFederatedRequest = _reflection.GeneratedProtocolMessageType('CheckObjectFederatedRequest', (_message.Message,), dict(
  DESCRIPTOR = _CHECKOBJECTFEDERATEDREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.CheckObjectFederatedRequest)
  ))
_sym_db.RegisterMessage(CheckObjectFederatedRequest)

CheckObjectFederatedResponse = _reflection.GeneratedProtocolMessageType('CheckObjectFederatedResponse', (_message.Message,), dict(
  DESCRIPTOR = _CHECKOBJECTFEDERATEDRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.CheckObjectFederatedResponse)
  ))
_sym_db.RegisterMessage(CheckObjectFederatedResponse)

GetDataClaysObjectIsFederatedWithRequest = _reflection.GeneratedProtocolMessageType('GetDataClaysObjectIsFederatedWithRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETDATACLAYSOBJECTISFEDERATEDWITHREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetDataClaysObjectIsFederatedWithRequest)
  ))
_sym_db.RegisterMessage(GetDataClaysObjectIsFederatedWithRequest)

GetDataClaysObjectIsFederatedWithResponse = _reflection.GeneratedProtocolMessageType('GetDataClaysObjectIsFederatedWithResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETDATACLAYSOBJECTISFEDERATEDWITHRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetDataClaysObjectIsFederatedWithResponse)
  ))
_sym_db.RegisterMessage(GetDataClaysObjectIsFederatedWithResponse)

GetExternalSourceDataClayOfObjectRequest = _reflection.GeneratedProtocolMessageType('GetExternalSourceDataClayOfObjectRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETEXTERNALSOURCEDATACLAYOFOBJECTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetExternalSourceDataClayOfObjectRequest)
  ))
_sym_db.RegisterMessage(GetExternalSourceDataClayOfObjectRequest)

GetExternalSourceDataClayOfObjectResponse = _reflection.GeneratedProtocolMessageType('GetExternalSourceDataClayOfObjectResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETEXTERNALSOURCEDATACLAYOFOBJECTRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetExternalSourceDataClayOfObjectResponse)
  ))
_sym_db.RegisterMessage(GetExternalSourceDataClayOfObjectResponse)

RegisterObjectForGCRequest = _reflection.GeneratedProtocolMessageType('RegisterObjectForGCRequest', (_message.Message,), dict(
  DESCRIPTOR = _REGISTEROBJECTFORGCREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RegisterObjectForGCRequest)
  ))
_sym_db.RegisterMessage(RegisterObjectForGCRequest)

RegisterObjectRequest = _reflection.GeneratedProtocolMessageType('RegisterObjectRequest', (_message.Message,), dict(
  DESCRIPTOR = _REGISTEROBJECTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RegisterObjectRequest)
  ))
_sym_db.RegisterMessage(RegisterObjectRequest)

GetExecutionEnvironmentsInfoRequest = _reflection.GeneratedProtocolMessageType('GetExecutionEnvironmentsInfoRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETEXECUTIONENVIRONMENTSINFOREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoRequest)
  ))
_sym_db.RegisterMessage(GetExecutionEnvironmentsInfoRequest)

GetExecutionEnvironmentsInfoResponse = _reflection.GeneratedProtocolMessageType('GetExecutionEnvironmentsInfoResponse', (_message.Message,), dict(

  ExecEnvsEntry = _reflection.GeneratedProtocolMessageType('ExecEnvsEntry', (_message.Message,), dict(
    DESCRIPTOR = _GETEXECUTIONENVIRONMENTSINFORESPONSE_EXECENVSENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoResponse.ExecEnvsEntry)
    ))
  ,
  DESCRIPTOR = _GETEXECUTIONENVIRONMENTSINFORESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsInfoResponse)
  ))
_sym_db.RegisterMessage(GetExecutionEnvironmentsInfoResponse)
_sym_db.RegisterMessage(GetExecutionEnvironmentsInfoResponse.ExecEnvsEntry)

GetExecutionEnvironmentsNamesRequest = _reflection.GeneratedProtocolMessageType('GetExecutionEnvironmentsNamesRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETEXECUTIONENVIRONMENTSNAMESREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsNamesRequest)
  ))
_sym_db.RegisterMessage(GetExecutionEnvironmentsNamesRequest)

GetExecutionEnvironmentsNamesResponse = _reflection.GeneratedProtocolMessageType('GetExecutionEnvironmentsNamesResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETEXECUTIONENVIRONMENTSNAMESRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetExecutionEnvironmentsNamesResponse)
  ))
_sym_db.RegisterMessage(GetExecutionEnvironmentsNamesResponse)

GetObjectInfoRequest = _reflection.GeneratedProtocolMessageType('GetObjectInfoRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETOBJECTINFOREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetObjectInfoRequest)
  ))
_sym_db.RegisterMessage(GetObjectInfoRequest)

GetObjectInfoResponse = _reflection.GeneratedProtocolMessageType('GetObjectInfoResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETOBJECTINFORESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetObjectInfoResponse)
  ))
_sym_db.RegisterMessage(GetObjectInfoResponse)

GetObjectFromAliasRequest = _reflection.GeneratedProtocolMessageType('GetObjectFromAliasRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETOBJECTFROMALIASREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetObjectFromAliasRequest)
  ))
_sym_db.RegisterMessage(GetObjectFromAliasRequest)

GetObjectFromAliasResponse = _reflection.GeneratedProtocolMessageType('GetObjectFromAliasResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETOBJECTFROMALIASRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetObjectFromAliasResponse)
  ))
_sym_db.RegisterMessage(GetObjectFromAliasResponse)

AddAliasRequest = _reflection.GeneratedProtocolMessageType('AddAliasRequest', (_message.Message,), dict(
  DESCRIPTOR = _ADDALIASREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.AddAliasRequest)
  ))
_sym_db.RegisterMessage(AddAliasRequest)

DeleteAliasRequest = _reflection.GeneratedProtocolMessageType('DeleteAliasRequest', (_message.Message,), dict(
  DESCRIPTOR = _DELETEALIASREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.DeleteAliasRequest)
  ))
_sym_db.RegisterMessage(DeleteAliasRequest)

GetObjectsMetaDataInfoOfClassForNMRequest = _reflection.GeneratedProtocolMessageType('GetObjectsMetaDataInfoOfClassForNMRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETOBJECTSMETADATAINFOOFCLASSFORNMREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMRequest)
  ))
_sym_db.RegisterMessage(GetObjectsMetaDataInfoOfClassForNMRequest)

GetObjectsMetaDataInfoOfClassForNMResponse = _reflection.GeneratedProtocolMessageType('GetObjectsMetaDataInfoOfClassForNMResponse', (_message.Message,), dict(

  MdataInfoEntry = _reflection.GeneratedProtocolMessageType('MdataInfoEntry', (_message.Message,), dict(
    DESCRIPTOR = _GETOBJECTSMETADATAINFOOFCLASSFORNMRESPONSE_MDATAINFOENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMResponse.MdataInfoEntry)
    ))
  ,
  DESCRIPTOR = _GETOBJECTSMETADATAINFOOFCLASSFORNMRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetObjectsMetaDataInfoOfClassForNMResponse)
  ))
_sym_db.RegisterMessage(GetObjectsMetaDataInfoOfClassForNMResponse)
_sym_db.RegisterMessage(GetObjectsMetaDataInfoOfClassForNMResponse.MdataInfoEntry)

SetDataSetIDRequest = _reflection.GeneratedProtocolMessageType('SetDataSetIDRequest', (_message.Message,), dict(
  DESCRIPTOR = _SETDATASETIDREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.SetDataSetIDRequest)
  ))
_sym_db.RegisterMessage(SetDataSetIDRequest)

SetDataSetIDFromGarbageCollectorRequest = _reflection.GeneratedProtocolMessageType('SetDataSetIDFromGarbageCollectorRequest', (_message.Message,), dict(
  DESCRIPTOR = _SETDATASETIDFROMGARBAGECOLLECTORREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.SetDataSetIDFromGarbageCollectorRequest)
  ))
_sym_db.RegisterMessage(SetDataSetIDFromGarbageCollectorRequest)

NewVersionRequest = _reflection.GeneratedProtocolMessageType('NewVersionRequest', (_message.Message,), dict(
  DESCRIPTOR = _NEWVERSIONREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewVersionRequest)
  ))
_sym_db.RegisterMessage(NewVersionRequest)

NewVersionResponse = _reflection.GeneratedProtocolMessageType('NewVersionResponse', (_message.Message,), dict(
  DESCRIPTOR = _NEWVERSIONRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewVersionResponse)
  ))
_sym_db.RegisterMessage(NewVersionResponse)

ConsolidateVersionRequest = _reflection.GeneratedProtocolMessageType('ConsolidateVersionRequest', (_message.Message,), dict(
  DESCRIPTOR = _CONSOLIDATEVERSIONREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.ConsolidateVersionRequest)
  ))
_sym_db.RegisterMessage(ConsolidateVersionRequest)

NewReplicaRequest = _reflection.GeneratedProtocolMessageType('NewReplicaRequest', (_message.Message,), dict(
  DESCRIPTOR = _NEWREPLICAREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewReplicaRequest)
  ))
_sym_db.RegisterMessage(NewReplicaRequest)

NewReplicaResponse = _reflection.GeneratedProtocolMessageType('NewReplicaResponse', (_message.Message,), dict(
  DESCRIPTOR = _NEWREPLICARESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.NewReplicaResponse)
  ))
_sym_db.RegisterMessage(NewReplicaResponse)

MoveObjectRequest = _reflection.GeneratedProtocolMessageType('MoveObjectRequest', (_message.Message,), dict(
  DESCRIPTOR = _MOVEOBJECTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.MoveObjectRequest)
  ))
_sym_db.RegisterMessage(MoveObjectRequest)

MoveObjectResponse = _reflection.GeneratedProtocolMessageType('MoveObjectResponse', (_message.Message,), dict(
  DESCRIPTOR = _MOVEOBJECTRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.MoveObjectResponse)
  ))
_sym_db.RegisterMessage(MoveObjectResponse)

SetObjectReadOnlyRequest = _reflection.GeneratedProtocolMessageType('SetObjectReadOnlyRequest', (_message.Message,), dict(
  DESCRIPTOR = _SETOBJECTREADONLYREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.SetObjectReadOnlyRequest)
  ))
_sym_db.RegisterMessage(SetObjectReadOnlyRequest)

SetObjectReadWriteRequest = _reflection.GeneratedProtocolMessageType('SetObjectReadWriteRequest', (_message.Message,), dict(
  DESCRIPTOR = _SETOBJECTREADWRITEREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.SetObjectReadWriteRequest)
  ))
_sym_db.RegisterMessage(SetObjectReadWriteRequest)

GetMetadataByOIDRequest = _reflection.GeneratedProtocolMessageType('GetMetadataByOIDRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETMETADATABYOIDREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetMetadataByOIDRequest)
  ))
_sym_db.RegisterMessage(GetMetadataByOIDRequest)

GetMetadataByOIDResponse = _reflection.GeneratedProtocolMessageType('GetMetadataByOIDResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETMETADATABYOIDRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetMetadataByOIDResponse)
  ))
_sym_db.RegisterMessage(GetMetadataByOIDResponse)

GetMetadataByOIDForDSRequest = _reflection.GeneratedProtocolMessageType('GetMetadataByOIDForDSRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETMETADATABYOIDFORDSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetMetadataByOIDForDSRequest)
  ))
_sym_db.RegisterMessage(GetMetadataByOIDForDSRequest)

GetMetadataByOIDForDSResponse = _reflection.GeneratedProtocolMessageType('GetMetadataByOIDForDSResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETMETADATABYOIDFORDSRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetMetadataByOIDForDSResponse)
  ))
_sym_db.RegisterMessage(GetMetadataByOIDForDSResponse)

ExecuteImplementationRequest = _reflection.GeneratedProtocolMessageType('ExecuteImplementationRequest', (_message.Message,), dict(
  DESCRIPTOR = _EXECUTEIMPLEMENTATIONREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.ExecuteImplementationRequest)
  ))
_sym_db.RegisterMessage(ExecuteImplementationRequest)

ExecuteImplementationResponse = _reflection.GeneratedProtocolMessageType('ExecuteImplementationResponse', (_message.Message,), dict(
  DESCRIPTOR = _EXECUTEIMPLEMENTATIONRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.ExecuteImplementationResponse)
  ))
_sym_db.RegisterMessage(ExecuteImplementationResponse)

ExecuteMethodOnTargetRequest = _reflection.GeneratedProtocolMessageType('ExecuteMethodOnTargetRequest', (_message.Message,), dict(
  DESCRIPTOR = _EXECUTEMETHODONTARGETREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.ExecuteMethodOnTargetRequest)
  ))
_sym_db.RegisterMessage(ExecuteMethodOnTargetRequest)

ExecuteMethodOnTargetResponse = _reflection.GeneratedProtocolMessageType('ExecuteMethodOnTargetResponse', (_message.Message,), dict(
  DESCRIPTOR = _EXECUTEMETHODONTARGETRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.ExecuteMethodOnTargetResponse)
  ))
_sym_db.RegisterMessage(ExecuteMethodOnTargetResponse)

SynchronizeFederatedObjectRequest = _reflection.GeneratedProtocolMessageType('SynchronizeFederatedObjectRequest', (_message.Message,), dict(
  DESCRIPTOR = _SYNCHRONIZEFEDERATEDOBJECTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.SynchronizeFederatedObjectRequest)
  ))
_sym_db.RegisterMessage(SynchronizeFederatedObjectRequest)

GetStubsRequest = _reflection.GeneratedProtocolMessageType('GetStubsRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETSTUBSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetStubsRequest)
  ))
_sym_db.RegisterMessage(GetStubsRequest)

GetStubsResponse = _reflection.GeneratedProtocolMessageType('GetStubsResponse', (_message.Message,), dict(

  StubsEntry = _reflection.GeneratedProtocolMessageType('StubsEntry', (_message.Message,), dict(
    DESCRIPTOR = _GETSTUBSRESPONSE_STUBSENTRY,
    __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
    # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetStubsResponse.StubsEntry)
    ))
  ,
  DESCRIPTOR = _GETSTUBSRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetStubsResponse)
  ))
_sym_db.RegisterMessage(GetStubsResponse)
_sym_db.RegisterMessage(GetStubsResponse.StubsEntry)

GetBabelStubsRequest = _reflection.GeneratedProtocolMessageType('GetBabelStubsRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETBABELSTUBSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetBabelStubsRequest)
  ))
_sym_db.RegisterMessage(GetBabelStubsRequest)

GetBabelStubsResponse = _reflection.GeneratedProtocolMessageType('GetBabelStubsResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETBABELSTUBSRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetBabelStubsResponse)
  ))
_sym_db.RegisterMessage(GetBabelStubsResponse)

GetClassNameForDSRequest = _reflection.GeneratedProtocolMessageType('GetClassNameForDSRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETCLASSNAMEFORDSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetClassNameForDSRequest)
  ))
_sym_db.RegisterMessage(GetClassNameForDSRequest)

GetClassNameForDSResponse = _reflection.GeneratedProtocolMessageType('GetClassNameForDSResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETCLASSNAMEFORDSRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetClassNameForDSResponse)
  ))
_sym_db.RegisterMessage(GetClassNameForDSResponse)

GetClassNameAndNamespaceForDSRequest = _reflection.GeneratedProtocolMessageType('GetClassNameAndNamespaceForDSRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETCLASSNAMEANDNAMESPACEFORDSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetClassNameAndNamespaceForDSRequest)
  ))
_sym_db.RegisterMessage(GetClassNameAndNamespaceForDSRequest)

GetClassNameAndNamespaceForDSResponse = _reflection.GeneratedProtocolMessageType('GetClassNameAndNamespaceForDSResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETCLASSNAMEANDNAMESPACEFORDSRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetClassNameAndNamespaceForDSResponse)
  ))
_sym_db.RegisterMessage(GetClassNameAndNamespaceForDSResponse)

RegisterECARequest = _reflection.GeneratedProtocolMessageType('RegisterECARequest', (_message.Message,), dict(
  DESCRIPTOR = _REGISTERECAREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.RegisterECARequest)
  ))
_sym_db.RegisterMessage(RegisterECARequest)

AdviseEventRequest = _reflection.GeneratedProtocolMessageType('AdviseEventRequest', (_message.Message,), dict(
  DESCRIPTOR = _ADVISEEVENTREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.AdviseEventRequest)
  ))
_sym_db.RegisterMessage(AdviseEventRequest)

GetContractIDOfDataClayProviderRequest = _reflection.GeneratedProtocolMessageType('GetContractIDOfDataClayProviderRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETCONTRACTIDOFDATACLAYPROVIDERREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetContractIDOfDataClayProviderRequest)
  ))
_sym_db.RegisterMessage(GetContractIDOfDataClayProviderRequest)

GetContractIDOfDataClayProviderResponse = _reflection.GeneratedProtocolMessageType('GetContractIDOfDataClayProviderResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETCONTRACTIDOFDATACLAYPROVIDERRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.GetContractIDOfDataClayProviderResponse)
  ))
_sym_db.RegisterMessage(GetContractIDOfDataClayProviderResponse)

ActivateTracingRequest = _reflection.GeneratedProtocolMessageType('ActivateTracingRequest', (_message.Message,), dict(
  DESCRIPTOR = _ACTIVATETRACINGREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.ActivateTracingRequest)
  ))
_sym_db.RegisterMessage(ActivateTracingRequest)

CloseSessionRequest = _reflection.GeneratedProtocolMessageType('CloseSessionRequest', (_message.Message,), dict(
  DESCRIPTOR = _CLOSESESSIONREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.CloseSessionRequest)
  ))
_sym_db.RegisterMessage(CloseSessionRequest)

UnregisterObjectsRequest = _reflection.GeneratedProtocolMessageType('UnregisterObjectsRequest', (_message.Message,), dict(
  DESCRIPTOR = _UNREGISTEROBJECTSREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.UnregisterObjectsRequest)
  ))
_sym_db.RegisterMessage(UnregisterObjectsRequest)

IsPrefetchingEnabledResponse = _reflection.GeneratedProtocolMessageType('IsPrefetchingEnabledResponse', (_message.Message,), dict(
  DESCRIPTOR = _ISPREFETCHINGENABLEDRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.IsPrefetchingEnabledResponse)
  ))
_sym_db.RegisterMessage(IsPrefetchingEnabledResponse)

ObjectExistsInDataClayRequest = _reflection.GeneratedProtocolMessageType('ObjectExistsInDataClayRequest', (_message.Message,), dict(
  DESCRIPTOR = _OBJECTEXISTSINDATACLAYREQUEST,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.ObjectExistsInDataClayRequest)
  ))
_sym_db.RegisterMessage(ObjectExistsInDataClayRequest)

ObjectExistsInDataClayResponse = _reflection.GeneratedProtocolMessageType('ObjectExistsInDataClayResponse', (_message.Message,), dict(
  DESCRIPTOR = _OBJECTEXISTSINDATACLAYRESPONSE,
  __module__ = 'dataclay.communication.grpc.messages.logicmodule.logicmodule_messages_pb2'
  # @@protoc_insertion_point(class_scope:dataclay.communication.grpc.logicmodule.ObjectExistsInDataClayResponse)
  ))
_sym_db.RegisterMessage(ObjectExistsInDataClayResponse)


DESCRIPTOR._options = None
_NEWCLASSREQUEST_NEWCLASSESENTRY._options = None
_NEWCLASSRESPONSE_NEWCLASSESENTRY._options = None
_NEWCLASSIDREQUEST_NEWCLASSESENTRY._options = None
_GETINFOOFCLASSESINNAMESPACERESPONSE_CLASSESINFOENTRY._options = None
_GETIMPORTEDCLASSESINFOINNAMESPACERESPONSE_IMPORTEDCLASSESENTRY._options = None
_GETCONTRACTIDSOFAPPLICANTRESPONSE_CONTRACTSENTRY._options = None
_GETCONTRACTIDSOFPROVIDERRESPONSE_CONTRACTSENTRY._options = None
_GETCONTRACTSOFAPPLICANTWITHPROVRESPONSE_CONTRACTSENTRY._options = None
_GETDATACONTRACTIDSOFAPPLICANTRESPONSE_DATACONTRACTSENTRY._options = None
_GETDATACONTRACTIDSOFPROVIDERRESPONSE_DATACONTRACTSENTRY._options = None
_NOTIFYFEDERATEDOBJECTSREQUEST_OBJECTSINFOENTRY._options = None
_NOTIFYFEDERATEDOBJECTSREQUEST_FEDERATEDOBJECTSENTRY._options = None
_GETEXECUTIONENVIRONMENTSINFORESPONSE_EXECENVSENTRY._options = None
_GETOBJECTSMETADATAINFOOFCLASSFORNMRESPONSE_MDATAINFOENTRY._options = None
_GETSTUBSRESPONSE_STUBSENTRY._options = None
# @@protoc_insertion_point(module_scope)
